/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeAnimation.h
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AEANIMATION_H_
#define AEANIMATION_H_
#include "astd.h"
#include "aetypes.h"
#include "aelayout.h"
#include "aearch.h"
#include "asig.h"

#include <stdio.h>

/*--------------------
 * Create Settings
 *--------------------*/
typedef struct
{
    aSignature_t    signature;
    aeAnimSubtype_t subtype;
    int             ledsPerEntry;   /* Note 'led' is same as 'pixel' */
    int             bytesPerEntry;  /* For data storage */
    int             allocateCount;  /* Number of entries to allocate initially */
    int             bytesPerFrame;  /* For animated frame */
    int             bitsPerPixel;   /* For animated frame */
    char            isMatrixArch;   /* Set to indicate row/column layout and data */
    int             layoutWidth;    /* Set to non-zero if isMatrixArch is true and subtype is strip */
    int             layoutHeight;   /* '' */
} aeAnimationDataSettings_t;

/*--------------------
 * Create & Delete APIs
 *--------------------*/
void                aeAnimation_InitDataSettings(aeAnimationDataSettings_t* settings);
aeAnimationData_t*  aeAnimation_Create(aeAnimationDataSettings_t* settings);
aeAnimationData_t*  aeAnimation_Copy(aeAnimationData_t* animData);
void                aeAnimation_Delete(aeAnimationData_t* animData);
void                aeAnimation_SetEditSubtype(aeAnimationData_t* animData, aeAnimSubtype_t editSubtype);

/*--------------------
 * Basic Info APIs
 *--------------------*/
const char*         aeAnimation_GetName(aeAnimationData_t* animData);
char*               aeAnimation_GetNameUnique(aeAnimationData_t* animData, char* nameStr, int nameStrLen); /* Returns a modified version of the name */
void                aeAnimation_SetName(aeAnimationData_t* animData, char* name);
char*               aeAnimation_GetNameVariant(aeAnimationData_t* animData, char* name, int namelen);
char*               aeAnimation_GetNameUniqueVariant(aeAnimationData_t* animData, char* name, int namelen);
void                aeAnimation_SetUniqueSuffix(aeAnimationData_t* animData, char* uniqueSuffixStr);
void                aeAnimation_ClearUniqueSuffix(aeAnimationData_t* animData);
aeAnimSubtype_t     aeAnimation_GetSubtype(aeAnimationData_t* animData);
int                 aeAnimation_GetSignature(aeAnimationData_t* animData);

/*--------------------
 * Animated Frame APIs
 *--------------------*/
void                aeAnimation_Reset(aeAnimationData_t* animData);
void                aeAnimation_Advance(aeAnimationData_t* animData);
void                aeAnimation_BuildFrame(aeAnimationData_t* animData);
void 				aeAnimation_RotateFrame(aeAnimationData_t* animData);
char                aeAnimation_IsAtInitialAnimationState(aeAnimationData_t* animData);
void                aeAnimation_HideAnimatedFrame(aeAnimationData_t* animData, char hideFrame);
char                aeAnimation_GetHideAnimatedFrame(aeAnimationData_t* animData);

/*--------------------
 * Flag & Status APIs
 *--------------------*/
char                aeAnimation_GetChangedFlag(aeAnimationData_t* animData);
void                aeAnimation_SetChangedFlag(aeAnimationData_t* animData);
void                aeAnimation_ResetChangedFlag(aeAnimationData_t* animData);
void                aeAnimation_SetFlag(aeAnimationData_t* animData, int flag);
void                aeAnimation_ClearFlag(aeAnimationData_t* animData, int flag);
int                 aeAnimation_GetFlag(aeAnimationData_t* animData);
void                aeAnimation_SetNeedsResizeOnFrameCountChange(aeAnimationData_t* animData, char flag);

/*--------------------
 * Frame Data APIs
 *--------------------*/
void                aeAnimation_AppendFrame(aeAnimationData_t* animData);
void                aeAnimation_DuplicateFrame(aeAnimationData_t* animData);
void                aeAnimation_DeleteCurrFrame(aeAnimationData_t* animData);
int                 aeAnimation_GetFrameCount(aeAnimationData_t* animData);
int                 aeAnimation_GetFrameData(aeAnimationData_t* animData, int frameIndex);
void                aeAnimation_GetFrameDataBuffer(aeAnimationData_t* animData, int frameIndex, unsigned char* pBuffer, int bufferLen);

/*--------------------
 * Pixel Data APIs
 *--------------------*/
void                aeAnimation_ClearCurrFrame(aeAnimationData_t* animData);
void                aeAnimation_InvertCurrFrame(aeAnimationData_t* animData);
void                aeAnimation_UpdateCurrFrameIndex(aeAnimationData_t* animData, aePosition_t pos);
aeLedValue_t        aeAnimation_GetLedValue(aeAnimationData_t* animData, int frameIndex, int ledIndex);
void                aeAnimation_SetLedValue(aeAnimationData_t* animData, int frameIndex, int ledIndex, aeLedValue_t value);
int                 aeAnimation_GetEditFrameIndex(aeAnimationData_t* animData);
int                 aeAnimation_GetAnimFrameIndex(aeAnimationData_t* animData);
aeLedValue_t        aeAnimation_Edit_GetLedValue(aeAnimationData_t* animData, int index);
void                aeAnimation_Edit_UpdateLedValue(aeAnimationData_t* animData, int index, aeLedValue_t value);

aeLedValue_t		aeAnimation_Anim_GetLedValue(aeAnimationData_t* animData, int ledIndex);

/*--------------------
 * Settings APIs
 *--------------------*/
void                aeAnimation_UpdateVariantIndex(aeAnimationData_t* animData, aePosition_t position);
void                aeAnimation_ChangeVariantCount(aeAnimationData_t* animData, aeChange_t change);
int                 aeAnimation_GetVariantCount(aeAnimationData_t* animData);
int                 aeAnimation_GetCurrentVariant(aeAnimationData_t* animData);
void                aeAnimation_SetCurrentVariant(aeAnimationData_t* animData, int variant);
/* Speed */
void                aeAnimation_ChangeSpeed(aeAnimationData_t* animData, aeChange_t change);
int                 aeAnimation_GetSpeed(aeAnimationData_t* animData);
int                 aeAnimation_GetSpeedAt(aeAnimationData_t* animData, int index);
/* Repeat */
void                aeAnimation_ChangeRepeat(aeAnimationData_t* animData, aeChange_t change);
int                 aeAnimation_GetRepeat(aeAnimationData_t* animData);
int                 aeAnimation_GetRepeatAt(aeAnimationData_t* animData, int index);
/* Direction */
void                aeAnimation_ToggleDirection(aeAnimationData_t* animData);
void                aeAnimation_SetDirection(aeAnimationData_t* animData, aeAnimDirection_t dir);
aeAnimDirection_t   aeAnimation_GetDirection(aeAnimationData_t* animData);
aeAnimDirection_t   aeAnimation_GetDirectionAt(aeAnimationData_t* animData, int index);
/* Style */
void                aeAnimation_ToggleWrapStyle(aeAnimationData_t* animData);
void                aeAnimation_SetWrapStyle(aeAnimationData_t* animData, aeAnimStyle_t style);
aeAnimStyle_t       aeAnimation_GetWrapStyle(aeAnimationData_t* animData);
aeAnimStyle_t       aeAnimation_GetWrapStyleAt(aeAnimationData_t* animData, int index);
/* Run Length */
void                aeAnimation_ChangeRunLength(aeAnimationData_t* animData, aeChange_t change);
int                 aeAnimation_GetRunLength(aeAnimationData_t* animData);
int                 aeAnimation_GetRunLengthAt(aeAnimationData_t* animData, int index);

/*--------------------
 * Owner or Storage APIs
 *--------------------*/
void                aeAnimation_SetSequence(aeAnimationData_t* animData, aeSequence_t* sequence);
aeSequence_t*       aeAnimation_GetSequence(aeAnimationData_t* animData);
void                aeAnimation_ClearSequence(aeAnimationData_t* animData);
char                aeAnimation_HasFilePath(aeAnimationData_t* animData);
void                aeAnimation_ClearFilePath(aeAnimationData_t* animData);
void			    aeAnimation_GetFilePath(aeAnimationData_t* animData, char* path, int pathlen);
void                aeAnimation_SetFilePath(aeAnimationData_t* animData, char* path);

/*--------------------
 * File IO APIs
 *--------------------*/
void                aeAnimation_IO_SetState(aeAnimationData_t* animData, AnimIOState_t iostate);
AnimIOState_t       aeAnimation_IO_GetState(aeAnimationData_t* animData);
void                aeAnimation_IO_Count_Reset(aeAnimationData_t* animData);
void                aeAnimation_IO_Count_Incr(aeAnimationData_t* animData);
int                 aeAnimation_IO_Count_Get(aeAnimationData_t* animData);
void                aeAnimation_IO_WriteLine(aeAnimationData_t* animData, char* line, int lineLen);
void                aeAnimation_IO_ReadLine(aeAnimationData_t* animData, char* line);
aErr_t              aeAnimation_Save(aeAnimationData_t* animData, char* path);
aErr_t 				aeAnimation_Write(aeAnimationData_t* animData, FILE* fileref, char* prefix, char* suffix);
aErr_t              aeAnimation_Read(aeAnimationData_t* animData, FILE* fileref, char* prefix, char* suffix);



#endif


