/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aearchlist.h
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AEARCHLIST_H_
#define AEARCHLIST_H_
#include "astd.h"
#include "aetypes.h"
#include "aelayout.h"
#include "aearch.h"
#include "aecodegen.h"

/*

    Platform List

    Platform Signature..identifies the platform (such as Medallion {round SLAK}, Matrix {bicolor Crossbar} etc )
    Architecture Signature...identify the underlying architecture (such as FiveSquare, SLAK, Crossbar)
    Layout Creation Funcs...create (and manage) the edit and animation layouts
    Name
    Description
*/


/*
    Architecture List

    Architecture signature
    Subtype Flags...lists which all subtypes supported by the architecture
    Name
    Description
*/

/*
    CodeGen List 
    CodeGenFuncs...list of functions to write code
    Name
    Description
*/




void            aeArchList_Arch_Add(aeArchitectureInfo_t* archInfo);
int             aeArchList_Arch_GetCount(void);
aSignature_t    aeArchList_Arch_GetSignature(int index);
char*           aeArchList_Arch_GetName(int index, char* name, int namelen);
void            aeArchList_Arch_GetBySignature(aSignature_t archSignature, aeArchitectureInfo_t* archInfo);


void            aeArchList_Layout_Add(aePlatform_t* platform);
int             aeArchList_Layout_GetCount(void);
char*           aeArchList_Layout_GetName(int index, char* name, int namelen);
aSignature_t    aeArchList_Layout_GetSignature(int index);
aSignature_t    aeArchList_Layout_GetArchSignatureFromIndex(int index);
aSignature_t    aeArchList_Layout_GetArchSignatureFromLayoutSignature(aSignature_t layoutSignature);
int             aeArchList_Layout_GetCountByArch(aSignature_t archSignature);
void            aeArchList_Layout_GetConfig(aSignature_t layoutSignature, int* layoutWidth, int* layoutHeight);
void            aeArchList_Layout_GetBySignature(aSignature_t layoutSignature, aePlatform_t* platform);
aeAnimSubtypeFlag_t aeArchList_Layout_GetSubtypeFlags(int index);

void            aeArchList_CodeGen_Add(aeCodeGen_t* codegen);
int             aeArchList_CodeGen_GetCount(void);
void            aeArchList_CodeGen_GetBySignature(aSignature_t codegenSignature, aeCodeGen_t* codegen);
aSignature_t    aeArchList_CodeGen_GetSignature(int index);
aSignature_t    aeArchList_CodeGen_GetArchSignature(int index);
int             aeArchList_CodeGen_GetCountByArch(aSignature_t archSignature);
aSignature_t    aeArchList_CodeGen_GetCodeGenSignatureByArchIndex(aSignature_t archSignature, int index);


#endif /* AEARCHLIST_H_ */

