/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aecodegen.h
 *
 *	This file describes the API for the generation of source code or assembly
 *	for the animations and sequences.
 *
 *	Each codegen module is specific to a particular architecture
 *	e.g. "FiveSquare Pic Assembly" or "POV8 Arduino C"
 *
 *------------------------------------------------------------------------------*/

#ifndef AECODEGEN_H_
#define AECODEGEN_H_
#include "astd.h"
#include "aesequence.h"

/*---------------------------------------------------------
 * 
 * per-architecure codegen ()
 *---------------------------------------------------------*/
/*
	architecture signature
	file extension
	description
	prefix
	suffix
	funcptr
*/

typedef aErr_t (*aeCodeGenFunc_t)(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref);

#define AECODEGEN_EXT_LEN 10
#define AECODEGEN_PRE_LEN 10
#define AECODEGEN_SUF_LEN 10
#define AECODEGEN_NAME_LEN 64
#define AECODEGEN_DESC_LEN 128

struct aeCodeGen_tag
{
    aSignature_t    codegenSignature;
	aSignature_t	archSignature;
	char			extension[AECODEGEN_EXT_LEN];
    char			prefix[AECODEGEN_PRE_LEN];
    char			suffix[AECODEGEN_SUF_LEN];
    char            name[AECODEGEN_NAME_LEN];
    char            desc[AECODEGEN_DESC_LEN];
	aeCodeGenFunc_t	codegenfunc;
    char            requiresUniqueNames;
    int             speedReduction; /* Set by platform */
};




#if 0 /* Not Used; replaced with codegenSignature */
/*-------------------
Animation Source Code Type
    Used for reading or writing data
-------------------*/
typedef enum
{
    aeAnimSourceCodeType_eNone = 0,         /* Used to write only data */

    aeAnimSourceCodeType_ePicAssemblyV1,    /* Pic Assembly options */
    aeAnimSourceCodeType_ePicAssemblyV2,
    aeAnimSourceCodeType_ePicAssemblyV3,

    aeAnimSourceCodeType_eArduinoV1,        /* Arduino options */
    aeAnimSourceCodeType_eArduinoV2,


    aeAnimSourceCodeType_Count
} aeAnimSourceCodeType_t;
#endif 


#endif /* AECODEGEN_H_ */
