/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aelayout.h
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AELAYOUT_H_
#define AELAYOUT_H_
#include "astd.h"

#define AELAYOUT_STANDARD_BOUNDS 1000

typedef enum
{
    aeElementType_eCustom = 0,
    aeElementType_e3mmRound,
    aeElementType_e5mmRound,
    aeElementType_e8mmRound,
    aeElementType_e10mmRound,
    aeElementType_e20mmRound,

    aeElementType_Count
} aeElementType_t;

typedef enum
{
    aeElementShape_eUnknown = 0,
	aeElementShape_eCustom,
	aeElementShape_eCircle,
	aeElementShape_eRectangle,

	aeElementShape_Count
} aeElementShape_t;

/* Matrix Origin */
/* For row/column layouts, can be used to specify where the first */
/* element is */
typedef enum
{
    aeLayoutMatrixOrigin_eTopLeft = 0,
    aeLayoutMatrixOrigin_eTopRight,
    aeLayoutMatrixOrigin_eBottomLeft,
    aeLayoutMatrixOrigin_eBottomRight
} aeLayoutMatrixOrigin_t;

/* Matrx Order */
/* For row/column layouts, can be used to specify if the second */
/* item is in the same row as the first or in the same column */
typedef enum
{
    aeLayoutMatrixOrder_eIncrAlongRow = 0,
    aeLayoutMatrixOrder_eIncrAlongCol,
} aeLayoutMatrixOrder_t;


#define AELAYOUT_MAX_LEDS_PER_ELEMENT 3


typedef unsigned long aeElementColor_t;
typedef unsigned long aeElementLedID_t;


typedef struct
{
    int                 ledCount;
    aeElementLedID_t    ids[AELAYOUT_MAX_LEDS_PER_ELEMENT];
    aeElementColor_t    colors[AELAYOUT_MAX_LEDS_PER_ELEMENT];
    aeElementType_t     elementType;
    unsigned long       locX;
    unsigned long       locY;
    int                 angle;
    aeRect_t            bounds;
} aeLayoutElement_t;


typedef struct aeLayout_tag aeLayout_t;


aeLayout_t*         aeLayout_Create(int elementCount);
void                aeLayout_Destroy(aeLayout_t* layout);
aErr_t              aeLayout_SetElementCount(aeLayout_t* layout, int newElementCount);
aeLayoutElement_t*  aeLayout_GetElement(aeLayout_t* layout, int index);

int                 aeLayout_GetElementCount(aeLayout_t* layout);
void                aeLayout_ScaleElement(aeLayoutElement_t* element, aeRect_t* displayExtent);
void                aeLayout_CalcBounds(aeLayout_t* layout, aeRect_t* displayExtent);
void                aeLayout_GetElementSize(aeElementType_t elementType, int* width, int* height);
aeElementShape_t    aeLayout_GetElementShape(aeElementType_t elementType);

aeRect_t            aeLayout_GetLayoutExtent(aeLayout_t* layout);
void                aeLayout_SetLayoutExtent(aeLayout_t* layout, aeRect_t extent);

/* Optional */
void                aeLayout_SetLayoutDimensions(aeLayout_t* layout, int rowCount, int colCount);
int                 aeLayout_GetLayoutDimensions_Rows(aeLayout_t* layout);
int                 aeLayout_GetLayoutDimensions_Cols(aeLayout_t* layout);
void                aeLayout_SetLayoutElementShape(aeLayout_t* layout, aeElementShape_t elementShape);
aeElementShape_t    aeLayout_GetLayoutElementShape(aeLayout_t* layout);



#endif
