/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeSequence.h
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AESEQUENCE_H_
#define AESEQUENCE_H_
#include "astd.h"
#include "aetypes.h"
#include "aelayout.h"
#include "aearch.h"
#include "asig.h"

#include <stdio.h>




/*--------------------
 * Create & Delete APIs
 *--------------------*/
aeSequence_t*   aeSequence_Create(aSignature_t layoutSignature);
void            aeSequence_Destroy(aeSequence_t* sequence);
void            aeSequence_AssignUntitledName(aeSequence_t* sequence);

/*--------------------
 * Basic Info APIs
 *--------------------*/
const char*     aeSequence_GetName(aeSequence_t* sequence);
void            aeSequence_SetName(aeSequence_t* sequence, char* name);
int             aeSequence_GetCount(aeSequence_t* sequence);
char            aeSequence_GetChangedFlag(aeSequence_t* sequence);
aSignature_t	aeSequence_GetSignature(aeSequence_t* sequence);

/*--------------------
 * Add/Remove/Update Animation APIs
 *--------------------*/
aErr_t          aeSequence_AttachAnimation(aeSequence_t* sequence, aeAnimationData_t* openedAnimData);
aErr_t          aeSequence_DetachAnimation(aeSequence_t* sequence, aeAnimationData_t* openedRef);
aErr_t          aeSequence_UpdateAnimation(aeSequence_t* sequence, aeAnimationData_t* origData);
void            aeSequence_ClearOpenedDataRef(aeSequence_t* sequence, aeAnimationData_t* openedRef);
void            aeSequence_SetAllMatchingOpenedDataRef(aeSequence_t* sequence, int setIndex);
void            aeSequence_DeleteAnimation(aeSequence_t* sequence, int animIndex);

/*--------------------
 * File IO APIs
 *--------------------*/
void            aeSequence_GetFilePath(aeSequence_t* sequence, char* path, int pathptr);
void            aeSequence_SetFilePath(aeSequence_t* sequence, char* path);
aErr_t          aeSequence_Write(aeSequence_t* sequence, FILE* fileref, char* prefix, char* suffix);
aErr_t          aeSequence_WriteCode(aeSequence_t* sequence, FILE* fileref);
aErr_t          aeSequence_Read(aeSequence_t* sequence, FILE* fileref, char* prefix, char* suffix);
aErr_t          aeSequence_Save(aeSequence_t* sequence, char* filepath);

/*--------------------
 * Select Animation APIs
 *--------------------*/
int             aeSequence_Selected_GetIndex(aeSequence_t* sequence);
void            aeSequence_Selected_SetIndex(aeSequence_t* sequence, int animIndex);
void            aeSequence_Selected_UpdateIndex(aeSequence_t* sequence, aePosition_t pos);
void            aeSequence_Selected_Move(aeSequence_t* sequence, aePosition_t pos);

/*--------------------
 * Active Set APIs
 *--------------------*/
void            aeSequence_ActiveSet_Select(aeSequence_t* sequence, aePosition_t position);
void            aeSequence_ActiveSet_DeleteCurr(aeSequence_t* sequence);
void            aeSequence_ActiveSet_Append(aeSequence_t* sequence);
int             aeSequence_ActiveSet_GetCount(aeSequence_t* sequence);
int             aeSequence_ActiveSet_GetIndex(aeSequence_t* sequence);
int             aeSequence_ActiveSet_GetCountItems(aeSequence_t* sequence);

/*--------------------
 * Animation APIs
 *--------------------*/
char                aeSequence_Anim_IsOpened(aeSequence_t* sequence, int animIndex);
char                aeSequence_Anim_IsChanged(aeSequence_t* sequence, int animIndex);
char                aeSequence_Anim_IsActive(aeSequence_t* sequence, int animIndex);
char                aeSequence_Anim_HasNeededAnimData(aeSequence_t* sequence, int animIndex);
void                aeSequence_Anim_SetActive(aeSequence_t* sequence, int animIndex, char active);
char                aeSequence_Anim_HasOrigData(aeSequence_t* sequence, int animIndex);
char                aeSequence_Anim_IsFirstActiveOccurance(aeSequence_t* sequence, int animIndex);
const char*         aeSequence_Anim_GetName(aeSequence_t* sequence, int animIndex);
int                 aeSequence_Anim_GetVariant(aeSequence_t* sequence, int animIndex);
int                 aeSequence_Anim_GetVariantCount(aeSequence_t* sequence, int animIndex);
void                aeSequence_Anim_VariantSelect(aeSequence_t* sequence, int animIndex, aePosition_t position);
aeAnimationData_t*  aeSequence_Anim_Get(aeSequence_t* sequence, int animIndex);
aeAnimationData_t*  aeSequence_Anim_GetOpenedDataRef(aeSequence_t* sequence, int animIndex);
aeAnimationData_t*  aeSequence_Anim_GetDataCopyForEdit(aeSequence_t* sequence, int animIndex);
void                aeSequence_Anim_OpenedForEdit(aeSequence_t* sequence, int animIndex, aeAnimationData_t* refData);

/*--------------------
 * Animated Display APIs
 *--------------------*/
void                aeSequence_Animated_Reset(aeSequence_t* sequence);
int                 aeSequence_Animated_GetIndex(aeSequence_t* sequence);
void                aeSequence_Animated_Advance(aeSequence_t* sequence);
void                aeSequence_Animated_Select(aeSequence_t* sequence, aePosition_t position, char allowLoop);
int                 aeSequence_Animated_GetSpeed(aeSequence_t* sequence);
aeAnimationData_t*  aeSequence_Animated_GetData(aeSequence_t* sequence);
char                aeSequence_Animated_AnimIsAtInitialAnimationState(aeSequence_t* sequence);
char                aeSequence_Animated_SeqIsAtInitialAnimationState(aeSequence_t* sequence);
#endif



