/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aetypes.h
 *
 *	Animator Standard Include
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AETYPES_H_
#define AETYPES_H_


/* Forward declaration of structures */
typedef struct aeAnimationData_tag aeAnimationData_t;
typedef struct aePlatform_tag      aePlatform_t;
typedef struct aeSequence_tag      aeSequence_t;
typedef struct aeCodeGen_tag       aeCodeGen_t;

typedef struct aeArchitectureInfo_tag aeArchitectureInfo_t;

/*
typedef struct aeArchitecture_tag  aeArchitecture_t;
*/

/*-------------------
Animation Subtype
    
-------------------*/
typedef enum aeAnimSubtype_t
{
    aeAnimSubtype_eFrameByFrame  = 0x4652414D, /* FRAM */
    aeAnimSubtype_ePixelList     = 0x5049584C, /* PIXL */
    aeAnimSubtype_eTile          = 0x41494A45, /* TILE */
    aeAnimSubtype_eStrip         = 0x53545250, /* STRP */
    aeAnimSubtype_eCommand       = 0x434D4E44, /* CMND */
    aeAnimSubtype_eFunction      = 0x46554E43  /* FUNC */
} aeAnimSubtype_t;

/*-------------------
Animation Subtype Flags
    
-------------------*/
typedef enum aeAnimSubtypeFlag_t
{
    aeAnimSubtypeFlag_eFrameByFrame  = (1 << 0),
    aeAnimSubtypeFlag_ePixelList     = (1 << 1),
    aeAnimSubtypeFlag_eTile          = (1 << 2),
    aeAnimSubtypeFlag_eStrip         = (1 << 3),
    aeAnimSubtypeFlag_eCommand       = (1 << 4),
    aeAnimSubtypeFlag_eFunction      = (1 << 5)
} aeAnimSubtypeFlag_t;


/*-------------------
Animation Subtype Flag Bits
    
-------------------*/
typedef enum aeAnimSubtypeFlagBit_t
{
    aeAnimSubtypeFlagBit_eFrameByFrame  = 0,
    aeAnimSubtypeFlagBit_ePixelList     = 1,
    aeAnimSubtypeFlagBit_eTile          = 2,
    aeAnimSubtypeFlagBit_eStrip         = 3,
    aeAnimSubtypeFlagBit_eCommand       = 4,
    aeAnimSubtypeFlagBit_eFunction      = 5,

} aeAnimSubtypeFlagBit_t;

/*-------------------
Animation Subtype Count
    
-------------------*/
#define AEANIMSUBTYPE_COUNT 6

/*-------------------
Flags
    
-------------------*/
#define AEANIMFLAG_NeedsLayoutResize    0x01


/*-------------------
Animation Styles
    Single-pass or Wrapped.
-------------------*/
typedef enum
{
    aeAnimStyle_eSinglePass = 0,
    aeAnimStyle_eWrapped
} aeAnimStyle_t;

/*-------------------
Animation Direction
    Used for strip animations
    Horizontal or vertical.
-------------------*/
typedef enum
{
    aeAnimDirection_eHorizontal = 0,
    aeAnimDirection_eVertical
} aeAnimDirection_t;

/*-------------------
Animation Direction
    Used for strip animations
    Horizontal or vertical.
-------------------*/
typedef enum
{
    aeAnimTransition_eImmediate = 0,
    aeAnimTransition_eHistory
} aeAnimTransition_t;



/*-------------------
Animation IO State
    Used for reading or writing data
-------------------*/
typedef enum
{
    AnimIOState_eSig = 0,               /* Deprecated -- now handled by caller */

    /* Data IO States */
    AnimIOState_eName,                  /* Name of the animation */
    AnimIOState_eDetails,               /* Number of frames and number of settings variants */
    AnimIOState_eVariants,              /* Settings info, one line for variant */
    AnimIOState_eFrames,                /* Frame data */


    AnimIOState_eAssembly,              /* Deprecated -- data now written to top of file -- For input, to read past */

    /* Code Generation States  */
    AnimIOState_eCodeFileHeader,        /* Header for the entire file; only used once */
    AnimIOState_eCodeDispatchHeader,    /* Header for the dispatch routine; only used once */
    AnimIOState_eCodeDispatchEntry,     /* "Goto" or "Call" line for the animation; called for each animation */
    AnimIOState_eCodeDispatchTrailer,   /* Trailer for the dispatch routine; only used once */
    AnimIOState_eCodeRoutineIntro,      /* Routine intro; called once */
    AnimIOState_eCodeRoutineHeader,     /* Routine header; called for each animation */
    AnimIOState_eCodeRoutineBody,       /* Routine body; called for each animation */
    AnimIOState_eCodeTableIntro,        /* Table intro; called once */
    AnimIOState_eCodeRoutineTrailer,    /* Routine trailer; called for each animation */
    AnimIOState_eCodeFileTrailer,       /* Trailer for entire file; only used once */

    AnimIOState_eDone,

    AnimIOState_Count
} AnimIOState_t;


#endif


