/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeanimationpane.c
 *
 *	
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aeanimationpane.h"
#include "aearch.h"
#include "aeplatform.h"
#include "aedebug.h"

#include <windows.h>

#define AEEDITORWND_ANIMATION_WIDTH_MIN  200
#define AEEDITORWND_ANIMATION_HEIGHT_MIN 100


/*======================================================================

    Animation Pane
    
======================================================================*/
void aeAnimationPane_Draw(aePane_t* pane, void* data, void* context);


void aeAnimationPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc  = aeAnimationPane_Draw;

	pane->minHeight = AEEDITORWND_ANIMATION_HEIGHT_MIN;
	pane->minWidth  = AEEDITORWND_ANIMATION_WIDTH_MIN;

}

void aeAnimationPane_Draw(aePane_t* pane, void* data, void* context)
{
	HDC         hdc = (HDC)context;
	int         index;
	//aeRect_t    r;
	aeLayout_t*	layout = NULL;
	HBRUSH		origBrush;
	HBRUSH 		currBrush;
	HBRUSH		tmpBrush;
	HPEN		origPen;
	aeColor_t	elementColor;
	COLORREF	currColor;
	COLORREF	nextColor;

	aeElementShape_t 	shape; 
	aeLayoutElement_t*	element;
	aeAnimationInfo_t*	ai = (aeAnimationInfo_t*)data;

    AEASSERT(pane);
    AEASSERT(ai);
    AEASSERT(ai->animData);

	if (ai != NULL)
		layout = ai->animLayout;

    AEASSERT(layout);

	if (layout != NULL)
	{
        element = aeLayout_GetElement(layout, 0);

        currColor = RGB(0xff, 0, 0);

		currBrush  = CreateSolidBrush(currColor);
		origBrush = SelectObject(hdc, currBrush);

		origPen = SelectObject(hdc, aeWinPlat_GetWhitePen());
		for (index = 0; index < aeLayout_GetElementCount(layout); index++)
		{
			element = aeLayout_GetElement(layout, index);

            /* Does the layout have a shape (which overrides the element shape?) */
            shape = aeLayout_GetLayoutElementShape(layout);

            /* If the layout shape is not specified, get the shape from the element */
            if (shape == aeElementShape_eUnknown)
                shape = aeLayout_GetElementShape(element->elementType);
			aeLayout_ScaleElement(element, &pane->loc);

			elementColor = aeArch_AnimData_GetElementColor(ai, index);
			nextColor = aeWinPlat_ConvertColor(elementColor);

			if (nextColor != currColor)
			{
				currColor = nextColor;
				currBrush  = CreateSolidBrush(currColor);
				tmpBrush = SelectObject(hdc, currBrush);
				DeleteObject(tmpBrush);
			}

			if (shape == aeElementShape_eCircle)
			{
				Ellipse(hdc, element->bounds.left, element->bounds.top, element->bounds.right, element->bounds.bottom);
			}
			else
			{
				Rectangle(hdc, element->bounds.left, element->bounds.top, element->bounds.right, element->bounds.bottom);
			}
		}

		SelectObject(hdc, origBrush);
		SelectObject(hdc, origPen);

		DeleteObject(currBrush);
	}
}


