/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeeditorwnd.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include "aemainwnd.h"
#include "aeeditorwnd.h"
#include "aesequencewnd.h"
#include "aewndlist.h"
#include "astd.h"
#include "asig.h"
#include "aepane.h"
#include "aeplatform.h"
#include "aedebug.h"
#include "aecommands.h"
#include "aesequence.h"
#include "aeanimation.h"
#include "aedialogs.h"
#include "aeanimationpane.h"
#include "aetoolbarpane.h"
#include "aegraphics.h"
#ifdef ANIM_HAS_GDLIB
#include "aegfxfile.h"
#endif

char szAEEditorWndClassName[] = "AEEditorWnd";


#define AEEDITORWND_INITIAL_WIDTH	320
#define	AEEDITORWND_INITIAL_HEIGHT	280

//#define AEEDITORWND_ANIMINFO_WIDTH  240
//#define AEEDITORWND_ANIMINFO_HEIGHT  80
//#define AEEDITORWND_EDIT_WIDTH      150

#define AEEDITORWND_EDIT_WIDTH_MIN  150
#define AEEDITORWND_EDIT_HEIGHT_MIN 150

#define AEEDITORWND_ANIMINFO_WIDTH_MIN  170
#define AEEDITORWND_ANIMINFO_HEIGHT_MIN 140

#define AEEDITORWND_EDITINFO_WIDTH_MIN  200
#define AEEDITORWND_EDITINFO_HEIGHT_MIN 100


enum
{
	AEEditorWnd_PaneID_eToolbar = 0,
	AEEditorWnd_PaneID_eAnimInfo,
	AEEditorWnd_PaneID_eAnimation,
	AEEditorWnd_PaneID_eEdit,
	AEEditorWnd_PaneID_eEditInfo,

	AEEditorWnd_PaneID_Count,
};



static aeToolbarButton_t sToolbarList[] = 
{
    { aeCmd_eFile_Save,     "S", 1 },
    { aeCmd_eFile_SaveAs,   "W", 1 },
    { aeCmd_eFile_ExportGif,"G", 1 },
    { aeCmd_eAnim_Name,     "M", 1 },

    { aeCmd_eAnim_Frame_Append,     "A", 1 },
    { aeCmd_eAnim_Frame_Duplicate,  "D", 1 },
    { aeCmd_eAnim_Frame_Delete,     "X", 1 },
    { aeCmd_eAnim_Frame_Clear,      "C", 1 },
    { aeCmd_eAnim_Frame_Invert,     "I", 1 },
    { aeCmd_eAnim_Speed_Increase,   "]", 1 },
    { aeCmd_eAnim_Speed_Decrease,   "[", 1 },
    { aeCmd_eAnim_Repeat_Increase,  "}", 1 },
    { aeCmd_eAnim_Repeat_Decrease,  "{", 1 },
    { aeCmd_eAnim_FrameIndex_First, "<<", 0 },
    { aeCmd_eAnim_FrameIndex_Prev,  "<", 1 },
    { aeCmd_eAnim_FrameIndex_Next,  ">", 1 },
    { aeCmd_eAnim_FrameIndex_Last,  ">>", 0 },
    { aeCmd_eAnim_Strip_Rotate,     "R", 1 },
    { aeCmd_eAnim_WrapStyle_Toggle, "T", 1 },
    { aeCmd_eAnim_RunLength_Decrease,  "(", 1 },
    { aeCmd_eAnim_RunLength_Increase,  ")", 1 },
    { aeCmd_eAnim_Variant_Add,        "p+", 0 },
    { aeCmd_eAnim_Variant_Remove,     "p-", 0 },
    { aeCmd_eAnim_Variant_Prev,     "p<", 0 },
    { aeCmd_eAnim_Variant_Next,     "p>", 0 },

    { aeCmd_eSequence_AttachAnim,       "s+", 0 },
    { aeCmd_eSequence_DetachAnim,       "s-", 0 }

};

static aeToolbarPaneData_t sToolbarData;

//#define AEEDITORWND_ANIMATION_WIDTH

#define ANIM_SPEED_FACTOR 1

typedef struct
{
    aeAnimationInfo_t*  data;
    aePaneList_t        panes;
    int                 hideAnimationCountdown;
    char                mouseDown;
} aeEditorWnd_Data_t;



LRESULT CALLBACK    aeEditorWnd_WndProc(HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam);
aeEditorWnd_Data_t* aeEditorWnd_CreateAndConfigData(aeAnimationInfo_t* animInfo);

void aeEditorWnd_InstallPanes(aeEditorWnd_Data_t* editInfo, HWND hWnd);
void aeEditorWnd_GetMinimumSize(aeEditorWnd_Data_t* editInfo, int* minWidth, int* minHeight);
void aeEditorWnd_PositionPanes(aeEditorWnd_Data_t* editInfo, int windowWidth, int windowHeight);
void aeEditorWnd_HandleResize(aeEditorWnd_Data_t* editInfo, int windowWidth, int windowHeight);

void aeEditorWnd_Destroy(aeEditorWnd_Data_t* data, HWND hwnd);
void aeEditorWnd_DestroyData(aeEditorWnd_Data_t* editInfo);
void aeEditorWnd_TimerEvt(aeEditorWnd_Data_t* editInfo, HWND hwnd);
void aeEditorWnd_CharEvt(aeEditorWnd_Data_t* editInfo, HWND hWnd, WPARAM wParam, BOOL charEvt);
void aeEditorWnd_Draw(aeEditorWnd_Data_t* editInfo, HWND hWnd, HDC hdc);
//void aeEditorWnd_MouseEvt(aeEditorWnd_Data_t* editInfo, HWND hWnd, aeMouseEvt_t mouseEvt, UINT modifiers, int xPos, int yPos);

aeuiWndResponse_t aeEditorWnd_HandleSave(aeAnimationData_t* animData, HWND hwnd, BOOL doSaveAs);
aeuiWndResponse_t aeEditorWnd_DoFileSave(aeAnimationData_t* animData, HWND hwnd, BOOL doSaveAs);

void aeEditorWnd_UpdateTimer(aeAnimationInfo_t* animInfo, HWND hWnd);
void aeEditorWnd_KillTimer(HWND hWnd);
static void aeEditorWnd_DoCommand(void* privateData, void* wndcontext, aeCommand_t command);
char* aeEditorWnd_p_MakeTitle(aeAnimationData_t* animData, char* titlestr, int titlestrlen);


BOOL ConfirmDeleteFrame(HWND hwnd);
BOOL ConfirmClearFrame(HWND hwnd);

void aeEditorWnd_ClearAnimationFromSequence(aeAnimationData_t* animData);
void aeEditorWnd_AttachAnimToSequence(aeAnimationInfo_t* animInfo);
void aeEditorWnd_DetachAnimation(aeAnimationInfo_t* animInfo);
void aeEditorWnd_RenameAnimation(aeAnimationData_t* animData, HWND hWnd);

void aeEditPane_ConfigFunc(aePane_t* pane);
void aeAnimationPane_ConfigFunc(aePane_t* pane);
void aeEditInfoPane_ConfigFunc(aePane_t* pane);
void aeAnimInfoPane_ConfigFunc(aePane_t* pane);
void aeToolbarPane_ConfigFunc(aePane_t* pane);

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static RECT aeRect_MakeWinRect(aeRect_t* ae)
{
	RECT wr;

	wr.top    = ae->top;
	wr.bottom = ae->bottom;
	wr.left   = ae->left;
	wr.right  = ae->right;

	return wr;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeEditorWnd_Init(void)
{
    WNDCLASS    wc;
    aErr_t      err = AERR_NO_ERROR;

    wc.lpszClassName 	= szAEEditorWndClassName;
    wc.hInstance 		= aeWinPlat_GetInstance();
    wc.lpfnWndProc		= aeEditorWnd_WndProc;
    wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
    wc.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
    wc.lpszMenuName		= NULL;
    wc.hbrBackground	= GetStockObject(WHITE_BRUSH);
    wc.style			= 0;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;

    if (!RegisterClass(&wc))
        err = AERR_OS_ERROR;

    return err;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeEditorWnd_Create(aeAnimationInfo_t* animInfo)
{
    aErr_t  err = AERR_NO_ERROR;
	HWND    hWnd;
    int     extW;
	int		extH;
	int		minWidth;
	int		minHeight;
    int     maxWidth;
    int     maxHeight;
    char    titlestr[50];

    aeRect_t            lr;
    aeEditorWnd_Data_t* ewData;

    ewData = aeEditorWnd_CreateAndConfigData(animInfo);
    if (ewData == NULL)
        err = AERR_OUT_OF_MEMORY;

    if (err == AERR_NO_ERROR)
    {
        extH = 30;
        extW = 10;

		/* Move the layout sizes to the min pane sizes */
        lr = aeLayout_GetLayoutExtent(ewData->data->animLayout);
		ewData->panes.pane[AEEditorWnd_PaneID_eAnimation].minWidth  = lr.right * 1;
		ewData->panes.pane[AEEditorWnd_PaneID_eAnimation].minHeight = lr.bottom * 1;

        lr = aeLayout_GetLayoutExtent(ewData->data->editLayout);
		ewData->panes.pane[AEEditorWnd_PaneID_eEdit].minWidth  = lr.right * 2;
		ewData->panes.pane[AEEditorWnd_PaneID_eEdit].minHeight = lr.bottom * 2;

		/* Ask the layout what size it min */
		aeEditorWnd_GetMinimumSize(ewData, &minWidth, &minHeight);

        /* Get the maximum size allowed */
        aeuiWnd_Window_GetMaxSize(&maxWidth, &maxHeight);

        if (minWidth > maxWidth)
            minWidth = maxWidth;

        if (minHeight > maxHeight)
            minHeight = maxHeight;

		/* Position the panes according to the size */
		aeEditorWnd_PositionPanes(ewData, minWidth, minHeight);
		aeLayout_CalcBounds(ewData->data->animLayout, &ewData->panes.pane[AEEditorWnd_PaneID_eAnimation].loc);
		aeLayout_CalcBounds(ewData->data->editLayout, &ewData->panes.pane[AEEditorWnd_PaneID_eEdit].loc);

        hWnd = CreateWindow( 	szAEEditorWndClassName, 
                                aeEditorWnd_p_MakeTitle(animInfo->animData, titlestr, sizeof(titlestr)), 
                                WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, 
                                minWidth + extW, minHeight + extH,
                                (HWND)NULL, (HMENU)NULL, (HANDLE)aeWinPlat_GetInstance(), (LPSTR)NULL);

        if (hWnd == NULL)
        {
            err = AERR_OS_ERROR;

            aeEditorWnd_DestroyData(ewData);
            ewData = NULL;
        }
        else
        {
			{ int i;
			for (i = 0; i < ewData->panes.count; i++)
				ewData->panes.pane[i].windowContext = hWnd;
			}

            aeui_WndList_Add(hWnd, ewData, ewData->data->animData, aeuiWndType_eAnimation);
            
            aeEditorWnd_UpdateTimer(ewData->data, hWnd);
            
            ShowWindow(hWnd, SW_SHOWDEFAULT);
            UpdateWindow( hWnd );
        }
    }

    return err;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_Destroy(aeEditorWnd_Data_t* editWndData,  HWND hWnd)
{
	aeAnimationInfo_t*  ai;
    aeAnimationData_t*  animData;

    AEASSERT(editWndData && editWndData->data);
    AEASSERT(editWndData->data->animData);

    /* Get a temporary local reference to the animation data */
    animData = editWndData->data->animData;

    /* If there is a sequence, then clean-up references */
    if (aeAnimation_GetSequence(animData) != NULL)
    {
        aeSequence_ClearOpenedDataRef(aeAnimation_GetSequence(animData), animData);

        aeSequenceWnd_InvalidateWindow(aeAnimation_GetSequence(animData));

        aeAnimation_ClearSequence(animData);
    }
    /* Clear the local animation data reference */
    animData = NULL;

    /* Stop the timer */
    aeEditorWnd_KillTimer(hWnd);

    /* Get the architecture info from window data and reset the reference in the window data */
	ai = editWndData->data;
    editWndData->data = NULL;

    /* Take the entry out of the list before free data */
    aeui_WndList_Delete(hWnd);

	/* Free the data associated with the window */
    aeEditorWnd_DestroyData(editWndData);
    editWndData = NULL;

	/* Give the architecture info back to the architecture to clean-up */
	aeArch_Destroy(ai);
	ai = NULL;
}


#define AEMOD_CTRL  1
#define AEMOD_SHIFT 2

static int aeMakeModifierFlags(int mods)
{
    int aeflags = 0;

    if ((mods & 0x8) != 0)
        aeflags |= AEMOD_CTRL;

    if ((mods & 0x4) != 0)
        aeflags |= AEMOD_SHIFT;

    return aeflags;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
LRESULT CALLBACK aeEditorWnd_WndProc( HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam )
{
	HDC         hdc;
	PAINTSTRUCT pstruct;
    int         modifiers;
    int         xPos;
    int         yPos;
    int         windowWidth, windowHeight;
    int         dataGone = 0;
    char        continueClose;

    aeEditorWnd_Data_t* data;

    data = aeui_WndList_GetDataA(hWnd);
	
	switch(messg)
	{
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &pstruct);
			
			aeEditorWnd_Draw(data, hWnd, hdc);
			
			EndPaint(hWnd, &pstruct);
            break;
        
        case WM_CHAR: 
            aeEditorWnd_CharEvt(data, hWnd, wParam, TRUE /* char */);
            break;

	    case WM_KEYDOWN: 
            aeEditorWnd_CharEvt(data, hWnd, wParam,  FALSE /* keydown */);
	        break;
	        
        case WM_TIMER:
            aeEditorWnd_TimerEvt(data, hWnd);
	 		break;
	 		
        case WM_LBUTTONDOWN:
            data->mouseDown = TRUE;
            xPos = LOWORD(lParam); 
            yPos = HIWORD(lParam);

            modifiers = aeMakeModifierFlags(wParam);
            aePane_HandleMouseEvt(&data->panes, data->data, hWnd, aeMouseEvt_eDown, modifiers, xPos, yPos);
            break;
            
        case WM_LBUTTONUP:
            if (data->mouseDown)
            {
                xPos = LOWORD(lParam); 
                yPos = HIWORD(lParam);
    
                modifiers = aeMakeModifierFlags(wParam);
                aePane_HandleMouseEvt(&data->panes, data->data, hWnd, aeMouseEvt_eUp, modifiers, xPos, yPos);
            }
            data->mouseDown = FALSE;
            break;

        case WM_MOUSEMOVE:
            if (data->mouseDown)
            {
                xPos = LOWORD(lParam); 
                yPos = HIWORD(lParam);
    
                modifiers = aeMakeModifierFlags(wParam);
                aePane_HandleMouseEvt(&data->panes, data->data, hWnd, aeMouseEvt_eMove, modifiers, xPos, yPos);
            }
       		break;

        case WM_CLOSE:
            continueClose = TRUE;

            if (data != NULL)
                continueClose = aeEditorWnd_RequestClose(data->data->animData);

            /* Pass to standard handling to continue with close */
            if (continueClose)
                return DefWindowProc(hWnd, messg, wParam, lParam);
            break;

        case WM_SIZE:
            windowWidth  = LOWORD(lParam); 
            windowHeight = HIWORD(lParam);
            aeEditorWnd_HandleResize(data, windowWidth, windowHeight);
            InvalidateRect(hWnd, NULL, TRUE);
		break;

        case WM_DESTROY:
            aeEditorWnd_Destroy(data, hWnd);
            dataGone = 1;
			break;
	
		default:
            return DefWindowProc(hWnd, messg, wParam, lParam);
	}

	/* Do I need to do anything else */
	if (!dataGone && (aeAnimation_GetFlag(data->data->animData) & AEANIMFLAG_NeedsLayoutResize))
	{
		data->data->platform.layoutCreateEdit(data->data->editLayout, data->data->animData);
        aeAnimation_ClearFlag(data->data->animData, AEANIMFLAG_NeedsLayoutResize);

		aeLayout_CalcBounds(data->data->editLayout, &data->panes.pane[AEEditorWnd_PaneID_eEdit].loc);
		/* Flag to indicate that next redraw should do an erase */
		data->panes.pane[AEEditorWnd_PaneID_eEdit].valueA = 1;

	}

	return 0;
}

/*----------------------------------------------------------------------
Summary
    Request Close

Desc
    This is called to request to close this window. This routine can 
    return FALSE to stop the close request.
----------------------------------------------------------------------*/
char aeEditorWnd_RequestClose(aeAnimationData_t* animData)
{
    char    continueClose = TRUE;
    HWND    hWnd;

    aeuiWndResponse_t response;

    AEASSERT(animData);
    hWnd = aeui_WndList_GetWndFromDataB(animData);
    AEASSERT(hWnd);

    if (aeAnimation_GetChangedFlag(animData))
    {
        response = aeuiWnd_DataChangedConfirmClose(aeAnimation_GetName(animData));

        if (response == aeuiWndResponse_eCancel)
            continueClose = FALSE;
        else if (response == aeuiWndResponse_eYes)
        {
           response = aeEditorWnd_HandleSave(animData, hWnd, FALSE);
           if (response == aeuiWndResponse_eCancel)
               continueClose = FALSE;
        }
        else
            ; /* Don't want to save and don't want to cancel */
    }

    return continueClose;
}

/*----------------------------------------------------------------------
Summary
    Clears the sequence from the animation

Desc
    This is a one-way clear. It is called from the sequence window.
----------------------------------------------------------------------*/
void aeEditorWnd_ClearSequence(aeAnimationData_t* animData)
{
    HWND  hWnd;

    AEASSERT(animData);

    hWnd = aeui_WndList_GetWndFromDataB(animData);
    AEASSERT(hWnd);

    InvalidateRect(hWnd, NULL, FALSE);

    aeAnimation_ClearSequence(animData);


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_HandleResize(aeEditorWnd_Data_t* editInfo, int windowWidth, int windowHeight)
{
    aeEditorWnd_PositionPanes(editInfo, windowWidth, windowHeight);

}
/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_BringForward(aeAnimationData_t* animData)
{
    void* wnd;

    wnd = aeui_WndList_GetWndFromDataB(animData);

    if (wnd)
        aeuiWnd_BringForward(wnd);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeEditorWnd_Data_t* aeEditorWnd_CreateAndConfigData(aeAnimationInfo_t* animInfo)
{
    aeEditorWnd_Data_t* editInfo;

    editInfo = (aeEditorWnd_Data_t*)malloc(sizeof(aeEditorWnd_Data_t));

    if (editInfo != NULL)
    {
        memset(editInfo, 0, sizeof(aeEditorWnd_Data_t));

        aePane_InitPaneList(&editInfo->panes);

        editInfo->data = animInfo;

        editInfo->hideAnimationCountdown = 0;

        aeEditorWnd_InstallPanes(editInfo, NULL);

    }

    return editInfo;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_InstallPanes(aeEditorWnd_Data_t* editInfo, HWND hWnd)
{
    aePane_t*   pane;

    /* Toolbar */
    sToolbarData.count = sizeof(sToolbarList)/sizeof(aeToolbarButton_t);
    sToolbarData.list  = sToolbarList;
    sToolbarData.handlerProc = aeEditorWnd_DoCommand;

    pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eToolbar];
    aeToolbarPane_InitData(pane, &sToolbarData);
    aeToolbarPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Animation Info */
    pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimInfo];
    aeAnimInfoPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Animation */
    pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimation];
    aeAnimationPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Edit */
    pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEdit];
    aeEditPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Edit Info */
    pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEditInfo];
    aeEditInfoPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    editInfo->panes.count = AEEditorWnd_PaneID_Count;

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_DestroyData(aeEditorWnd_Data_t* editInfo)
{
    AEASSERT(editInfo);

    free(editInfo);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_UpdateTimer(aeAnimationInfo_t* animInfo, HWND hWnd)
{
	int millisec;
	int speed;

    speed = aeAnimation_GetSpeed(animInfo->animData);

	if (speed == 0)
		speed = 0xc0;

	millisec = speed * ANIM_SPEED_FACTOR;
	
	if (millisec < 10)
		millisec = 10;
	
	SetTimer(hWnd, 1, millisec, NULL);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_KillTimer(HWND hWnd)
{
    KillTimer(hWnd, 0);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_TimerEvt(aeEditorWnd_Data_t* editInfo, HWND hWnd)
{
    if (editInfo->hideAnimationCountdown == 0)
    {
        aeAnimation_Advance(editInfo->data->animData);
        InvalidateRect(hWnd, NULL, FALSE);

        if (aeAnimation_IsAtInitialAnimationState(editInfo->data->animData))
        {
            aeAnimation_HideAnimatedFrame(editInfo->data->animData, TRUE);
            editInfo->hideAnimationCountdown = 5;
        }

    }
    else
    {
        editInfo->hideAnimationCountdown--;
        if (editInfo->hideAnimationCountdown <= 0)
        {
            editInfo->hideAnimationCountdown = 0;
            aeAnimation_HideAnimatedFrame(editInfo->data->animData, FALSE);
            InvalidateRect(hWnd, NULL, FALSE);
        }
    }
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_CharEvt(aeEditorWnd_Data_t* editInfo, HWND hWnd, WPARAM wParam, BOOL charEvt)
{
    unsigned char   ch;
    aeCommand_t     cmd = aeCmd_eNull;
    int             i;

    if (charEvt)
    {
        ch = wParam;

        if (ch >= 'a' && ch <= 'z')
            ch = ch + 'A' - 'a';

        for (i = 0; i < sizeof(sToolbarList)/sizeof(*sToolbarList) && cmd == aeCmd_eNull; i++)
        {
            /* Must be one char long string and allowed as a keypress */
            if (sToolbarList[i].charAllowed && sToolbarList[i].desc[1] == 0 && sToolbarList[i].desc[0] == ch)
                cmd = sToolbarList[i].cmd;
        }

        if (cmd == aeCmd_eNull)
        {
            switch (ch)
            { 
                case 'O':
                case 'o':
                    cmd = aeCmd_eFile_Open;
                    break;
    
                case 'n':
                case 'N':
                    cmd = aeCmd_eFile_New;
                    break;

                #if 0
                case '?':
                    aeAnimation_Dbg_Log(editInfo->data->animData);
                    break;
                #endif
  
            } 
        }
    }
    else /* keydown */
    {
        switch (wParam)
        { 
            case VK_LEFT: 
                cmd = aeCmd_eAnim_FrameIndex_Prev;
                break; 
            case VK_RIGHT: 
                cmd = aeCmd_eAnim_FrameIndex_Next;
                break; 
            case VK_UP: 
                cmd = aeCmd_eAnim_FrameIndex_First;
                break; 
            case VK_DOWN:
                cmd = aeCmd_eAnim_FrameIndex_Last; 
                break; 
        } 
    }

    if (cmd != aeCmd_eNull)
        aeEditorWnd_DoCommand(editInfo->data, hWnd, cmd);

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_Draw(aeEditorWnd_Data_t* editInfo, HWND hWnd, HDC hdc)
{
    int         paneIndex;

    aePane_t*   pane;
    
    for (paneIndex = 0; paneIndex < editInfo->panes.count; paneIndex++)
    {
        pane = &editInfo->panes.pane[paneIndex];

        /* aegfxFrameRect(hdc, &pane->loc, 0xc0c0c0); */

        if (pane->drawFunc != NULL)
            (*pane->drawFunc)(pane, editInfo->data, hdc);
    }
	

}

#if 0
/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_MouseEvt(aeEditorWnd_Data_t* editInfo, HWND hWnd, aeMouseEvt_t mouseEvt, UINT modifiers, int xPos, int yPos)
{
    int         paneIndex;
    aeRect_t    r;
    aePane_t*   pane;

    for (paneIndex = 0; paneIndex < editInfo->panes.count; paneIndex++)
    {
        pane = &editInfo->panes.pane[paneIndex];
        r = pane->loc;

        if (r.left <= xPos && xPos <= r.right && r.top <= yPos && yPos <= r.bottom)
        {
            /* Track which pane had the mouse down */
            if (mouseEvt == aeMouseEvt_eDown)
                editInfo->hasMouse = pane;

            /* Only the pane with the mouse down gets mouse move; others get mouse over */
            else if (editInfo->hasMouse != pane && mouseEvt == aeMouseEvt_eMove)
                mouseEvt = aeMouseEvt_eOver;

            if (pane->mouseFunc != NULL)
            {
                (pane->mouseFunc)(pane, mouseEvt, modifiers, editInfo->data, hWnd, xPos, yPos);
            }

            /* Only process one */
            break;
        }
    }
}
#endif
/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeuiWndResponse_t aeEditorWnd_HandleSave(aeAnimationData_t* animData, HWND hwnd, BOOL doSaveAs)
{
    aErr_t  err;
    aeuiWndResponse_t response = aeuiWndResponse_eOK;

    if (aeAnimation_GetSequence(animData) != NULL)
    {
        err = aeSequence_UpdateAnimation(aeAnimation_GetSequence(animData), animData);

        if (err == AERR_NO_ERROR)
        {
            aeAnimation_ResetChangedFlag(animData);
            aeSequenceWnd_InvalidateWindow(aeAnimation_GetSequence(animData));
        }
        else
        {
            response = aeuiWndResponse_eCancel;
            aeuiWnd_Error("Error saving to sequence", err);
        }
    }
    else
    {
        response = aeEditorWnd_DoFileSave(animData, hwnd, FALSE);
    }

    return response;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeuiWndResponse_t aeEditorWnd_DoFileSave(aeAnimationData_t* animData, HWND hwnd, BOOL doSaveAs)
{
    aErr_t  err;
    char    path[MAX_ANIM_PATH];
    char	filename[MAX_ANIM_FILE];
    BOOL	doSave;
    char    titlestr[50];
    int     i;

    OPENFILENAME      ofn;
    aeuiWndResponse_t response = aeuiWndResponse_eOK;
    
    /* Get the previous path, if any. If not, this will set path to empty string */
    aeAnimation_GetFilePath(animData, path, MAX_ANIM_PATH);


    if (strlen(path) == 0 || doSaveAs)
    {
        memset(&filename, 0, sizeof(filename));

        /* If there is no path (because this animation has never been saved) */
        /* then use the name of the animation */
        if (strlen(path) == 0)
            strcpy(path, aeAnimation_GetName(animData));
        
        memset(&ofn, 0, sizeof(ofn));
        ofn.lStructSize     = sizeof(ofn);
        ofn.hwndOwner       = hwnd;
        ofn.lpstrFile       = path;
        ofn.nMaxFile        = sizeof(path);
        ofn.lpstrFileTitle  = filename;
        ofn.nMaxFileTitle   = sizeof(filename);
        ofn.lpstrDefExt     = "asm";
    	ofn.Flags           = OFN_OVERWRITEPROMPT;
        
        doSave = GetSaveFileName(&ofn);

        if (doSave)
        {
	        aeAnimation_SetFilePath(animData, path);

            /* Update the name of the animation */
            for (i = 0; i < strlen(filename); i++)
                if (filename[i] == '.')
                    filename[i] = 0;
                    
            aeAnimation_SetName(animData, filename);

            aeuiWnd_ChangeTitle(hwnd, aeEditorWnd_p_MakeTitle(animData, titlestr, sizeof(titlestr)));

        }
        else
        {
            response = aeuiWndResponse_eCancel;
        }
    }
    else
    {
        doSave = TRUE;
    }
    
    if (doSave)
    {
        err = aeAnimation_Save(animData, path);

        if (err != AERR_NO_ERROR)
        {
            aeuiWnd_Error("Error saving to file", err);
            response = aeuiWndResponse_eCancel;
        }
    }

    return response;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_AttachAnimToSequence(aeAnimationInfo_t* animInfo)
{
    aErr_t          err = AERR_NO_ERROR;
    aeSequence_t*   sequence;
    char            doAttach;

    /* Note that we can attach the animation more than once to the same sequence */
    /* This allows the (same variant of the) animation to be played more than once in the sequence */
    doAttach = aeSequenceWnd_FindSequenceForArch(aeAnimation_GetSignature(animInfo->animData), &sequence);

    if (doAttach)
    {
        AEASSERT(sequence);

        /* Attach the copy to the animation */
        err = aeSequence_AttachAnimation(sequence, animInfo->animData);
        
        if (err == AERR_NO_ERROR)
        {
            aeSequenceWnd_InvalidateWindow(sequence);
        
            /* Free any path */
            if (aeAnimation_HasFilePath(animInfo->animData))
                aeAnimation_ClearFilePath(animInfo->animData);

            /* Mark as belonging to a sequence */
            aeAnimation_SetSequence(animInfo->animData, sequence);
        
            /* Clear the changed flag */
            aeAnimation_ResetChangedFlag(animInfo->animData);
        
        }
        else
        {
            aeuiWnd_Error("Error attaching animation to sequence window", err);
        }
    }
    else
    {
        aeuiWnd_Error("No sequence window found to attach animation", AERR_GENERAL_ERROR);
    }
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_DetachAnimation(aeAnimationInfo_t* animInfo)
{
    /* Clear any reference in the animation to a sequence and */
    /* clear the reference in the sequence to the animation */
    /* This is a two way clear */
    aeEditorWnd_ClearAnimationFromSequence(animInfo->animData);

    /* Clear any previous file path */
    aeAnimation_ClearFilePath(animInfo->animData);
}

/*----------------------------------------------------------------------
Summary
    Ask user to provide name for animation

Desc

----------------------------------------------------------------------*/
void aeEditorWnd_RenameAnimation(aeAnimationData_t* animData, HWND hWnd)
{
    aeuiWndResponse_t   response;
    char                newName[ANIM_NAME_LEN];

    strcpy(newName, aeAnimation_GetName(animData));

    response = aeuiWnd_RequestString(aeWinPlat_GetInstance(), hWnd, 
                                     "Animation Name", "Enter a name for the animation", newName, sizeof(newName));

    if (response == aeuiWndResponse_eOK)
    {
        aeAnimation_SetName(animData, newName);

    }

}

/*----------------------------------------------------------------------
Summary
    

Desc
    Clear any reference in the animation to a sequence and
    clear the reference in the sequence to the animation
    This is a two way clear

----------------------------------------------------------------------*/
void aeEditorWnd_ClearAnimationFromSequence(aeAnimationData_t* animData)
{
    aeSequence_t* sequence;

    sequence = aeAnimation_GetSequence(animData);

    if (sequence != NULL)
    {
        /* Remove sequence from animation */
        aeAnimation_ClearSequence(animData);

        /* Detach animation from sequence */
        aeSequence_ClearOpenedDataRef(sequence, animData);
        aeSequenceWnd_InvalidateWindow(sequence);
    }
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_DoCommand(void* privateData, void* wndcontext, aeCommand_t command)
{
    HWND                hWnd = (HWND)wndcontext;
    aeAnimationInfo_t*  animInfo = (aeAnimationInfo_t*)privateData;

    AEASSERT(animInfo);

    switch (command)
    { 
        case aeCmd_eFile_Save:
            aeEditorWnd_HandleSave(animInfo->animData, hWnd, FALSE);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eFile_SaveAs:
            aeEditorWnd_ClearAnimationFromSequence(animInfo->animData);
            aeEditorWnd_DoFileSave(animInfo->animData, hWnd, TRUE);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eFile_ExportGif:
            #ifdef ANIM_HAS_GDLIB
            {
                char path[100];
                strcpy(path, aeAnimation_GetName(animInfo->animData));
                strcat(path, ".gif");
                aeGfxFile_ExportAnimAsGif(animInfo->animData, path);
            }
            #endif
            break;

        case aeCmd_eAnim_Name:
            aeEditorWnd_RenameAnimation(animInfo->animData, hWnd);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Frame_Delete:
            aeAnimation_DeleteCurrFrame(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Frame_Duplicate:
            aeAnimation_DuplicateFrame(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Frame_Append:
            aeAnimation_AppendFrame(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Frame_Clear:
            aeAnimation_ClearCurrFrame(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Frame_Invert:
            aeAnimation_InvertCurrFrame(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Strip_Rotate:
	        aeAnimation_ToggleDirection(animInfo->animData);
            aeAnimation_SetFlag(animInfo->animData, AEANIMFLAG_NeedsLayoutResize);
            aeAnimation_SetChangedFlag(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Speed_Increase:
            aeAnimation_ChangeSpeed(animInfo->animData, aeChange_eIncrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
            {
                InvalidateRect(hWnd, NULL, FALSE);
                aeEditorWnd_UpdateTimer(animInfo, hWnd);
            }		            	
            break;

        case aeCmd_eAnim_Speed_Decrease:
            aeAnimation_ChangeSpeed(animInfo->animData, aeChange_eDecrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
            {
                InvalidateRect(hWnd, NULL, FALSE);
                aeEditorWnd_UpdateTimer(animInfo, hWnd);
            }		            	
            break;
        case aeCmd_eAnim_Repeat_Increase:
        case aeCmd_eAnim_Repeat_Decrease:
            aeAnimation_ChangeRepeat(animInfo->animData, (command == aeCmd_eAnim_Repeat_Increase) ? aeChange_eIncrease : aeChange_eDecrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
                InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_FrameIndex_Prev: 
            aeAnimation_UpdateCurrFrameIndex(animInfo->animData, aePosition_ePrev);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_FrameIndex_Next: 
            aeAnimation_UpdateCurrFrameIndex(animInfo->animData, aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_FrameIndex_First: 
            aeAnimation_UpdateCurrFrameIndex(animInfo->animData, aePosition_eFirst);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eAnim_FrameIndex_Last: 
            aeAnimation_UpdateCurrFrameIndex(animInfo->animData, aePosition_eLast);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eAnim_RunLength_Increase:
	        aeAnimation_ChangeRunLength(animInfo->animData, aeChange_eIncrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
                InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_RunLength_Decrease:
	        aeAnimation_ChangeRunLength(animInfo->animData, aeChange_eDecrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
                InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_WrapStyle_Toggle:
            aeAnimation_ToggleWrapStyle(animInfo->animData);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eAnim_Variant_Add:
        case aeCmd_eAnim_Variant_Remove:
	        aeAnimation_ChangeVariantCount(animInfo->animData, (command == aeCmd_eAnim_Variant_Add) ? aeChange_eIncrease : aeChange_eDecrease);
            if (aeAnimation_GetChangedFlag(animInfo->animData))
            {
                InvalidateRect(hWnd, NULL, FALSE);
                aeEditorWnd_UpdateTimer(animInfo, hWnd);
            }
            break;

        case aeCmd_eAnim_Variant_Prev:
        case aeCmd_eAnim_Variant_Next:
	        aeAnimation_UpdateVariantIndex(animInfo->animData, (command == aeCmd_eAnim_Variant_Prev) ? aePosition_ePrev : aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            aeEditorWnd_UpdateTimer(animInfo, hWnd);
            break;

        case aeCmd_eSequence_AttachAnim:
            aeEditorWnd_AttachAnimToSequence(animInfo);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eSequence_DetachAnim:
            aeEditorWnd_DetachAnimation(animInfo);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eFile_Open:
        case aeCmd_eFile_New:
        default:
            aeMainWnd_DoCommand(hWnd, command);
            break;

    } 


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
BOOL ConfirmDeleteFrame(HWND hwnd)
{
	BOOL doIt = TRUE;
	
	return doIt;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
BOOL ConfirmClearFrame(HWND hwnd)
{
	BOOL doIt = TRUE;
	
	return doIt;
}


/*
  WHITE_BRUSH = 0x80000000,
  LTGRAY_BRUSH = 0x80000001,
  GRAY_BRUSH = 0x80000002,
  DKGRAY_BRUSH = 0x80000003,
  BLACK_BRUSH = 0x80000004,
  NULL_BRUSH = 0x80000005,
  WHITE_PEN = 0x80000006,
  BLACK_PEN = 0x80000007,
  NULL_PEN = 0x80000008,
  OEM_FIXED_FONT = 0x8000000A,
  ANSI_FIXED_FONT = 0x8000000B,
  ANSI_VAR_FONT = 0x8000000C,
  SYSTEM_FONT = 0x8000000D,
  DEVICE_DEFAULT_FONT = 0x8000000E,
  DEFAULT_PALETTE = 0x8000000F,
  SYSTEM_FIXED_FONT = 0x80000010,
  DEFAULT_GUI_FONT = 0x80000011,
  DC_BRUSH = 0x80000012,
  DC_PEN = 0x80000013

*/


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_GetMinimumSize(aeEditorWnd_Data_t* editInfo, int* minWidth, int* minHeight)
{
	aePane_t*	pane;
	aePane_t*	paneL;
	aePane_t*	paneR;
	int			minW;
	int			minH;

	/* Toolbar */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eToolbar];
	minW = pane->minWidth;
	minH = pane->minHeight;

	/* Animation Info */
	paneL = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimInfo];

	paneR = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimation];

	if (paneL->minHeight > paneR->minHeight)
		minH += paneL->minHeight;
	else
		minH += paneR->minHeight;

	if (minW < paneL->minWidth + paneR->minWidth)
		minW = paneL->minWidth + paneR->minWidth;

	paneL = &editInfo->panes.pane[AEEditorWnd_PaneID_eEdit];

	paneR = &editInfo->panes.pane[AEEditorWnd_PaneID_eEditInfo];

	if (paneL->minHeight > paneR->minHeight)
		minH += paneL->minHeight;
	else
		minH += paneR->minHeight;

	if (minW < paneL->minWidth + paneR->minWidth)
		minW = paneL->minWidth + paneR->minWidth;


	*minWidth  = minW;
	*minHeight = minH;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditorWnd_PositionPanes(aeEditorWnd_Data_t* editInfo, int windowWidth, int windowHeight)
{
	aePane_t*	pane;
	aeRect_t	pr;
	int			wmA, wmB;
	int			wA, wB;
	int			hmTL, hmTR, hmBL, hmBR;
	int			hmT, hmB;
    int         editW;

	/* Toolbar */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eToolbar];
	pr.left   = 0;
	pr.top    = 0;
	pr.right  = windowWidth;
	pr.bottom = pane->minHeight;
	pane->loc = pr;

	/* Animation Info */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimInfo];
	wmA = pane->minWidth;
	hmTL = pane->minHeight;

	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimation];
	wmB = pane->minWidth;
	hmTR = pane->minHeight;

	wA = windowWidth - wmB;
	wB = wmB;

	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEdit];
	wmA = pane->minWidth;
	hmBL = pane->minHeight;

	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEditInfo];
	wmB = pane->minWidth;
    editW = pane->minWidth;
	hmBR = pane->minHeight;

	hmT = (hmTL < hmTR) ? hmTR : hmTL;
	hmB = (hmBL < hmBR) ? hmBR : hmBL;


	/* Animation Info */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimInfo];
	pr.top    = pr.bottom;
	pr.bottom = pr.top + hmT;
	pr.left   = 0;
	pr.right  = wA;
	pane->loc = pr;

	/* Animation */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eAnimation];
	pr.left  = pr.right;
	pr.right = windowWidth;
	pane->loc = pr;
    aeLayout_CalcBounds(editInfo->data->animLayout, &pr);

	/* Edit */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEdit];
	pr.top    = pr.bottom;
	pr.bottom = pr.top + hmB;
	pr.left   = 0;
    if (pane->minWidth + editW > windowWidth) /* combined width is too wide */
        pr.right = windowWidth - editW;
    else if (pane->minWidth + editW < windowWidth)
        pr.right = windowWidth - editW;
    else
        pr.right = pane->minWidth;

	//pr.right  = (pane->minWidth < (windowWidth - editW)) ? (windowWidth - editW) : pane->minWidth;
	pane->loc = pr;
    aeLayout_CalcBounds(editInfo->data->editLayout, &pr);

	/* Edit Info */
	pane = &editInfo->panes.pane[AEEditorWnd_PaneID_eEditInfo];
	pr.left   = pr.right;
	pr.right  = windowWidth;
	pane->loc = pr;


}

/*======================================================================

    Edit Pane
    
======================================================================*/
void aeEditPane_Draw(aePane_t* pane, void* data, void* context);
void aeEditPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* data, void* context, int xPos, int yPos);



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc  = aeEditPane_Draw;
    pane->mouseFunc = aeEditPane_MouseEvt;

	pane->minHeight = AEEDITORWND_EDIT_HEIGHT_MIN;
	pane->minWidth  = AEEDITORWND_EDIT_WIDTH_MIN;

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditPane_Draw(aePane_t* pane, void* data, void* context)
{
    HDC         hdc = (HDC)context;
    int         index;
	aeLayout_t*	layout = NULL;
	HBRUSH		origBrush;
	HBRUSH 		currBrush;
	HBRUSH		tmpBrush;
	HPEN		origPen;
	aeColor_t	elementColor;
	COLORREF	currColor;
	COLORREF	nextColor;
	RECT		winR;

	aeElementShape_t	shape;
	aeLayoutElement_t*	element;
	aeAnimationInfo_t*	ai = (aeAnimationInfo_t*)data;


	if (ai != NULL)
		layout = ai->editLayout;



	if (layout != NULL)
	{
		/* valueA flag for erase background */
		if (pane->valueA)
		{
			winR = aeRect_MakeWinRect(&pane->loc);
			FillRect(hdc, &winR, (HBRUSH)GetStockObject(WHITE_BRUSH));
			pane->valueA = 0;
		}
		currColor = RGB(0xff, 0, 0);

		currBrush  = CreateSolidBrush(currColor);
		origBrush = SelectObject(hdc, currBrush);

		origPen = SelectObject(hdc, aeWinPlat_GetWhitePen());
		for (index = 0; index < aeLayout_GetElementCount(layout); index++)
		{
			element = aeLayout_GetElement(layout, index);

            shape = aeLayout_GetLayoutElementShape(layout);
            if (shape == aeElementShape_eUnknown)
                shape = aeLayout_GetElementShape(element->elementType);

			aeLayout_ScaleElement(element, &pane->loc);

			elementColor = aeArch_EditData_GetElementColor(ai, index);
			nextColor = aeWinPlat_ConvertColor(elementColor);

			if (nextColor != currColor)
			{
				currColor = nextColor;
				currBrush  = CreateSolidBrush(currColor);
				tmpBrush = SelectObject(hdc, currBrush);
				DeleteObject(tmpBrush);
			}
				
			
			if (shape == aeElementShape_eCircle)
			{
				Ellipse(hdc, element->bounds.left, element->bounds.top, element->bounds.right, element->bounds.bottom);
			}
			else
			{
				Rectangle(hdc, element->bounds.left, element->bounds.top, element->bounds.right, element->bounds.bottom);
			}
		}

		SelectObject(hdc, origBrush);
		SelectObject(hdc, origPen);

		DeleteObject(currBrush);
	}
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* data, void* context, int xPos, int yPos)
{
	short i;
	#if 0
	aeLedValue_t		currVal;
	aeLedValue_t		newVal;
	#endif
	aeLayout_t*			layout = NULL;
	aeLayoutElement_t*	element;
	aeAnimationInfo_t*	ai = (aeAnimationInfo_t*)data;

	if (ai != NULL)
		layout = ai->editLayout;

    /* Only process mouse down or move events */
	if (layout != NULL && (mouseEvt == aeMouseEvt_eDown || mouseEvt == aeMouseEvt_eMove))
	{
		for (i = 0; i < aeLayout_GetElementCount(layout); i++)
		{
			element = aeLayout_GetElement(layout, i);


			if (aeRect_LocInRect(&element->bounds, xPos, yPos))
			{
                /* If it is a mouse move and the element is the same as the last */
                /* time in here, then ignore it. Otherwise we will continuously */
                /* toggle the element */
                if (mouseEvt == aeMouseEvt_eMove && pane->valueB == i)
                {
                }
                else
                {
                    /* Ctrl-click on pixel list will add frames */
                    if ((modifiers & AEMOD_CTRL) != 0 &&
                        aeAnimation_GetSubtype(ai->animData) == aeAnimSubtype_ePixelList &&
                        mouseEvt == aeMouseEvt_eDown)
                    {
                        aeAnimation_AppendFrame(ai->animData);
                    }
    
                    aeArch_EditData_AdvanceElementColor(ai, i);
    				aeAnimation_SetChangedFlag(ai->animData);
    
    				InvalidateRect((HWND)(pane->windowContext), NULL, FALSE);
    
                    /* Remember last item */
                    pane->valueB = i;
                }

                /* Only allow update of one */
                break;
			}
		}
	}
}


/*======================================================================

    Edit Info Pane
    
======================================================================*/
void aeEditInfoPane_Draw(aePane_t* pane, void* data, void* context);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditInfoPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc  = aeEditInfoPane_Draw;

	pane->minHeight = AEEDITORWND_ANIMINFO_HEIGHT_MIN;
	pane->minWidth  = AEEDITORWND_ANIMINFO_WIDTH_MIN;

}

#define FRAME_DOT_SIZE     10
#define FRAME_DOT_SPACE     5
#define FRAME_DOT_BORDER_X 10
#define FRAME_DOT_ORIG_Y   80
#define FRAME_DOT_CLR_UNSEL 0xc0c0c0
#define FRAME_DOT_CLR_SEL   0x000000

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEditInfoPane_Draw(aePane_t* pane, void* data, void* context)
{
	HDC     hdc = (HDC)context;
    HFONT   oldFont;
    char    str[200];
	int		i;
	int		editIndex;
	int		animIndex;
	int		frameCount;
	int		x, y;
	int		row, col;
	int		dotsPerRow;
	RECT	wr;
	HBRUSH	hbr;
	int		maxFrameCount;
	int		lineHeight = 15;

	aeAnimationInfo_t* ai = (aeAnimationInfo_t*)data;


	/* Get the frame info */
	frameCount = aeAnimation_GetFrameCount(ai->animData);
	animIndex  = aeAnimation_GetAnimFrameIndex(ai->animData);
	editIndex  = aeAnimation_GetEditFrameIndex(ai->animData);

    oldFont = SelectObject(hdc, aeWinPlat_GetNormalFont());

	sprintf(str, "Edit Information");
	TextOut(hdc, pane->loc.left+ 5, pane->loc.top + 5, str, strlen(str) );

	sprintf(str, "Frames: %d     ", frameCount);
    TextOut(hdc, pane->loc.left+ 5, pane->loc.top + 5 + lineHeight, str, strlen(str) );

	sprintf(str, "Current: %d     ", editIndex+1);
	TextOut(hdc, pane->loc.left+ 5, pane->loc.top + 5 + lineHeight*2, str, strlen(str) );

    SelectObject(hdc, oldFont);

	/* Determine how many dots we can put on a row */
	dotsPerRow = ((pane->loc.right - pane->loc.left) - FRAME_DOT_BORDER_X * 2 + FRAME_DOT_SPACE)/(FRAME_DOT_SIZE + FRAME_DOT_SPACE);

	/* If we drew more frames last time than the current frame count (because some were deleted) */
	/* then use the old value to erase the old squares */
	maxFrameCount = frameCount;
	if (maxFrameCount < pane->valueA)
		maxFrameCount = pane->valueA;

	/* Don't draw the frame count for Strip subtype */
	/* Actually, we need to know the current frame for delete, insert, and duplicate */
	//if (ai->data->subtype != aeAnimSubtype_eStrip)
	{
		for (i = 0; i < maxFrameCount; i++)
		{
			col = (i % dotsPerRow);
			row = (i / dotsPerRow);
	
			x = pane->loc.left + FRAME_DOT_BORDER_X + col * (FRAME_DOT_SIZE + FRAME_DOT_SPACE);
			y = pane->loc.top  + FRAME_DOT_ORIG_Y   + row * (FRAME_DOT_SIZE + FRAME_DOT_SPACE);
	
			wr.top = y;
			wr.bottom = y + FRAME_DOT_SIZE;
			wr.left = x;
			wr.right = x + FRAME_DOT_SIZE;
	
			/* All frames and current frame */
			hbr = (HBRUSH)GetStockObject((i != editIndex) ? LTGRAY_BRUSH : BLACK_BRUSH);
			if (i >= frameCount)
				hbr = (HBRUSH)GetStockObject(WHITE_BRUSH);
			
			FillRect(hdc, &wr, hbr);
	
			/* Animation Index */
			/* Note that this only makes sense for frame-by-frame animations */
			if (aeAnimation_GetSubtype(ai->animData) == aeAnimSubtype_eFrameByFrame)
			{
				wr.top = wr.bottom + 2;
				wr.bottom += 4;
				hbr = (HBRUSH)GetStockObject((i == animIndex) ? LTGRAY_BRUSH : WHITE_BRUSH);
			}
	
			FillRect(hdc, &wr, hbr);
		}
	}

	/* Remember the last frame dot drawn, so we can erase cleanly later */
	pane->valueA = frameCount;
}



/*======================================================================

    Animation Info Pane
    
======================================================================*/
void aeAnimInfoPane_Draw(aePane_t* pane, void* data, void* context);

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeAnimInfoPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc  = aeAnimInfoPane_Draw;

	pane->minHeight = AEEDITORWND_ANIMINFO_HEIGHT_MIN;
	pane->minWidth  = AEEDITORWND_ANIMINFO_WIDTH_MIN;

}

enum
{
    aeAnimSettingEdit_eSpeed       = (1 << 1),

    aeAnimSettingEdit_eRepeat      = (1 << 2),
    aeAnimSettingEdit_eRunLength   = (1 << 3),
    aeAnimSettingEdit_eStyle       = (1 << 4),
    aeAnimSettingEdit_eDirection   = (1 << 5),
    aeAnimSettingEdit_eVector      = (1 << 6),
    aeAnimSettingEdit_eTransStyle  = (1 << 7),
    aeAnimSettingEdit_eTransDur    = (1 << 8),
    aeAnimSettingEdit_eReserved0   = (1 << 9),
    aeAnimSettingEdit_eReserved1   = (1 <<10)

};


struct
{
    char*    label;
    int     flag;
}
sLabelList[] =
{
    {"Speed",       aeAnimSettingEdit_eSpeed},
    {"Repeat",      aeAnimSettingEdit_eRepeat},
    {"Run length",  aeAnimSettingEdit_eRunLength},
    {"Style",       aeAnimSettingEdit_eStyle},
    {"Direction",   aeAnimSettingEdit_eDirection},
    {"Vector",      aeAnimSettingEdit_eVector},
    {"Trans Style", aeAnimSettingEdit_eTransStyle},
    {"Trans Dur",   aeAnimSettingEdit_eTransDur},
    {"Res0",        aeAnimSettingEdit_eReserved0}, 
    {"Res1",        aeAnimSettingEdit_eReserved1}
};

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeAnimInfoPane_Draw(aePane_t* pane, void* data, void* context)
{
	HDC     hdc = (HDC)context;
    HFONT   oldFont;
    char   str[200];
    char	name[50];
	int		lineHeight;
	int		xPos;
	int		yPos;
	int		colWidth = 55;
	int		ySettingStart;
	int		i;
	aeAnimationInfo_t*	ai = (aeAnimationInfo_t*)data;
	aeAnimationData_t*	animData;
	aeAnimSubtype_t		subtype;
    aeAnimDirection_t   direction;
    unsigned int        editFlags;

	lineHeight = 15;

	AEASSERT(ai);
	
	animData = ai->animData;

	subtype = aeAnimation_GetSubtype(animData);
	
    editFlags = aeAnimSettingEdit_eSpeed | aeAnimSettingEdit_eRepeat;
    if (subtype == aeAnimSubtype_eFrameByFrame)
        editFlags |= 0;
	else if (subtype == aeAnimSubtype_ePixelList)
        editFlags |= (aeAnimSettingEdit_eRunLength | aeAnimSettingEdit_eStyle);
    else if (subtype == aeAnimSubtype_eStrip)
        editFlags |= (aeAnimSettingEdit_eStyle | aeAnimSettingEdit_eDirection);

    oldFont = SelectObject(hdc, aeWinPlat_GetNormalFont());

	xPos = pane->loc.left + 5;
	yPos = pane->loc.top + 5;

    #if 0
	sprintf(str, "Animation Information");
    TextOut( hdc, xPos, yPos, str, strlen(str) );
	yPos += lineHeight;
    #endif
    
	strcpy(str, "Name:");
	TextOut(hdc, xPos, yPos, str, strlen(str));
	sprintf(str, "%s %c                           ", aeAnimation_GetName(animData), (aeAnimation_GetChangedFlag(animData) ? '*' : ' '));
	TextOut(hdc, xPos + colWidth, yPos, str, strlen(str));
	yPos += lineHeight;

    if (aeAnimation_HasFilePath(animData))
        strcpy(str, "Saves to file            ");
    else if (aeAnimation_GetSequence(animData) != NULL)
        strcpy(str, "Saves to sequence                ");
    else
        strcpy(str, "Never saved                ");
	TextOut(hdc, xPos + colWidth, yPos, str, strlen(str));
	yPos += lineHeight;

    sprintf(str, "%s", aeArch_AnimSubtype_GetStr(subtype, name, sizeof(name)));
    TextOut(hdc, xPos + colWidth, yPos, str, strlen(str));
    yPos += lineHeight;

    ySettingStart = yPos;

    yPos += lineHeight; /* line for "current" marker */

    for (i = 0; i < sizeof(sLabelList)/sizeof(*sLabelList); i++)
    {
        if (editFlags & sLabelList[i].flag)
        {
            TextOut(hdc, xPos, yPos, sLabelList[i].label, strlen(sLabelList[i].label));
            yPos += lineHeight;
        }
    }

    /* Move back to top row and shift over to columns start */
    xPos += colWidth;
    colWidth = 50;

    for (i = 0; i < aeAnimation_GetVariantCount(animData); i++)
    {
        yPos = ySettingStart;

        strcpy(str,  (i == aeAnimation_GetCurrentVariant(animData)) ? "v" : "  ");
        TextOut(hdc,xPos + colWidth * i, yPos, str, strlen(str));
        yPos += lineHeight;

        if (editFlags & aeAnimSettingEdit_eSpeed)
        {
        	sprintf(str, "0x%2.2x    ", aeAnimation_GetSpeedAt(animData, i));
        	TextOut(hdc, xPos + colWidth * i, yPos, str, strlen(str));
        	yPos += lineHeight;
        }

        if (editFlags & aeAnimSettingEdit_eRepeat)
        {
        	sprintf(str, "%d    ",  aeAnimation_GetRepeatAt(animData, i));
        	TextOut(hdc, xPos + colWidth * i, yPos, str, strlen(str));
        	yPos += lineHeight;
        }


       if (editFlags & aeAnimSettingEdit_eRunLength)
       {
            sprintf(str, "%d   ",  aeAnimation_GetRunLengthAt(animData, i));
            TextOut(hdc, xPos + colWidth * i, yPos, str, strlen(str));
            yPos += lineHeight;
       }

       if (editFlags & aeAnimSettingEdit_eStyle)
       {
            sprintf(str, "%s  ", aeArch_AnimStyle_GetStr(aeAnimation_GetWrapStyleAt(animData, i), name, sizeof(name)));
            TextOut(hdc, xPos + colWidth * i, yPos, str, strlen(str));
            yPos += lineHeight;
       }

       if (editFlags & aeAnimSettingEdit_eDirection)
       {
            direction = aeAnimation_GetDirectionAt(animData, i);
            sprintf(str, "%s  ", (direction == aeAnimDirection_eHorizontal) ? "Horz" : "Vert");
            TextOut(hdc, xPos + colWidth * i, yPos, str, strlen(str));
            yPos += lineHeight;
       }
    }

    /* Erase everything past the last column */
    if (aeAnimation_GetVariantCount(animData) < ANIM_MAX_VARIANTS)
    {
        RECT wr;

        wr.top    = ySettingStart;
        wr.left   = xPos + colWidth * aeAnimation_GetVariantCount(animData);
        wr.bottom = wr.top + lineHeight * 6;
        wr.right  = wr.left + colWidth * ANIM_MAX_VARIANTS;


        FillRect(hdc, &wr, (HBRUSH)GetStockObject(WHITE_BRUSH));

    }



    SelectObject(hdc, oldFont);


}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
char* aeEditorWnd_p_MakeTitle(aeAnimationData_t* animData, char* titlestr, int titlestrlen)
{
    AEASSERT(titlestr && titlestrlen);

    sprintf(titlestr, "Animation: %s", aeAnimation_GetName(animData));

    return titlestr;
}


