/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aemainwnd.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include "aemainwnd.h"
#include "aeplatform.h"
#include "aeeditorwnd.h"
#include "aesequencewnd.h"
#include "aecommands.h"
#include "animator.h"
#include "asig.h"
#include "aedebug.h"
#include "aearch.h"
#include "aesequence.h"
#include "aeanimation.h"
#include "aefileio.h"
#include "aearchlist.h"
#include "aepane.h"

#include "aewndlist.h"
#include "aedialogs.h"
#include "aegraphics.h"
#include "aeanimationpane.h"

#define SHOW_LAYOUT

LRESULT CALLBACK aeMainWnd_WndProc(HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam);

/* Commands */
void doFileOpen(void);
void doFileSave(void);
void doFileSaveAs(void);
void doNewEditWnd(aSignature_t layoutSig, aeAnimSubtype_t subtype);
void doNewSequenceWnd(aSignature_t animSignature);
int  doQuitApp(void);

void ReadDataFromFile(FILE* fileref, char* prefix, char* suffix, char* path);
void ReadAnimationFromFile(FILE* fileref, aSignature_t animSig, aeAnimSubtype_t animSubtype, char* prefix, char* suffix, char* path);
void ReadSequenceFromFile(FILE* fileref, aSignature_t animSignature, char* prefix, char* suffix, char* path);

void SetupGridList(void);

char aeMainWnd_RequestCloseAllWindows(void);


static char szMainWnd_ClassName[] = "aeMainWndClassName";
static HWND sMainWnd = NULL;

enum
{
    aeMain_Null = 0,
    aeMain_Cmd,
    aeMain_Anim,
    aeMain_Sequence,
    aeMain_Layout
};

#if 0
    typedef struct
    {
        int                 which;
        char                desc[100];
        aSignature_t        layoutSignature;
        aeCommand_t         cmd;
        aeAnimSubtype_t     subtype;
    
    } MainOption_t;
    
    
    #define MAX_MAIN_ITEMS 20
    int          sOptionCount = 0;
    MainOption_t sOptionList[MAX_MAIN_ITEMS];

#endif 

#define MAX_GRID_ITEMS 20
#define MAX_SUBITEMS   10

#define MAINWND_GRID_HEIGHT_MIN  60
#define MAINWND_GRID_WIDTH      120

#define MAINWND_ANIM_HORZSPACE   5
#define MAINWND_ANIM_VERTSPACE   5
#define MAINWND_ANIM_VERTOFFSET 20
#define MAINWND_ANIM_HORZSIZE   (MAINWND_GRID_WIDTH - 2*MAINWND_ANIM_HORZSPACE)
#define MAINWND_ANIM_VERTSIZE   60

#define MAINWND_MAX_SUBTYPES 3


typedef struct
{
    int                 yOffset;
    int                 which;
    aeAnimSubtype_t     subtype;
} GridSubItem_t;

typedef struct
{
    int                 which;
    aeCommand_t         cmd;
    char                desc[100];
    aSignature_t        layoutSignature;
    int                 subitemCount;
    GridSubItem_t       subitem[MAX_SUBITEMS];

    #ifdef SHOW_LAYOUT
    aeAnimationInfo_t*  animInfo;
    aePane_t            animPane;
    #endif
} GridItem_t;

typedef struct
{
    int     itemCount;
    int     colCount;
    int     rowCount;

    int     colWidth;
    int     rowHeight;
    int     vertMargin;
    int     horzMargin;
    int     vertSpace;
    int     horzSpace;

    GridItem_t  items[MAX_GRID_ITEMS];
} Grid_t;



Grid_t sMainGrid;


int  Grid_CalcWidth(Grid_t* grid);
int  Grid_CalcHeight(Grid_t* grid);
void Grid_CalcRect(Grid_t* grid, int index, int* xLoc, int* yLoc, int* width, int* height);
char Grid_FindHitRect(Grid_t* grid, int* index, int xLoc, int yLoc);


void aeMainWnd_DrawAnimation(HDC hdc, GridItem_t* gridItem, int xLoc, int yLoc, int width, int height);
void aeMainWnd_DrawCmd(HDC hdc, GridItem_t* gridItem, int xLoc, int yLoc, int width, int height);
void aeMainWnd_ClickAnimation(HWND hWnd, GridItem_t* gridItem, int xOffset, int yOffset);

/* Line Height */
int sLineHeight = 14;

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
int WINAPI aeMainWin_Create(void)
{
	HWND hWnd;

    int     width;
    int     height;


    SetupGridList();


    /* Grid Setup */
    sMainGrid.rowHeight  = MAINWND_GRID_HEIGHT_MIN + MAINWND_MAX_SUBTYPES * sLineHeight;
    sMainGrid.colWidth   = MAINWND_GRID_WIDTH;
    sMainGrid.vertSpace  = 5;
    sMainGrid.horzSpace  = 10;
    sMainGrid.vertMargin = 10;
    sMainGrid.horzMargin = 5;

    #ifdef SHOW_LAYOUT
    sMainGrid.rowHeight += MAINWND_ANIM_VERTSIZE;
    #endif

    /* Grid Layout */
    sMainGrid.rowCount = 1;
    sMainGrid.colCount = sMainGrid.itemCount;

    /* Window size */
    width  = Grid_CalcWidth(&sMainGrid) + 10;
    height = Grid_CalcHeight(&sMainGrid) + 20;

	hWnd = CreateWindow( 	szMainWnd_ClassName, "Animator v0.2",
							WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
							width, height, (HWND)NULL, (HMENU)NULL,
							(HANDLE)aeWinPlat_GetInstance(), (LPSTR)NULL);

	ShowWindow(hWnd, SW_SHOWDEFAULT);
	UpdateWindow( hWnd );
	
	/* For Use by File Dialogs */
	sMainWnd = hWnd;


	return 0;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeMainWnd_Init(void)
{
    aErr_t      err = AERR_NO_ERROR;
	WNDCLASS    wc;

    wc.lpszClassName 	= szMainWnd_ClassName;
    wc.hInstance 		= aeWinPlat_GetInstance();
    wc.lpfnWndProc		= aeMainWnd_WndProc;
    wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
    wc.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
    wc.lpszMenuName		= NULL;
    wc.hbrBackground	= GetStockObject( WHITE_BRUSH );
    wc.style			= 0;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;

    if (!RegisterClass(&wc))
        err = AERR_OS_ERROR;

    return err;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
#if 0
static RECT aeMainWnd_CalcItemRect(int item)
{
	RECT	wr;
	int		lineSpace  = 5;

	wr.left   = 3;
	wr.right  = 200;
	wr.top    = 10 + item * (sLineHeight + lineSpace);
	wr.bottom = wr.top + sLineHeight;

	return wr;
}
#endif

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
LRESULT CALLBACK aeMainWnd_WndProc( HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam )
{
	HDC         hdc;
	PAINTSTRUCT pstruct;
    HFONT       oldFont;
	RECT		wr;
	int			xPos, yPos;
	int			editIndex;
    char        continueClose;
    aeRect_t    r;

	switch(messg)
	{
		case WM_PAINT:
		hdc = BeginPaint(hWnd, &pstruct );

        oldFont = SelectObject(hdc, aeWinPlat_GetNormalFont());

        {
            int i;
            int xLoc, yLoc, width, height;

            /* Draw the grid */
            for (i = 0; i < sMainGrid.itemCount; i++)
            {
                Grid_CalcRect(&sMainGrid, i, &xLoc, &yLoc, &width, &height);

                r.left = xLoc;
                r.top  = yLoc;
                r.right = xLoc + width;
                r.bottom = yLoc + height;
                aegfxFrameRect(hdc, &r, 0x808080);

                if (sMainGrid.items[i].which == aeMain_Cmd)
                    aeMainWnd_DrawCmd(hdc, &sMainGrid.items[i], xLoc, yLoc, width, height);
                else if (sMainGrid.items[i].which == aeMain_Layout)
                    aeMainWnd_DrawAnimation(hdc, &sMainGrid.items[i], xLoc, yLoc, width, height);
                else
                    TextOut( hdc, wr.left + 5, wr.top + 2, sMainGrid.items[i].desc, strlen(sMainGrid.items[i].desc) );

            }
        }
        
        SelectObject(hdc, oldFont);
		EndPaint(hWnd, &pstruct );
		break;

        case WM_CHAR:
            aeuiWnd_TestDialog(aeWinPlat_GetInstance(), hWnd);
            break;

        case WM_DESTROY:
            aeMainWnd_DoCommand(hWnd, aeCmd_eQuit);
    		break;

        case WM_CLOSE:
            continueClose = aeMainWnd_RequestCloseAllWindows();

            /* Pass to standard handling to continue with close */
            if (continueClose)
                return DefWindowProc(hWnd, messg, wParam, lParam);
            break;
		
        case WM_LBUTTONDOWN:
            xPos = LOWORD(lParam); 
            yPos = HIWORD(lParam);
            

            if (Grid_FindHitRect(&sMainGrid, &editIndex, xPos, yPos))
            {
                int xLoc, yLoc, width, height;
                if (sMainGrid.items[editIndex].which == aeMain_Cmd)
                    aeMainWnd_DoCommand(hWnd, sMainGrid.items[editIndex].cmd);
                else if (sMainGrid.items[editIndex].which == aeMain_Layout)
                {
                    Grid_CalcRect(&sMainGrid, editIndex, &xLoc, &yLoc, &width, &height);
                    aeMainWnd_ClickAnimation(hWnd, &sMainGrid.items[editIndex], xPos - xLoc, yPos - yLoc);
                }

            }
		break;
		
		default:
		return( DefWindowProc( hWnd, messg, wParam, lParam ) );
	}

	return( 0L );
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeMainWnd_DrawCmd(HDC hdc, GridItem_t* gridItem, int xLoc, int yLoc, int width, int height)
{
    TextOut( hdc, xLoc + 5, yLoc + 2, gridItem->desc, strlen(gridItem->desc) );
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeMainWnd_DrawAnimation(HDC hdc, GridItem_t* gridItem, int xLoc, int yLoc, int width, int height)
{
    int     y;
    int     x;
    char    str[100];
    int     i;

    #ifdef SHOW_LAYOUT
    aePane_t* pane;
    #endif
    
    x = xLoc + 5;
    y = yLoc + 2;

    TextOut( hdc, x, y, gridItem->desc, strlen(gridItem->desc) );

    for (i = 0; i < gridItem->subitemCount; i++)
    {
        if (gridItem->subitem[i].which == aeMain_Sequence)
            strcpy(str, "Sequence...");
        else if (gridItem->subitem[i].subtype == aeAnimSubtype_eFrameByFrame)
            strcpy(str, "Frame-by-frame...");
        else if (gridItem->subitem[i].subtype == aeAnimSubtype_ePixelList)
            strcpy(str, "Pixel List...");
        else if (gridItem->subitem[i].subtype == aeAnimSubtype_eStrip)
            strcpy(str, "Strip...");

        aegfxDrawString(hdc, x, yLoc + gridItem->subitem[i].yOffset, str, 0);
        /*TextOut(hdc, x, yLoc + gridItem->subitem[i].yOffset, str, strlen(str));*/
    }


    #ifdef SHOW_LAYOUT
    pane = &gridItem->animPane;
    pane->loc.top    = yLoc + MAINWND_ANIM_VERTOFFSET;
    pane->loc.left   = xLoc + MAINWND_ANIM_HORZSPACE;
    pane->loc.bottom = pane->loc.top  + MAINWND_ANIM_VERTSIZE;
    pane->loc.right  = pane->loc.left + MAINWND_ANIM_HORZSIZE;

    /* aegfxFrameRect(hdc, &pane->loc, AECOLOR_BLACK); */
    aeLayout_CalcBounds(gridItem->animInfo->animLayout, &pane->loc);

    if (pane->drawFunc != NULL)
        (*pane->drawFunc)(pane, gridItem->animInfo, hdc);
    #endif

}

void aeMainWnd_ClickAnimation(HWND hWnd, GridItem_t* gridItem, int xOffset, int yOffset)
{
    int i;
    int yRef;
    int subitemHit = -1;
    GridSubItem_t*   si;

    for (i = 0; i < gridItem->subitemCount && subitemHit == -1; i++)
    {
        yRef = gridItem->subitem[i].yOffset;
        if (yOffset >= yRef && yOffset < yRef + sLineHeight)
            subitemHit = i;
    }

    if (subitemHit != -1)
    {
        si = &gridItem->subitem[subitemHit];

        if (si->which == aeMain_Sequence)
        {
            doNewSequenceWnd(gridItem->layoutSignature);
        }
        else if (si->which == aeMain_Anim)
        {
            doNewEditWnd(gridItem->layoutSignature, si->subtype);
        }
    }
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
char aeMainWnd_RequestCloseAllWindows(void)
{
    char    continueClose = TRUE;
    int     wndCount;
    int     index;

    aeAnimationData_t*  animData;
    aeSequence_t*       sequence;

    wndCount = aeui_WndList_Type_GetCount(aeuiWndType_eAnimation);

    for (index = 0; (index < wndCount) && continueClose; index++)
    {
        animData = (aeAnimationData_t*)aeui_WndList_Type_GetDataBByTypeIndex(aeuiWndType_eAnimation, index);
        continueClose = aeEditorWnd_RequestClose(animData);
    }

    wndCount = aeui_WndList_Type_GetCount(aeuiWndType_eSequence);
    for (index = 0; index < wndCount && continueClose; index++)
    {
        sequence = (aeSequence_t*)aeui_WndList_Type_GetDataBByTypeIndex(aeuiWndType_eSequence, index);
        continueClose = aeSequenceWnd_RequestClose(sequence);
    }


    return continueClose;
}
/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void doNewEditWnd(aSignature_t layoutSig, aeAnimSubtype_t subtype)
{
	aeAnimationInfo_t* ai;

	ai = aeArch_CreateWithSignatureSubtype(layoutSig, subtype);
	if (ai != NULL)
	{
		
        aeEditorWnd_Create(ai);
        
	}

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void doNewSequenceWnd(aSignature_t animSignature)
{
	aeSequence_t* seq;

	seq = aeSequence_Create(animSignature);

    if (seq != NULL)
    {
        aeSequence_AssignUntitledName(seq);
    	
        aeSequenceWnd_Create(seq);
    }
    else
    {
        aeuiWnd_Error("Unable to create new sequence", AERR_OUT_OF_MEMORY);
    }
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
int  doQuitApp(void)
{
    int continueQuit = 1;
    /* Any un-saved windows? */

    return continueQuit;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static char getFileOpenPath(char *pathstr, int pathlen)
{
    char    path[MAX_ANIM_PATH];
    char	filename[MAX_ANIM_FILE];
    OPENFILENAME    ofn;
    BOOL	doOpen;

    memset(&path, 0, sizeof(path));
    memset(&filename, 0, sizeof(filename));

    memset(&ofn, 0, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = sMainWnd;
    ofn.lpstrFile = path;
    ofn.nMaxFile = sizeof(path);
    ofn.lpstrFileTitle = filename;
    ofn.nMaxFileTitle = sizeof(filename);
    ofn.lpstrDefExt = "asm";
	//ofn.Flags = OFN_HIDEREADYONLY | OFN_CREATEPROMPT;
    
    doOpen = GetOpenFileName(&ofn);

    if (doOpen)
    {
        strncpy(pathstr, path, pathlen);
        pathstr[pathlen - 1] =0;
    }
    return doOpen;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void ReadAnimationFromFile(FILE* fileref, aSignature_t animSig, aeAnimSubtype_t animSubtype, char* prefix, char* suffix, char* path)
{
    aErr_t  err = AERR_NO_ERROR; 
	aeAnimationInfo_t*  animInfo = NULL;

    /* Create Animation Data */
    if (err == AERR_NO_ERROR)
        animInfo = aeArch_CreateWithSignatureSubtype(animSig, animSubtype);

    if (animInfo != NULL)
    {
        err = aeAnimation_Read(animInfo->animData, fileref, prefix, suffix);

        /* Save the path for the next save */
        aeAnimation_SetFilePath(animInfo->animData, path);

        #if 1
		animInfo->platform.layoutCreateEdit(animInfo->editLayout, animInfo->animData);
        #endif
        
        /* Create a window and pass the animInfo */
        aeEditorWnd_Create(animInfo);
    }
    
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void ReadSequenceFromFile(FILE* fileref, aSignature_t animSignature, char* prefix, char* suffix, char* path)
{
	aErr_t			err = AERR_NO_ERROR;
    aeSequence_t*   sequence = NULL;
    int             i;
    int             nameStart;
    char            name[ANIM_NAME_LEN];

    /* Create Sequence Data */
    sequence = aeSequence_Create(animSignature);


    if (sequence != NULL)
    {
        err = aeSequence_Read(sequence, fileref, prefix, suffix);
        
        if (err == AERR_NO_ERROR)
        {
        	aeSequence_SetFilePath(sequence, path);

            nameStart = 0;
            for (i = strlen(path) - 1; i >= 0; i--)
                if (path[i] == PATH_SEPARATOR_CHAR)
                {
                    nameStart = i + 1;
                    break;
                }

            strncpy(name, path + nameStart, sizeof(name));
            for (i = 0; i < strlen(name); i++)
                if (name[i] == '.')
                    name[i] = 0;

            if (strlen(name) > 0)
                aeSequence_SetName(sequence, name);

            aeSequenceWnd_Create(sequence);
        }
        else
        {
            aeSequence_Destroy(sequence);
            sequence = NULL;
            aeuiWnd_Error("Unable to read animation sequence", err);
        }
        
    }
    else
    {
        aeuiWnd_Error("Out of memory when to creating sequence", AERR_OUT_OF_MEMORY);
    }
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void ReadDataFromFile(FILE* fileref, char* prefix, char* suffix, char* path)
{
    char    fileline[250];
    char    line[250];
    aErr_t  err; 
    aSignature_t		signature;
    aeAnimSubtype_t		subtype;
	

    /* Read and Parse first line. Should be ANIM:XXXX:YYYY */
    err = aeIO_ReadLineFromFile(fileref, fileline, sizeof(fileline));
    if (err == AERR_NO_ERROR)
    {
        aeIO_RemovePrefixSuffix(fileline, prefix, suffix, line, sizeof(line));
        err = aeArch_ParseSignatureLine(line, &signature, &subtype);

        if (err == AERR_NO_ERROR)
        {
            if (subtype == ASIG_SEQUENCE)
            {
                ReadSequenceFromFile(fileref, signature, prefix, suffix, path);
            }
            else
            {
                ReadAnimationFromFile(fileref, signature, subtype, prefix, suffix, path);
            }
        }
        else
        {
            aeuiWnd_Error("Unable to parse signature line of file", err);
        }
    }
    else
    {
        aeuiWnd_Error("Unable to read line from file", err);
    }
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void doFileOpen(void)
{
    char    path[MAX_ANIM_PATH];
    char    doOpen;
    FILE*   f;

    doOpen = getFileOpenPath(path, sizeof(path));

    if (doOpen)
    {
        f = fopen(path, "r");

        if (f != NULL)
        {
            ReadDataFromFile(f, "; ", "\n", path);

            fclose(f);
            f = NULL;
        }
        else
        {
            aeuiWnd_Error("Unable to open file", AERR_FILE_NOT_FOUND);
        }
    }
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void doFileSave(void)
{

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void doFileSaveAs(void)
{
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeMainWnd_DoCommand(HWND hwnd, aeCommand_t command)
{

    switch (command)
    { 
        case aeCmd_eQuit:
            if (doQuitApp())
               PostQuitMessage(0);
            break;

        case aeCmd_eFile_Open:
            doFileOpen();
            break;

        case aeCmd_eFile_New:
            break;

        default:
            break;
    }
}


#if 0

{
	{	aeMain_Anim, 	
        "FiveSquare:Frame-by-frame",
        aeFiveSquare_SetupArch,				
        aeFiveSquare_InitData,			
        ASIG_FIVESQUARE,
        aeAnimSubtype_eFrameByFrame		},

	{	aeMain_Anim, 		
        "FiveSquare:Pixel list"	, 	
        aeFiveSquare_SetupArch,				
        aeFiveSquare_InitData,			
        ASIG_FIVESQUARE,
        aeAnimSubtype_ePixelList		},

    {	aeMain_Anim, 			
        "FiveSquare:Strip"	, 	
        aeFiveSquare_SetupArch,				
        aeFiveSquare_InitData,			
        ASIG_FIVESQUARE,
        aeAnimSubtype_eStrip			},

	{	aeMain_Anim, 	
        "CircleTwoColor:Frame-by-frame", 	
        aeSlakCircleTwoColor_SetupArch,		
        aeSlakCircleTwoColor_InitData,	
        ASIG_CIRCLETWO,
        aeAnimSubtype_eFrameByFrame	},

    {	aeMain_Seq, 			
        "FiveSquare Sequence"	, 	
        aeFiveSquare_SetupArch,				
        aeFiveSquare_InitData,		
        ASIG_FIVESQUARE,
        aeAnimSubtype_eStrip			},


    {	aeMain_Seq, 			
        "CircleTwo Sequence", 	
        aeSlakCircleTwoColor_SetupArch,				
        aeSlakCircleTwoColor_InitData,		
        ASIG_CIRCLETWO,
        aeAnimSubtype_eStrip			}
};

#endif

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
#if 0
void SetupOptionList(void)
{
    MainOption_t    opt;
    int             layoutCount;
    int             layoutIndex;
    aeAnimSubtypeFlag_t flags;


    memset(&opt, 0, sizeof(opt));
    strcpy(opt.desc, "Open File...");
    opt.which = aeMain_Cmd;
    opt.cmd   = aeCmd_eFile_Open;
    sOptionList[sOptionCount++] = opt;

    memset(&opt, 0, sizeof(opt));
    opt.which = aeMain_Null;
    sOptionList[sOptionCount++] = opt;

    layoutCount = aeArchList_Layout_GetCount();

    for (layoutIndex = 0; layoutIndex < layoutCount; layoutIndex++)
    {
        /* Start with a sequence */
        memset(&opt, 0, sizeof(opt));
        opt.which = aeMain_Sequence;
        opt.layoutSignature = aeArchList_Layout_GetSignature(layoutIndex);
        aeArchList_Layout_GetName(layoutIndex, opt.desc, sizeof(opt.desc));
        strcat(opt.desc, " Sequence");
        sOptionList[sOptionCount++] = opt;

        /* Get the flags */
        flags = aeArchList_Layout_GetSubtypeFlags(layoutIndex);

        if (flags & aeAnimSubtypeFlag_eFrameByFrame)
        {
            memset(&opt, 0, sizeof(opt));
            opt.which = aeMain_Anim;
            opt.layoutSignature = aeArchList_Layout_GetSignature(layoutIndex);
            opt.subtype = aeAnimSubtype_eFrameByFrame;
            aeArchList_Layout_GetName(layoutIndex, opt.desc, sizeof(opt.desc));
            strcat(opt.desc, " Frame-by-Frame");
            sOptionList[sOptionCount++] = opt;
        }

        if (flags & aeAnimSubtypeFlag_ePixelList)
        {
            memset(&opt, 0, sizeof(opt));
            opt.which = aeMain_Anim;
            opt.layoutSignature = aeArchList_Layout_GetSignature(layoutIndex);
            opt.subtype = aeAnimSubtype_ePixelList;
            aeArchList_Layout_GetName(layoutIndex, opt.desc, sizeof(opt.desc));
            strcat(opt.desc, " Pixel List");
            sOptionList[sOptionCount++] = opt;
        }

        if (flags & aeAnimSubtypeFlag_eStrip)
        {
            memset(&opt, 0, sizeof(opt));
            opt.which = aeMain_Anim;
            opt.layoutSignature = aeArchList_Layout_GetSignature(layoutIndex);
            opt.subtype = aeAnimSubtype_eStrip;
            aeArchList_Layout_GetName(layoutIndex, opt.desc, sizeof(opt.desc));
            strcat(opt.desc, " Strip");
            sOptionList[sOptionCount++] = opt;
        }

        /* End with a blank */
        memset(&opt, 0, sizeof(opt));
        opt.which = aeMain_Null;
        sOptionList[sOptionCount++] = opt;

    }

}
#endif

void SetupGridList(void)
{
    GridItem_t      gi;
    GridSubItem_t   si;
    int             layoutCount;
    int             layoutIndex;
    aeAnimSubtypeFlag_t flags;
    int             yOffset;

    memset(&gi, 0, sizeof(gi));
    strcpy(gi.desc, "Open File...");
    gi.which = aeMain_Cmd;
    gi.cmd   = aeCmd_eFile_Open;
    sMainGrid.items[sMainGrid.itemCount++] = gi;

    layoutCount = aeArchList_Layout_GetCount();

    for (layoutIndex = 0; layoutIndex < layoutCount; layoutIndex++)
    {
        memset(&gi, 0, sizeof(gi));
        aeArchList_Layout_GetName(layoutIndex, gi.desc, sizeof(gi.desc));
        gi.layoutSignature = aeArchList_Layout_GetSignature(layoutIndex);

        yOffset = 20;

        #ifdef SHOW_LAYOUT
        yOffset += MAINWND_ANIM_VERTSIZE;
        #endif
        
        si.which = aeMain_Sequence;
        si.yOffset = yOffset;
        gi.subitem[gi.subitemCount++] = si;

        yOffset += sLineHeight + sLineHeight/2;

        /* Get the flags */
        flags = aeArchList_Layout_GetSubtypeFlags(layoutIndex);

        if (flags & aeAnimSubtypeFlag_eFrameByFrame)
        {
            si.which   = aeMain_Anim;
            si.yOffset = yOffset;
            si.subtype = aeAnimSubtype_eFrameByFrame;
            yOffset += sLineHeight;
            gi.subitem[gi.subitemCount++] = si;
        }

        if (flags & aeAnimSubtypeFlag_ePixelList)
        {
            si.which   = aeMain_Anim;
            si.yOffset = yOffset;
            si.subtype = aeAnimSubtype_ePixelList;
            yOffset += sLineHeight;
            gi.subitem[gi.subitemCount++] = si;
        }

        if (flags & aeAnimSubtypeFlag_eStrip)
        {
            si.which   = aeMain_Anim;
            si.yOffset = yOffset;
            si.subtype = aeAnimSubtype_eStrip;
            yOffset += sLineHeight;
            gi.subitem[gi.subitemCount++] = si;
        }

        gi.which = aeMain_Layout;

        #ifdef SHOW_LAYOUT
        gi.animInfo = aeArch_CreateWithSignatureSubtype(gi.layoutSignature, aeAnimSubtype_eFrameByFrame);

        /* Set all values in current frame to '1' and build a frame to display */
        aeAnimation_InvertCurrFrame(gi.animInfo->animData);
        aeAnimation_BuildFrame(gi.animInfo->animData);

        aeAnimationPane_ConfigFunc(&gi.animPane);
        //pane->windowContext = hWnd;
        gi.animPane.minWidth  = MAINWND_ANIM_HORZSIZE;
        gi.animPane.minHeight = MAINWND_ANIM_VERTSIZE;
        #endif
        
        sMainGrid.items[sMainGrid.itemCount++] = gi;

    }
}



int Grid_CalcHeight(Grid_t* grid)
{
    int height;

    height = grid->vertMargin * 2;
    height += grid->rowCount * grid->rowHeight;
    if (grid->rowCount > 0)
        height += (grid->rowCount - 1) * grid->vertSpace;

    return height;
}

int Grid_CalcWidth(Grid_t* grid)
{
    int width;

    width = grid->horzMargin * 2;
    width += grid->colCount * grid->colWidth;
    if (grid->colCount > 0)
        width += (grid->colCount - 1) * grid->horzSpace;
    return width;
}


void Grid_CalcRect(Grid_t* grid, int index, int* xLoc, int* yLoc, int* width, int* height)
{
    int row;
    int col;

    row = index / grid->colCount;
    col = index % grid->colCount;

    *xLoc   = grid->horzMargin + col * (grid->colWidth  + grid->horzSpace);
    *width  = grid->colWidth;
    *yLoc   = grid->vertMargin + row * (grid->rowHeight + grid->vertSpace);
    *height = grid->rowHeight;
}

char Grid_FindHitRect(Grid_t* grid, int* index, int xPos, int yPos)
{
    int     found = -1;
    int     i;
    int     xLoc, yLoc;
    int     width, height;

    for (i = 0; i < grid->itemCount && found == -1; i++)
    {
        Grid_CalcRect(grid, i, &xLoc, &yLoc, &width, &height);

        if (xLoc <= xPos && xPos <= xLoc + width && yLoc <= yPos && yPos <= yLoc + height)
            found = i;
    }

    if (found != -1 && index != NULL)
        *index = found;
    
    return found != -1;
}

