/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aepane.h
 *
 *	
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef AEPANE_H_
#define AEPANE_H_

#include "astd.h"

typedef struct aePane_tag aePane_t;
typedef enum
{
    aeMouseEvt_eNull = 0,
    aeMouseEvt_eDown,
    aeMouseEvt_eUp,
    aeMouseEvt_eMove,       /* Mouse button is down */
    aeMouseEvt_eOver,       /* Mouse button is up */
    aeMouseEvt_eEnter,
    aeMouseEvt_eLeave,

    aeMouseEvt_Count
} aeMouseEvt_t;

typedef void (*aePane_Draw_t)(aePane_t* pane, void* data, void* context);
typedef void (*aePane_MouseEvt_t)(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* data, void* context, int xPos, int yPos);
typedef void (*aePane_Erase_t)(void);
typedef void (*aePane_CharEvt_t)(void);
typedef void (*aePane_KeyEvt_t)(void);



struct aePane_tag
{
    aeRect_t    		loc;
    aePane_Draw_t       drawFunc;
    aePane_MouseEvt_t   mouseFunc;
    aePane_Erase_t      eraseFunc;
    aePane_CharEvt_t    charFunc;
    aePane_KeyEvt_t     keyEvt;
	void*				windowContext;
    void*               paneData;
	int					minWidth;
	int					minHeight;
	int					valueA;			/* Pane variables, for pane use */
	int					valueB;
};


#define AEPANE_MAX_PANES    10


typedef struct
{
    int         count;
    int         focus;
    int         hasMouse;
    aePane_t    pane[AEPANE_MAX_PANES];
} aePaneList_t;

void aePane_InitPaneList(aePaneList_t* paneList);

void aePane_HandleMouseEvt(aePaneList_t* paneSuper, void* privateData, void* wndcontext, aeMouseEvt_t mouseEvt, int modifiers, int xPos, int yPos);

#endif /* AEPANE_H_ */
