/* aeplatform.c */
#include <windows.h>
#include "astd.h"
#include "aeplatform.h"
#include "aedebug.h"
#include "aewndlist.h"

/* Platform Info */
static HINSTANCE	sInstance = NULL;
static HFONT		sNormalFont = NULL;
static HPEN			sWhitePen = NULL;
static HPEN			sGrayPen = NULL;


aErr_t aeWinPlat_CreateFonts(void);
aErr_t aeWinPlat_CreateGfxObjects(void);

/*----------------------------------------------------------------------
Summary
    Perform Platform initialization

Desc
    
----------------------------------------------------------------------*/
void aeWinPlat_Init(HINSTANCE hInstance)
{
    sInstance = hInstance;

    /* Required to address crash in xxx */
    /* See http://www.ureader.com/message/821299.aspx */
    CoInitialize(NULL);

    aeWinPlat_CreateFonts();
	aeWinPlat_CreateGfxObjects();

    aeui_WndList_Init();
}

/*----------------------------------------------------------------------
Summary
    Get the single "instance" of the app

Desc
    
----------------------------------------------------------------------*/
HINSTANCE aeWinPlat_GetInstance(void)
{
    AEASSERT(sInstance);

    return sInstance;
}


/*----------------------------------------------------------------------
Summary
    Create the fonts for the app

Desc
    
----------------------------------------------------------------------*/
aErr_t aeWinPlat_CreateFonts(void)
{
    LOGFONT logfont;

    memset(&logfont, 0, sizeof(logfont));
    logfont.lfHeight = 14;
    strcpy(logfont.lfFaceName, "Arial");

    sNormalFont = CreateFontIndirect(&logfont);

    return AERR_NO_ERROR;
}

/*----------------------------------------------------------------------
Summary
    Create any necessary graphics objects

Desc
    
----------------------------------------------------------------------*/
aErr_t aeWinPlat_CreateGfxObjects(void)
{
	sWhitePen = CreatePen(0, 0, RGB(0xff, 0xff, 0xff));

    sGrayPen =  CreatePen(0, 0, RGB(0x80, 0x80, 0x80));
    return AERR_NO_ERROR;
}

/*----------------------------------------------------------------------
Summary
    Get the basic font for the app

Desc
    
----------------------------------------------------------------------*/
HFONT aeWinPlat_GetNormalFont(void)
{
    AEASSERT(sNormalFont);

    return sNormalFont;
}

/*----------------------------------------------------------------------
Summary
    Get the white pen

Desc
    
----------------------------------------------------------------------*/
HPEN aeWinPlat_GetWhitePen(void)
{
	AEASSERT(sWhitePen);

	return sWhitePen;
}

/*----------------------------------------------------------------------
Summary
    Get animated speed

Desc
    
----------------------------------------------------------------------*/
HPEN aeWinPlat_GetGrayPen(void)
{
	AEASSERT(sGrayPen);

	return sGrayPen;
}

/*----------------------------------------------------------------------
Summary
    Get the gray pen

Desc
    
----------------------------------------------------------------------*/
COLORREF aeWinPlat_ConvertColor(aeColor_t aecolor)
{
	COLORREF wincolor;

	wincolor = ((aecolor & 0x0000ff) << 16) |
			   ((aecolor & 0x00ff00)) |
			   ((aecolor & 0xff0000) >> 16);

	return wincolor;
}

