/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aesequencewnd.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include "animator.h"
#include "aemainwnd.h"
#include "aesequencewnd.h"
#include "aeeditorwnd.h"
#include "aewndlist.h"
#include "astd.h"
#include "asig.h"
#include "aepane.h"
#include "aeplatform.h"
#include "aedebug.h"
#include "aecommands.h"
#include "aesequence.h"
#include "aeanimation.h"
#include "aeanimationpane.h"
#include "aedialogs.h"
#include "aetoolbarpane.h"
#include "aegraphics.h"

#ifdef ANIM_HAS_GDLIB
#include "aegfxfile.h"
#endif

char szAESequenceWndClassName[] = "AESequenceWnd";


#define AESEQUENCEWND_INITIAL_WIDTH     320
#define	AESEQUENCEWND_INITIAL_HEIGHT	280

#define AESEQUENCEWND_SEQINFO_HEIGHT_MIN   60
#define AESEQUENCEWND_SEQINFO_WIDTH_MIN   100


LRESULT CALLBACK aeSequenceWnd_WndProc(HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam);


typedef struct
{
    aeSequence_t*       sequence;
    aeAnimationData_t*  blankAnimData;
    aeAnimationInfo_t*  animInfo;
    aePaneList_t        panes;
} aeSequenceWnd_Data_t;



void                  aeSequenceWnd_CharEvt(aeSequence_t* sequence, HWND hWnd, WPARAM wParam, BOOL charEvt);
void                  aeSequenceWnd_TimerEvt(aeSequenceWnd_Data_t* swdata, HWND hwnd);
void                  aeSequenceWnd_UpdateTimer(aeSequence_t* sequence, HWND hWnd);
void                  aeSequenceWnd_KillTimer(HWND hWnd);
void                  aeSequenceWnd_Destroy(aeSequenceWnd_Data_t* data, HWND hwnd);
void                  aeSequenceWnd_DestroyData(aeSequenceWnd_Data_t* seqWndData);
aeSequenceWnd_Data_t* aeSequenceWnd_CreateAndConfigData(aeSequence_t* sequence);
void                  aeSequenceWnd_HandleAnimClick(HWND hWnd, aeSequence_t* sequence, int animIndex);
void                  aeSequenceWnd_OpenAnimation(HWND hWnd, aeSequence_t* sequence, int animIndex);
void                  aeSequenceWnd_BringAnimationForward(HWND hWnd, aeSequence_t* sequence, int animIndex);
aeuiWndResponse_t     aeSequenceWnd_HandleSave(aeSequence_t* sequence, HWND hwnd, BOOL doSaveAs);
aeuiWndResponse_t     aeSequenceWnd_DoFileSave(aeSequence_t* sequence, HWND hwnd, BOOL doSaveAs);
static void           aeSequenceWnd_DoCommand(void* privateData, void* wndcontext, aeCommand_t command);
char*                 aeSequenceWnd_p_MakeTitle(aeSequence_t* sequence, char* titlestr, int titlestrlen);
void                  aeSequenceWnd_InstallPanes(aeSequenceWnd_Data_t* swData, HWND hWnd);
void                  aeSequenceWnd_GetMinimumSize(aeSequenceWnd_Data_t* swData, int* minWidth, int* minHeight);
void                  aeSequenceWnd_PositionPanes(aeSequenceWnd_Data_t* swData, int windowWidth, int windowHeight);
void                  aeSequenceWnd_HandleResize(aeSequenceWnd_Data_t* swData, int windowWidth, int windowHeight);
void                  aeAnimationListPane_ConfigFunc(aePane_t* pane);
void                  aeSequenceInfoPane_ConfigFunc(aePane_t* pane);


#define SEQITEM_MAXROWS     30
#define SEQITEM_WIDTH      200
#define SEQITEM_SPACE        0
#define SEQITEM_HEIGHT      16
#define SEQITEM_VERTOFFSET   0
#define SEQITEM_HORZOFFSET   0

#define SEQANIM_VERTOFFSET   0
#define SEQANIM_HORZOFFSET 160
#define SEQANIM_VERTSIZE    52
#define SEQANIM_HORZSIZE    52

#define MIN_SEQITEM_HEIGHT (10 * (SEQITEM_SPACE + SEQITEM_HEIGHT))

int aeSequenceWnd_CalcItemsHeight(int itemCount);
int aeSequenceWnd_CalcItemsWidth(int itemCount);

/* Line Height */
static int sLineHeight = 14;



enum
{
	AESequenceWnd_PaneID_eToolbar = 0,
	AESequenceWnd_PaneID_eSequenceInfo,
    AESequenceWnd_PaneID_eAnimation,
	AESequenceWnd_PaneID_eAnimationList,

	AESequenceWnd_PaneID_Count
};





static aeToolbarButton_t sToolbarList[] = 
{
    { aeCmd_eFile_Save,     			"S", 1 },
    { aeCmd_eFile_SaveAs,   			"W", 1 },
	{ aeCmd_eFile_Open, 				"O", 1 },
	{ aeCmd_eFile_New, 					"N", 1 },
    { aeCmd_eFile_ExportGif,    		"G", 1 },
	
	{ aeCmd_eSequence_Delete, 			"X", 1 },
	{ aeCmd_eSequence_ToggleActive, 	"I", 1 },
	{ aeCmd_eSequence_ActiveSet_Append, "=", 1 },
	{ aeCmd_eSequence_ActiveSet_Delete, "-", 1 },
	{ aeCmd_eSequence_ActiveSet_Prev, 	"<", 1 },
	{ aeCmd_eSequence_ActiveSet_Next, 	">", 1 },
	{ aeCmd_eSequence_Animated_Prev, 	"[", 1 },
	{ aeCmd_eSequence_Animated_Next, 	"]", 1 },
	{ aeCmd_eSequence_Variant_Prev, 	"{", 1 },
	{ aeCmd_eSequence_Variant_Next, 	"}", 1 },

	{ aeCmd_eSequence_SelectPrev,       "<-", 0 },
	{ aeCmd_eSequence_SelectNext,       "->", 0 },
	{ aeCmd_eSequence_MoveUp,           "^",  0 },
	{ aeCmd_eSequence_MoveDown,         "v",  0 }

};

static aeToolbarPaneData_t sToolbarData;

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeSequenceWnd_Create(aeSequence_t* sequence)
{
    aErr_t  err = AERR_NO_ERROR;
	HWND    hWnd;
    int     extW;
	int		extH;
	int		minWidth;
	int		minHeight;
    int     maxWidth;
    int     maxHeight;
    int     itemCount;
    char    titlestr[50];
    aeSequenceWnd_Data_t* swData;

    swData = aeSequenceWnd_CreateAndConfigData(sequence);

    if (swData == NULL)
        err = AERR_OUT_OF_MEMORY;

    if (err == AERR_NO_ERROR)
    {
        extH = 30;
        extW = 8;

        itemCount = aeSequence_GetCount(sequence);
        if (itemCount < 5)
            itemCount = 5;

		/* Ask the layout what size it min */
		aeSequenceWnd_GetMinimumSize(swData, &minWidth, &minHeight);

        /* Get the maximum size allowed */
        aeuiWnd_Window_GetMaxSize(&maxWidth, &maxHeight);

        if (minWidth > maxWidth)
            minWidth = maxWidth;

        if (minHeight > maxHeight)
            minHeight = maxHeight;

		/* Position the panes according to the size */
		aeSequenceWnd_PositionPanes(swData, minWidth, minHeight);

        hWnd = CreateWindow( 	szAESequenceWndClassName, aeSequenceWnd_p_MakeTitle(sequence, titlestr, sizeof(titlestr)), WS_OVERLAPPEDWINDOW,
                                CW_USEDEFAULT, CW_USEDEFAULT, 
                                minWidth + extW, minHeight + extH,
                                (HWND)NULL, (HMENU)NULL, (HANDLE)aeWinPlat_GetInstance(), (LPSTR)NULL);

        if (hWnd == NULL)
        {
            err = AERR_OS_ERROR;

            aeSequenceWnd_DestroyData(swData);
            swData = NULL;
        }
        else
        {
			{ int i;
			for (i = 0; i < swData->panes.count; i++)
				swData->panes.pane[i].windowContext = hWnd;
			}

            aeui_WndList_Add(hWnd, swData, swData->sequence, aeuiWndType_eSequence);

            ShowWindow(hWnd, SW_SHOWDEFAULT);
            UpdateWindow( hWnd );

            aeSequenceWnd_UpdateTimer(swData->sequence, hWnd);
        }
    }

    return err;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeSequenceWnd_Init(void)
{
    aErr_t      err = AERR_NO_ERROR;
	WNDCLASS    wc;

    wc.lpszClassName 	= szAESequenceWndClassName;
    wc.hInstance 		= aeWinPlat_GetInstance();
    wc.lpfnWndProc		= aeSequenceWnd_WndProc;
    wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
    wc.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
    wc.lpszMenuName		= NULL;
    wc.hbrBackground	= GetStockObject( WHITE_BRUSH );
    wc.style			= 0;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;

    if (!RegisterClass(&wc))
        err = AERR_OS_ERROR;

    return err;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_InstallPanes(aeSequenceWnd_Data_t* swData, HWND hWnd)
{
    aePane_t*   pane;
    int         itemCount;

    /* Toolbar */
    sToolbarData.count = sizeof(sToolbarList)/sizeof(aeToolbarButton_t);
    sToolbarData.list  = sToolbarList;
    sToolbarData.handlerProc = aeSequenceWnd_DoCommand;

    pane = &swData->panes.pane[AESequenceWnd_PaneID_eToolbar];
    aeToolbarPane_InitData(pane, &sToolbarData);
    aeToolbarPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Sequence Info */
    pane = &swData->panes.pane[AESequenceWnd_PaneID_eSequenceInfo];
    aeSequenceInfoPane_ConfigFunc(pane);
	pane->windowContext = hWnd;

    /* Animation */
    pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimation];
    aeAnimationPane_ConfigFunc(pane);
	pane->windowContext = hWnd;
    pane->minWidth  = SEQANIM_HORZSIZE;
    pane->minHeight = SEQANIM_VERTSIZE;

    /* Animation List */
    pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimationList];
    aeAnimationListPane_ConfigFunc(pane);
	pane->windowContext = hWnd;
    itemCount = aeSequence_GetCount(swData->sequence);
    pane->minHeight = aeSequenceWnd_CalcItemsHeight(itemCount);
    pane->minWidth  = aeSequenceWnd_CalcItemsWidth(itemCount);

    swData->panes.count = AESequenceWnd_PaneID_Count;

}
/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
int aeSequenceWnd_CalcItemsWidth(int itemCount)
{
    int cols;

    cols = itemCount / SEQITEM_MAXROWS;

    return SEQITEM_HORZOFFSET + (cols + 1) * (SEQITEM_WIDTH + SEQITEM_SPACE);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
int aeSequenceWnd_CalcItemsHeight(int itemCount)
{
    int rows;
    int height;

    if (itemCount > SEQITEM_MAXROWS)
        rows = SEQITEM_MAXROWS;
    else
        rows = itemCount;

    height = SEQITEM_VERTOFFSET + (rows) * (SEQITEM_HEIGHT + SEQITEM_SPACE);

    if (height < MIN_SEQITEM_HEIGHT)
        height = MIN_SEQITEM_HEIGHT;

    return height;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static aeRect_t aeSequenceWnd_CalcItemRect(int item, int rectHeight)
{
	aeRect_t	r;
    int     row;
    int     col;
    int     itemHeight;
    int     maxRows;

    itemHeight = SEQITEM_HEIGHT + SEQITEM_SPACE;
    maxRows = rectHeight / itemHeight;

    if (maxRows == 0)
        maxRows = 1;


    row = item % maxRows;
    col = item / maxRows;

	r.left   = SEQITEM_HORZOFFSET + col * (SEQITEM_WIDTH + SEQITEM_SPACE);
	r.right  = r.left + SEQITEM_WIDTH;
	r.top    = SEQITEM_VERTOFFSET + row * (SEQITEM_HEIGHT + SEQITEM_SPACE);
	r.bottom = r.top + SEQITEM_HEIGHT;

	return r;
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
LRESULT CALLBACK aeSequenceWnd_WndProc( HWND hWnd, UINT messg, WPARAM wParam, LPARAM lParam )
{
	HDC         hdc;
	PAINTSTRUCT pstruct;
    HFONT       oldFont;
	int			xPos, yPos;
    int         windowWidth;
    int         windowHeight;
    char        continueClose;
    int         paneIndex;
    aePane_t*   pane;
    aeSequenceWnd_Data_t*	swdata;
    aeSequence_t*			sequence = NULL;

    swdata = aeui_WndList_GetDataA(hWnd);
    
    if (swdata != NULL)
	    sequence = swdata->sequence;

	switch(messg)
	{
        case WM_PAINT:
            if (sequence && aeSequence_GetCount(sequence) > 0)
                swdata->animInfo->animData = aeSequence_Animated_GetData(sequence);
            else
                swdata->animInfo->animData = swdata->blankAnimData;

               
			hdc = BeginPaint(hWnd, &pstruct);
            oldFont = SelectObject(hdc, aeWinPlat_GetNormalFont());
            for (paneIndex = 0; paneIndex < swdata->panes.count; paneIndex++)
            {
                pane = &swdata->panes.pane[paneIndex];
                /* aegfxFrameRect(hdc, &pane->loc, AECOLOR_BLACK); */

                if (pane->drawFunc != NULL)
                {
                    if (paneIndex == AESequenceWnd_PaneID_eAnimation)
                        (*pane->drawFunc)(pane, swdata->animInfo, hdc);
                    else
                        (*pane->drawFunc)(pane, swdata->sequence, hdc);
                }
            }
            SelectObject(hdc, oldFont);
			EndPaint(hWnd, &pstruct);
		break;
		
        case WM_CHAR: 
            if (sequence != NULL)
                aeSequenceWnd_CharEvt(sequence, hWnd, wParam, TRUE /* char */);
            break;

        case WM_KEYDOWN: 
            if (sequence != NULL)
                aeSequenceWnd_CharEvt(sequence, hWnd, wParam,  FALSE /* keydown */);
            break;

        case WM_CLOSE:
            continueClose = TRUE;
            if (sequence != NULL)
                continueClose = aeSequenceWnd_RequestClose(sequence);

            /* Pass to standard handling to continue with close */
            if (continueClose)
                return DefWindowProc(hWnd, messg, wParam, lParam);
            break;

        case WM_DESTROY:
            if (sequence != NULL)
            {
                aeSequenceWnd_Destroy(swdata, hWnd);
            }
    		break;
		
        case WM_TIMER:
            aeSequenceWnd_TimerEvt(swdata, hWnd);
            break;

        case WM_LBUTTONDOWN:
            xPos = LOWORD(lParam); 
            yPos = HIWORD(lParam);

            aePane_HandleMouseEvt(&swdata->panes, swdata->sequence, hWnd, aeMouseEvt_eDown, 0, xPos, yPos);
		break;
		

        case WM_SIZE:
            windowWidth  = LOWORD(lParam); 
            windowHeight = HIWORD(lParam);
            aeSequenceWnd_HandleResize(swdata, windowWidth, windowHeight);
            InvalidateRect(hWnd, NULL, TRUE);
		break;

        default:
		return DefWindowProc( hWnd, messg, wParam, lParam );
	}

	return( 0L );
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
char aeSequenceWnd_RequestClose(aeSequence_t* sequence)
{
    char    continueClose = TRUE;
    HWND    hWnd;
    aeuiWndResponse_t response;

    AEASSERT(sequence);

    hWnd = aeui_WndList_GetWndFromDataB(sequence);
    AEASSERT(hWnd);

    if (aeSequence_GetChangedFlag(sequence))
    {
        response = aeuiWnd_DataChangedConfirmClose(aeSequence_GetName(sequence));

        if (response == aeuiWndResponse_eCancel)
            continueClose = FALSE;
        else if (response == aeuiWndResponse_eYes)
        {
           response = aeSequenceWnd_HandleSave(sequence, hWnd, FALSE);
           if (response == aeuiWndResponse_eCancel)
               continueClose = FALSE;
        }
        else
            ; /* Don't want to save and don't want to cancel */
    }


    return continueClose;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_Destroy(aeSequenceWnd_Data_t* seqWndData,  HWND hWnd)
{
    aeSequence_t*   sequence;
    int             index;
    aeAnimationData_t*  animData;


    AEASSERT(seqWndData && seqWndData->sequence);

    /* Clear the timer */
    aeSequenceWnd_KillTimer(hWnd);

    /* For each animation in the sequence, if the animation is open, then we need to clear the reference from the */
    /* open window back to the sequence */

    for (index = 0; index < aeSequence_GetCount(seqWndData->sequence); index++)
    {
        if (aeSequence_Anim_IsOpened(seqWndData->sequence, index))
        {
            animData = aeSequence_Anim_GetOpenedDataRef(seqWndData->sequence, index);
            aeSequence_ClearOpenedDataRef(seqWndData->sequence, animData);

            /* Detach the editor window from the sequence */
            aeEditorWnd_ClearSequence(animData);
        }

    }


    /* Remove the sequence reference from the window data */
    sequence = seqWndData->sequence;
    seqWndData->sequence = NULL;

    /* Take the entry out of the list before free data */
    aeui_WndList_Delete(hWnd);

	/* Free the data associated with the window */
    aeSequenceWnd_DestroyData(seqWndData);
    seqWndData = NULL;

    /* Delete the sequence */
    aeSequence_Destroy(sequence);
    sequence = NULL;

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_DestroyData(aeSequenceWnd_Data_t* seqWndData)
{
    AEASSERT(seqWndData);

    free(seqWndData);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_CharEvt(aeSequence_t* sequence, HWND hWnd, WPARAM wParam, BOOL charEvt)
{
    unsigned char   ch;
    aeCommand_t     cmd = aeCmd_eNull;
    int             i;

    if (charEvt)
    {
        ch = wParam;

        if (ch >= 'a' && ch <= 'z')
            ch = ch + 'A' - 'a';

        for (i = 0; i < sizeof(sToolbarList)/sizeof(*sToolbarList) && cmd == aeCmd_eNull; i++)
        {
            /* Must be one char long string and allowed as a keypress */
            if (sToolbarList[i].charAllowed && sToolbarList[i].desc[1] == 0 && sToolbarList[i].desc[0] == ch)
                cmd = sToolbarList[i].cmd;
        }

        if (cmd == aeCmd_eNull)
        {
            switch (ch)
            { 
                /*
                case 'O':
                case 'o':
                    cmd = aeCmd_eFile_Open;
                    break;
    
                case 'n':
                case 'N':
                    cmd = aeCmd_eFile_New;
    
                case 'S':
                case 's':
                    cmd = aeCmd_eFile_Save;
                    break;

                case 'W':
                case 'w':
                    cmd = aeCmd_eFile_SaveAs;
                    break;

                case 'X':
                case 'x':
                    cmd = aeCmd_eSequence_Delete;
                    break;

                case 'I':
                case 'i':
                    cmd = aeCmd_eSequence_ToggleActive;
                    break;

                case '+':
                case '=':
                    cmd = aeCmd_eSequence_ActiveSet_Append;
                    break;

                case '-':
                    cmd = aeCmd_eSequence_ActiveSet_Delete;
                    break;

                case '<':
                    cmd = aeCmd_eSequence_ActiveSet_Prev;
                    break;

                case '>':
                    cmd = aeCmd_eSequence_ActiveSet_Next;
                    break;

                case '[':
                    cmd = aeCmd_eSequence_Animated_Prev;
                    break;

                case ']':
                    cmd = aeCmd_eSequence_Animated_Next;
                    break;

                case '{':
                    cmd = aeCmd_eSequence_Variant_Prev;
                    break;

                case '}':
                    cmd = aeCmd_eSequence_Variant_Next;
                    break;
                    */

            } 
        }
    }
    else /* keydown */
    {
        switch (wParam)
        { 
            case VK_LEFT: 
                cmd = aeCmd_eSequence_SelectPrev;
                break; 
            case VK_RIGHT: 
                cmd = aeCmd_eSequence_SelectNext;
                break; 
            case VK_UP: 
                cmd = aeCmd_eSequence_MoveUp;
                break; 
            case VK_DOWN:
                cmd = aeCmd_eSequence_MoveDown; 
                break; 
        } 
    }
    
    if (cmd != aeCmd_eNull)
        aeSequenceWnd_DoCommand(sequence, hWnd, cmd);

}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_UpdateTimer(aeSequence_t* sequence, HWND hWnd)
{
	int millisec;
	int speed;

    speed = aeSequence_Animated_GetSpeed(sequence);

	if (speed == 0)
		speed = 0xc0;

	millisec = speed;
	
	if (millisec < 10)
		millisec = 10;
	
	SetTimer(hWnd, 1, millisec, NULL);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_KillTimer(HWND hWnd)
{
    KillTimer(hWnd, 0);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_TimerEvt(aeSequenceWnd_Data_t* swdata, HWND hWnd)
{
    aeSequence_Animated_Advance(swdata->sequence);
    InvalidateRect(hWnd, NULL, FALSE);

    if (aeSequence_Animated_AnimIsAtInitialAnimationState(swdata->sequence))
        aeSequenceWnd_UpdateTimer(swdata->sequence, hWnd);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_DoCommand(void* privateData, void* wndcontext, aeCommand_t command)
{
    int     index;
    char    activeFlag;

    aeSequence_t* sequence = (aeSequence_t*)privateData;
    HWND hWnd = (HWND)wndcontext;

    switch (command)
    { 
        case aeCmd_eFile_Save:
            aeSequenceWnd_HandleSave(sequence, hWnd, FALSE);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eFile_SaveAs:
            {
                aeSequenceWnd_DoFileSave(sequence, hWnd, TRUE);
            }
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eFile_ExportGif:
            #ifdef ANIM_HAS_GDLIB
            aeGfxFile_ExportSequenceAsGif(sequence, "test.gif");
            #endif
            break;

        case aeCmd_eSequence_SelectPrev:
            aeSequence_Selected_UpdateIndex(sequence, aePosition_ePrev);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eSequence_SelectNext:
            aeSequence_Selected_UpdateIndex(sequence, aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eSequence_Delete:
            if (aeSequence_GetCount(sequence) > 0)
            {
                index = aeSequence_Selected_GetIndex(sequence);

                if (aeSequence_Anim_IsOpened(sequence, index))
                {
                    aeuiWnd_Error("The animation is open in a window and can not be deleted.", AERR_GENERAL_ERROR);
                }
                else
                {
                    aeuiWndResponse_t resp;
                    resp = aeuiWnd_AskOKCancel("Are you sure you to delete this animation? It can not be undone.");

                    if (resp == aeuiWndResponse_eOK)
                    {
                        aeSequence_DeleteAnimation(sequence, index);
                        InvalidateRect(hWnd, NULL, TRUE);
                    }
                }
            }
            break;

        case aeCmd_eSequence_MoveUp:
            aeSequence_Selected_Move(sequence, aePosition_ePrev);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eSequence_MoveDown:
            aeSequence_Selected_Move(sequence, aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            break;

        case aeCmd_eSequence_ToggleActive:
            if (aeSequence_GetCount(sequence) > 0)
            {
                index = aeSequence_Selected_GetIndex(sequence);
                activeFlag = aeSequence_Anim_IsActive(sequence, index);
                aeSequence_Anim_SetActive(sequence, index, !activeFlag);
                InvalidateRect(hWnd, NULL, FALSE);
            }
            break;

        case aeCmd_eSequence_ActiveSet_Prev:
        case aeCmd_eSequence_ActiveSet_Next:
            aeSequence_ActiveSet_Select(sequence, (command == aeCmd_eSequence_ActiveSet_Prev) ? aePosition_ePrev : aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eSequence_ActiveSet_Append:
            aeSequence_ActiveSet_Append(sequence);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eSequence_ActiveSet_Delete:
            aeSequence_ActiveSet_DeleteCurr(sequence);
            InvalidateRect(hWnd, NULL, FALSE);
            break; 

        case aeCmd_eSequence_Animated_Prev:
        case aeCmd_eSequence_Animated_Next:
            aeSequence_Animated_Select(sequence, (command == aeCmd_eSequence_Animated_Prev) ? aePosition_ePrev : aePosition_eNext, FALSE);
            InvalidateRect(hWnd, NULL, FALSE);
            aeSequenceWnd_UpdateTimer(sequence, hWnd);
            break; 

        case aeCmd_eSequence_Variant_Prev:
        case aeCmd_eSequence_Variant_Next:
            index = aeSequence_Selected_GetIndex(sequence);
            aeSequence_Anim_VariantSelect(sequence, index, (command == aeCmd_eSequence_Variant_Prev) ? aePosition_ePrev : aePosition_eNext);
            InvalidateRect(hWnd, NULL, FALSE);
            aeSequenceWnd_UpdateTimer(sequence, hWnd);
            break; 

        
        case aeCmd_eFile_Open:
        case aeCmd_eFile_New:
        default:
            aeMainWnd_DoCommand(hWnd, command);
            break;

    } 


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_BringForward(aeSequence_t* sequence)
{
    AEASSERTED("aeSequenceWnd_BringForward: to do");
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_OpenAnimation(HWND hWnd, aeSequence_t* sequence, int animIndex)
{
    aeAnimationData_t*  copyData;
    aeAnimationInfo_t*  animInfo;
    


    copyData = aeSequence_Anim_GetDataCopyForEdit(sequence, animIndex);

    if (copyData != NULL)
    {
        animInfo = aeArch_CreateWithData(copyData);
        if (animInfo != NULL)
        {

            aeSequence_Anim_OpenedForEdit(sequence, animIndex, copyData);
            aeSequence_SetAllMatchingOpenedDataRef(sequence, animIndex);

            /* Associate the animation with the sequence */
            aeAnimation_SetSequence(copyData, sequence);

            /* Copied data is considered 'clean', even if original data is 'changed' */
            aeAnimation_ResetChangedFlag(copyData);


            /* Open the window */
            aeEditorWnd_Create(animInfo);
        }
        else
        {
            aeAnimation_Delete(copyData);
            copyData = NULL;
        }
    }


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_BringAnimationForward(HWND hWnd, aeSequence_t* sequence, int animIndex)
{
    aeAnimationData_t* animData;

    AEASSERT(sequence);

    animData = aeSequence_Anim_GetOpenedDataRef(sequence, animIndex);

    aeEditorWnd_BringForward(animData);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_HandleAnimClick(HWND hWnd, aeSequence_t* sequence, int animIndex)
{
    AEASSERT(sequence);
    AEASSERT(animIndex >= 0 && animIndex < aeSequence_GetCount(sequence));

    if (aeSequence_Anim_IsOpened(sequence, animIndex))
    {
        aeSequenceWnd_BringAnimationForward(hWnd, sequence, animIndex);
    }
    else
    {
        aeSequenceWnd_OpenAnimation(hWnd, sequence, animIndex);
    }

    /* Make the opened item the selected item */
    aeSequence_Selected_SetIndex(sequence, animIndex);

    aeSequenceWnd_InvalidateWindow(sequence);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeSequenceWnd_Data_t* aeSequenceWnd_CreateAndConfigData(aeSequence_t* sequence)
{
    aeSequenceWnd_Data_t* swData;

    swData = malloc(sizeof(aeSequenceWnd_Data_t));

    if (swData != NULL)
    {
        memset(swData, 0, sizeof(aeSequenceWnd_Data_t));

        swData->sequence = sequence;

        aePane_InitPaneList(&swData->panes);
        aeSequenceWnd_InstallPanes(swData, NULL);


        swData->animInfo = aeArch_CreateWithSignatureSubtype(aeSequence_GetSignature(sequence), aeAnimSubtype_eFrameByFrame);

        /* Keep the original around */
        swData->blankAnimData = swData->animInfo->animData;

		//aeLayout_CalcBounds(&swData->animInfo->animLayout, &swData->panes.pane[AESequenceWnd_PaneID_eAnimation].loc);
    }

    return swData;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_InvalidateWindow(aeSequence_t* sequence)
{
    HWND    hWnd = NULL;
    int     count;
    int     index;
    aeSequence_t* seqsearch;

    count = aeui_WndList_Type_GetCount(aeuiWndType_eSequence);

    for (index = 0; index < count && hWnd == NULL; index++)
    {
        seqsearch = aeui_WndList_Type_GetDataBByTypeIndex(aeuiWndType_eSequence, index);

        if (seqsearch == sequence)
            hWnd = aeui_WndList_Type_GetWndByTypeIndex(aeuiWndType_eSequence, index);
    }

    if (hWnd != NULL)
        InvalidateRect(hWnd, NULL, 1 /* erase */);
    else
        AEASSERTED("sequence hwnd not found");

}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeuiWndResponse_t aeSequenceWnd_HandleSave(aeSequence_t* sequence, HWND hwnd, BOOL doSaveAs)
{
    aeuiWndResponse_t response;
    response = aeSequenceWnd_DoFileSave(sequence, hwnd, FALSE /* not saveAs */);

    return response;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeuiWndResponse_t aeSequenceWnd_DoFileSave(aeSequence_t* sequence, HWND hwnd, BOOL doSaveAs)
{
    aErr_t  err;
    char    path[MAX_ANIM_PATH];
    char	filename[MAX_ANIM_FILE];
    BOOL	doSave;
    int     i;
    char    titlestr[50];
    OPENFILENAME        ofn;
    aeuiWndResponse_t   response = aeuiWndResponse_eOK;
    
    /* Get the previous path, if any. If not, this will set path to empty string */
    aeSequence_GetFilePath(sequence, path, MAX_ANIM_PATH);


    if (strlen(path) == 0 || doSaveAs)
    {
        memset(&filename, 0, sizeof(filename));
        
        memset(&ofn, 0, sizeof(ofn));
        ofn.lStructSize     = sizeof(ofn);
        ofn.hwndOwner       = hwnd;
        ofn.lpstrFile       = path;
        ofn.nMaxFile        = sizeof(path);
        ofn.lpstrFileTitle  = filename;
        ofn.nMaxFileTitle   = sizeof(filename);
        ofn.lpstrDefExt     = "asm";
    	ofn.Flags           = OFN_OVERWRITEPROMPT;
        
        doSave = GetSaveFileName(&ofn);

        if (doSave)
        {
	        aeSequence_SetFilePath(sequence, path);

            /* Update the name of the animation */
            for (i = 0; i < strlen(filename); i++)
                if (filename[i] == '.')
                    filename[i] = 0;
                    
            aeSequence_SetName(sequence, filename);

            aeuiWnd_ChangeTitle(hwnd, aeSequenceWnd_p_MakeTitle(sequence, titlestr, sizeof(titlestr)));
        }
        else
        {
            response = aeuiWndResponse_eCancel;
        }
    }
    else
    {
        doSave = TRUE;
    }
    
    if (doSave)
    {
        err = aeSequence_Save(sequence, path);
        if (err != AERR_NO_ERROR)
        {
            aeuiWnd_Error("Error saving sequence to file", err);
            response = aeuiWndResponse_eCancel;
        }
    }

    return response;
}





/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
char aeSequenceWnd_FindSequenceForArch(aSignature_t signature, aeSequence_t** pSequence)
{
    char            windowFound = FALSE;
    aeSequence_t*   sequence = NULL;
    int             count;
    int             index;

    count = aeui_WndList_Type_GetCount(aeuiWndType_eSequence);

    for (index = 0; index < count && !windowFound; index++)
    {
        sequence = aeui_WndList_Type_GetDataBByTypeIndex(aeuiWndType_eSequence, index);
        AEASSERT(sequence);

        if (aeSequence_GetSignature(sequence) == signature)
        {
            windowFound = TRUE;
        }
    }

    if (windowFound && pSequence != NULL)
        *pSequence = sequence;

    return windowFound;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
char* aeSequenceWnd_p_MakeTitle(aeSequence_t* sequence, char* titlestr, int titlestrlen)
{
    AEASSERT(titlestr && titlestrlen);

    sprintf(titlestr, "Sequence: %s", aeSequence_GetName(sequence));

    return titlestr;
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_PositionPanes(aeSequenceWnd_Data_t* swData, int windowWidth, int windowHeight)
{
	aePane_t*	pane;
	aeRect_t	pr;
	int			wmA, wmB;
	int			wA, wB;
	int			hmTL, hmTR;
	int			hmT, hmB;

	/* Toolbar */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eToolbar];
	pr.left   = 0;
	pr.top    = 0;
	pr.right  = windowWidth;
	pr.bottom = pane->minHeight;
	pane->loc = pr;

	/* Sequence Info */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eSequenceInfo];
	wmA = pane->minWidth;
	hmTL = pane->minHeight;

    /* Animation */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimation];
	wmB = pane->minWidth;
	hmTR = pane->minHeight;

	wA = windowWidth - wmB;
	wB = wmB;

    /* Animation List */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimationList];
	wmA = pane->minWidth;
	hmB = pane->minHeight;

	hmT = (hmTL < hmTR) ? hmTR : hmTL;


	/* Sequence Info */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eSequenceInfo];
	pr.top    = pr.bottom;
	pr.bottom = pr.top + hmT;
	pr.left   = 0;
	pr.right  = wA;
	pane->loc = pr;

	/* Animation */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimation];
	pr.left  = pr.right;
	pr.right = windowWidth;
	pane->loc = pr;
    aeLayout_CalcBounds(swData->animInfo->animLayout, &swData->panes.pane[AESequenceWnd_PaneID_eAnimation].loc);

    /* Animation List */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eAnimationList];
	pr.top    = pr.bottom;
	//pr.bottom = (windowHeight > pr.top + pane->minHeight) ? windowHeight : pr.top + pane->minHeight;
    if (windowHeight > pr.top)
        pr.bottom = windowHeight;
    else
        pr.bottom = pr.top + MIN_SEQITEM_HEIGHT;

	pr.left   = 0;
	pr.right  = (windowWidth > pane->minWidth) ? windowWidth : pane->minWidth;
	pane->loc = pr;


}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_HandleResize(aeSequenceWnd_Data_t* swData, int windowWidth, int windowHeight)
{
    aeSequenceWnd_PositionPanes(swData, windowWidth, windowHeight);

}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceWnd_GetMinimumSize(aeSequenceWnd_Data_t* swData, int* minWidth, int* minHeight)
{
	aePane_t*	pane;
	aePane_t*	paneL;
	aePane_t*	paneR;
	int			minW;
	int			minH;
    int         itemCount;
    int         minListWidth;
    int         minListHeight;

    itemCount = aeSequence_GetCount(swData->sequence);

	/* Toolbar */
	pane = &swData->panes.pane[AESequenceWnd_PaneID_eToolbar];
	minW = pane->minWidth;
	minH = pane->minHeight;

    /* Sequence Info */
	paneL = &swData->panes.pane[AESequenceWnd_PaneID_eSequenceInfo];

    /* Animation */
	paneR = &swData->panes.pane[AESequenceWnd_PaneID_eAnimation];

	if (paneL->minHeight > paneR->minHeight)
		minH += paneL->minHeight;
	else
		minH += paneR->minHeight;

	if (minW < paneL->minWidth + paneR->minWidth)
		minW = paneL->minWidth + paneR->minWidth;

    /* Animation List */
	paneL = &swData->panes.pane[AESequenceWnd_PaneID_eAnimationList];
    minListWidth  = aeSequenceWnd_CalcItemsWidth(itemCount);
    minListHeight = aeSequenceWnd_CalcItemsHeight(itemCount);

    minH += minListHeight;

	if (minW < minListWidth)
		minW = minListWidth;


	*minWidth  = minW;
	*minHeight = minH;
}

/*======================================================================

    Sequence Info Pane
    
======================================================================*/
void aeSequenceInfoPane_Draw(aePane_t* pane, void* data, void* context);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceInfoPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc  = aeSequenceInfoPane_Draw;

	pane->minHeight = AESEQUENCEWND_SEQINFO_HEIGHT_MIN;
	pane->minWidth  = AESEQUENCEWND_SEQINFO_WIDTH_MIN;

}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeSequenceInfoPane_Draw(aePane_t* pane, void* data, void* context)
{
	HDC     hdc = (HDC)context;
    char    str[200];
    int     left;
    int     top;
    char    changed;
    aeSequence_t* sequence = NULL;

    sequence = (aeSequence_t*)data;
    AEASSERT(sequence);

    left = pane->loc.left + 10;
    top  = pane->loc.top  + 3;

    changed = aeSequence_GetChangedFlag(sequence);

    sprintf(str, "Sequence '%s' %c     ", aeSequence_GetName(sequence), changed?'*':' ');
    TextOut( hdc, left, top, str, strlen(str) );
    top += sLineHeight;
    sprintf(str, "Active set %d of %d", aeSequence_ActiveSet_GetIndex(sequence)+1, aeSequence_ActiveSet_GetCount(sequence));
    TextOut( hdc, left, top, str, strlen(str) );
    top += sLineHeight;
    sprintf(str, "Active %d of %d items", aeSequence_ActiveSet_GetCountItems(sequence), aeSequence_GetCount(sequence));
    TextOut( hdc, left, top, str, strlen(str) );
    top += sLineHeight;

}



/*======================================================================

    Animation List Pane
    
======================================================================*/
void aeAnimationListPane_Draw(aePane_t* pane, void* data, void* context);
void aeAnimationListPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* windowdata, void* context, int xPos, int yPos);

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeAnimationListPane_ConfigFunc(aePane_t* pane)
{
    pane->drawFunc   = aeAnimationListPane_Draw;
    pane->mouseFunc  = aeAnimationListPane_MouseEvt;

	pane->minHeight = 600 /* AEEDITORWND_ANIMINFO_HEIGHT_MIN */;
	pane->minWidth  = 200 /* AEEDITORWND_ANIMINFO_WIDTH_MIN */;

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeAnimationListPane_Draw(aePane_t* pane, void* data, void* context)
{
	HDC         hdc = (HDC)context;
    char        str[200];
	int		    i;
	aeRect_t    r;
    char		changed;
    char        opened;
    char        active;
    int         variant;
    int         variantCount;
    int         selectedIndex;
    int         animatedIndex;
    int         paneHeight;

    aeSequence_t*			sequence = NULL;
    const char*             seqname;
    char                    info[20];

    int                     labelSpace = 10;
    int                     left, top;

    AEASSERT(context);
    sequence = (aeSequence_t*)data;

    if (sequence != NULL)
    {
        selectedIndex = aeSequence_Selected_GetIndex(sequence);
        animatedIndex = aeSequence_Animated_GetIndex(sequence);

        paneHeight = pane->loc.bottom - pane->loc.top;

        for (i = 0; i < aeSequence_GetCount(sequence); i++)
        {
            r = aeSequenceWnd_CalcItemRect(i, paneHeight);
            left = pane->loc.left + r.left;
            top  = pane->loc.top  + r.top;

            #if 0
            r.left += pane->loc.left;
            r.right += pane->loc.left;
            r.top  += pane->loc.top;
            r.bottom += pane->loc.top;
            aegfxFrameRect(hdc, &r, AECOLOR_BLACK);
            #endif

            seqname = aeSequence_Anim_GetName(sequence, i);
            changed = aeSequence_Anim_IsChanged(sequence, i);
            opened  = aeSequence_Anim_IsOpened(sequence, i);
            active  = aeSequence_Anim_IsActive(sequence, i);
            variant = aeSequence_Anim_GetVariant(sequence, i);
            variantCount = aeSequence_Anim_GetVariantCount(sequence, i);

            SetTextColor(hdc, active ? RGB(0x00, 0x00, 0x00) : RGB(0xc0, 0xc0, 0xc0));

            left += 2;
            top  += 1;

            /* Changed flag */
            strcpy(info, changed ? "*" : " ");
            TextOut( hdc, left, top, info, strlen(info));
            left += labelSpace;

            /* Opened flag */
            strcpy(info, opened ? "O" : "  ");
            TextOut( hdc, left, top, info, strlen(info));
            left += labelSpace;

            /* Animated */
            strcpy(info, (animatedIndex == i) ? "-" : "  ");
            TextOut( hdc, left, top, info, strlen(info));
            left += labelSpace;

            /* Name */
            if (variantCount == 1)
                sprintf(str, "%s %c               ", seqname, (selectedIndex == i) ? '<' : ' ');
            else
                sprintf(str, "%s.%d %c               ", seqname, variant+1, (selectedIndex == i) ? '<' : ' ');

            TextOut( hdc, left, top, str, strlen(str));
        }

    }

}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeAnimationListPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* windowdata, void* context, int xPos, int yPos)
{
    int             i;
    int             editIndex;
    aeSequence_t*   sequence;
    HWND            hWnd = (HWND)context;
    aeRect_t        r;
    int             paneHeight;

    sequence = (aeSequence_t*)windowdata;

    if (sequence != NULL && mouseEvt == aeMouseEvt_eDown)
    {
        paneHeight = pane->loc.bottom - pane->loc.top;

        editIndex = -1;
        for (i = 0; i < aeSequence_GetCount(sequence) && editIndex == -1; i++)
        {
            r = aeSequenceWnd_CalcItemRect(i, paneHeight);
            r.left   += pane->loc.left;
            r.right  += pane->loc.left;
            r.top    += pane->loc.top;
            r.bottom += pane->loc.top;
            if (r.left <= xPos && xPos <= r.right && r.top <= yPos && yPos <= r.bottom)
                editIndex = i;
        }
        if (editIndex != -1)
        {
            aeSequenceWnd_HandleAnimClick(hWnd, sequence, editIndex);
        }
    }
}

