/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aetoolbarpane.c
 *
 *	
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aetoolbarpane.h"
#include "aearch.h"
#include "aeplatform.h"
#include "aedebug.h"
#include "aegraphics.h"

#include <windows.h>

#define AETOOLBAR_HEIGHT      24
#define AETOOLBAR_TOOL_WIDTH  22
#define AETOOLBAR_TOOL_SPACE   0


/*======================================================================

    Toolbar Pane
    
======================================================================*/
void aeToolbarPane_Draw(aePane_t* pane, void* data, void* context);
void aeToolbarPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* data, void* context, int xPos, int yPos);



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static void aeToolbarPane_CalcRect(aePane_t* pane, int index, aeRect_t* toolRect)
{
	aeRect_t	r;
	

    r.top    = pane->loc.top;
    r.left   = pane->loc.left + (AETOOLBAR_TOOL_WIDTH + AETOOLBAR_TOOL_SPACE) * index;
    r.bottom = pane->loc.bottom;
    r.right  = r.left + AETOOLBAR_TOOL_WIDTH;
    
    *toolRect = r;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/

static int aeToolbarPane_FindRect(aePane_t* pane, int xPos, int yPos)
{
	int	result = -1;
	int	i;
	aeRect_t r;

    aeToolbarPaneData_t* data;

    data = (aeToolbarPaneData_t*)pane->paneData;

	for (i = 0; data && i < data->count && result == -1; i++)
	{
		aeToolbarPane_CalcRect(pane, i, &r);
		
		if (aeRect_LocInRect(&r, xPos, yPos))
			result = i;
	}
	
	return result;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeToolbarPane_InitData(aePane_t* pane, aeToolbarPaneData_t* toolbarData)
{
    pane->paneData = toolbarData;
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeToolbarPane_ConfigFunc(aePane_t* pane)
{
    int count;
    aeToolbarPaneData_t* data;

    data = (aeToolbarPaneData_t*)pane->paneData;

    AEASSERT(data);

    count = (data) ? data->count : 2; /* 2: just in case data is NULL */


    pane->drawFunc  = aeToolbarPane_Draw;
    pane->mouseFunc = aeToolbarPane_MouseEvt;

	pane->minHeight = AETOOLBAR_HEIGHT;
	pane->minWidth  = count * (AETOOLBAR_TOOL_WIDTH + AETOOLBAR_TOOL_SPACE);

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeToolbarPane_Draw(aePane_t* pane, void* windowdata, void* context)
{
	HDC     hdc = (HDC)context;
    aeRect_t r;
    int		i;
    HFONT   oldFont;
    aeToolbarPaneData_t* data;

    data = (aeToolbarPaneData_t*)pane->paneData;
    AEASSERT(data);

    /* Can't do this until I can text on a colored background */
    /* aegfxFillRect(context, &pane->loc, 0xc0c0c0); */

    oldFont = SelectObject(hdc, aeWinPlat_GetNormalFont());

    for (i = 0; data && i < data->count; i++)
    {
	    aeToolbarPane_CalcRect(pane, i, &r);

        aegfxFrameRect(context, &r, AECOLOR_BLACK);

        aegfxDrawString(hdc, r.left+ 5, r.top + 5, data->list[i].desc, AECOLOR_BLACK);
    }

    SelectObject(hdc, oldFont);

}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeToolbarPane_MouseEvt(aePane_t* pane, aeMouseEvt_t mouseEvt, int modifiers, void* windowdata, void* context, int xPos, int yPos)
{
    int                     toolIndex;
    aeCommand_t             cmd;
    aeToolbarPaneData_t*    toolbardata;

    toolbardata = (aeToolbarPaneData_t*)pane->paneData;
    AEASSERT(toolbardata);

    if (mouseEvt == aeMouseEvt_eDown)
    {
        toolIndex = aeToolbarPane_FindRect(pane, xPos, yPos);
        
        if (toolIndex != -1)
        {
            cmd = toolbardata->list[toolIndex].cmd;

            (toolbardata->handlerProc)(windowdata, pane->windowContext, cmd);
        }
    }
}


