/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  animator.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include <windows.h>
#include "animator.h"
#include "aemainwnd.h"
#include "aeeditorwnd.h"
#include "aesequencewnd.h"
#include "aeplatform.h"
#include "asig.h"
#include "aearch.h"
#include "aeanimation.h"


#include "aefivesquare.h"
#include "aefiveseven.h"
#include "aefivesevenpic.h"
#include "aeslak.h"
#include "aeslakcircletwocolor.h"
#include "aehexagon.h"
#include "aepov8.h"
#include "aepov12.h"
#include "aepov8slakpic.h"
#include "aeeightsquare.h"
#include "aeeightsquaremax7219testpic.h"
#include "aefivesquarepumpkin.h"

static void ProcessCommandLine(char* cmdLine);

static void RegisterPlatforms(void);

/* Win32 entry-point routine */
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPreInst, LPSTR lpszCmdLine, int nCmdShow)
{
	MSG lpMsg;
	aErr_t err;

    RegisterPlatforms();
	
	/* Init */
    aeWinPlat_Init(hInst);

    err = aeEditorWnd_Init();
    
    err = aeMainWnd_Init();

    err = aeSequenceWnd_Init();

    /* Create Initial Window */
    aeMainWin_Create();

    /* Process Command line */
    ProcessCommandLine(lpszCmdLine);

    /* Run the message loop */
	while( GetMessage( &lpMsg, NULL, 0, 0 ) )
	{
		TranslateMessage( &lpMsg );
		DispatchMessage( &lpMsg );
	}

	return lpMsg.wParam;
}


void ProcessCommandLine(char* cmdLine)
{

}


void RegisterPlatforms(void)
{
    /* FiveSquare: Arch, Layout, Codegen */
    aeFiveSquare_Arch_Register();
    aeFiveSquare_Layout_Register();
    aeFiveSquare_CodeGen_Register();

    aeFiveSquarePumpkin_Layout_Register();

    /* Hexagon: Arch, Layout */
    aeHexagon_Arch_Register();
    aeHexagon_Layout_Register();

    /* SLAK: Arch, CodeGen */
    aeSLAK_Arch_Register();
    aeSLAK_CodeGen_Register();
    aePOV8_SLAK_CodeGen_Register();

    /* SLAK: Layouts */
    aePOV8_Layout_Register();
    //aePOV12_Layout_Register();
    aeSLAK_CircleTwoColor_Layout_Register();

    //aeSLAK_NewDimension_Register();

    /* EightSquare: Arch, Layout, Codegen */
    aeEightSquare_Arch_Register();
    aeEightSquare_Layout_Register();
    aeEightSquare_CodeGen_Register();
    //aeEightSquareMax7219Test_CodeGen_Register();

    /* FiveSeven: Arch, Layout, Codegen */
    aeFiveSeven_Arch_Register();
    aeFiveSeven_Layout_Register();
    aeFiveSeven_CodeGen_Register();

}


