/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  asig.h
 *
 *  Animation Signatures
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef ASIG_H_
#define ASIG_H_
#include "astd.h"

/*--- clean-up -----*/

    #define ASIG_FIVESQUARE  0x35535155 /* '5SQU' */
    #define ASIG_SLAK        0x534C414B /* 'SLAK' */
    #define ASIG_CIRCLETWO   0x43495232 /* 'CIR2' */
    #define ASIG_HEXAGON     0x48455841 /* 'HEXA' */
    #define ASIG_POV8        0x504F5638 /* 'POV8' */
    #define ASIG_EIGHTSQUARE 0x38535155 /* '8SQU' */
    #define ASIG_POV8_SLAK_ASM      0x5056534D /* 'PVSM' */
    #define ASIG_FIVESQUARE_PICASM  0x3553514D /* '5SQM' */


/*------------------------------------------------------------------------------
 - Foundation Signatures
 ------------------------------------------------------------------------------*/
    #define ASIG_ANIMATION  0x414E494D /* 'ANIM' */
    #define ASIG_SEQUENCE   0x53455155 /* 'SEQU' */
    #define ASIG_UNKNOWN    0x3F3F3F3F /* '????' */


/*------------------------------------------------------------------------------
 - FiveSquare
 ------------------------------------------------------------------------------*/
    /* Architecture */
    #define ASIG_AR_FIVESQUARE          0x35535155 /* '5SQU' */
    
    /* Layout(s) */
    #define ASIG_LO_FIVESQUARE          0x35535155 /* '5SQU' */
    #define ASIG_LO_FIVESQUAREPUMPKIN   0x35535150 /* '5SQP' */
    
    /* Codegen(s) */
    #define ASIG_CG_FIVESQUARE_PICASM   0x3553514D /* '5SQM' */


/*------------------------------------------------------------------------------
 - FiveSeven
 ------------------------------------------------------------------------------*/
    /* Architecture */
    #define ASIG_AR_FIVESEVEN  0x35534556 /* '5SEV' */
    
    /* Layout(s) */
    #define ASIG_LO_FIVESEVEN  0x35534556 /* '5SQU' */
    
    /* Codegen(s) */
    #define ASIG_CG_FIVESEVEN_PICASM  0x3553564D /* '5SVM' */


/*------------------------------------------------------------------------------
 - SLAK
 ------------------------------------------------------------------------------*/
    /* Architecture */
    #define ASIG_AR_SLAK        0x534C414B /* 'SLAK' */
    
    /* Layout(s) */
    #define ASIG_LO_CIRCLETWO   0x43495232 /* 'CIR2' */
    #define ASIG_LO_POV8        0x504F5638 /* 'POV8' */
    #define ASIG_LO_POV12       0x50563132 /* 'PV12' */
    
    /* Codegen(s) */
    #define ASIG_CG_SLAK_ASM           0x534C414B /* 'SLAK' */
    #define ASIG_CG_POV8_SLAK_ASM      0x5056534D /* 'PVSM' */


/*------------------------------------------------------------------------------
 - Hexagon
 ------------------------------------------------------------------------------*/
    /* Architecture */
    #define ASIG_AR_HEXAGON     0x48455841 /* 'HEXA' */
    
    /* Layout(s) */
    #define ASIG_LO_HEXAGON     0x48455841 /* 'HEXA' */
    
    /* Codegen(s) */


/*------------------------------------------------------------------------------
 - EightSquare
 ------------------------------------------------------------------------------*/
    /* Architecture */
    #define ASIG_AR_EIGHTSQUARE 0x38535155 /* '8SQU' */
    
    /* Layout(s) */
    #define ASIG_LO_EIGHTSQUARE 0x38535155 /* '8SQU' */
    
    /* Codegen(s) */
    #define ASIG_CG_EIGHTSQUARE_7219TEST_ASM 0x38534D54 /* '8SMT' EightSquare Max7219 Test */
    #define ASIG_CG_EIGHTSQUARE_ASM          0x38535155 /* '8SQU' EightSquare */





#endif

