/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  astd.h
 *
 *	Animator Standard Include
 *
 *
 *
 *------------------------------------------------------------------------------*/

#ifndef ASTD_H_
#define ASTD_H_

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

typedef char aeBool_t;

typedef int aErr_t;

#define AERR_NO_ERROR           0
#define AERR_OUT_OF_MEMORY      1
#define AERR_GENERAL_ERROR      2
#define AERR_OS_ERROR           3
#define AERR_END_OF_FILE        10
#define AERR_FILE_NOT_FOUND     11
#define AERR_FILE_DATA_FORMAT   12
#define AERR_FILE_CREATE_FAILED 13
#define AERR_GIF_IMAGE_CREATE_FAILED 14

#define MAX_LED_FRAMES 100
#define BYTES_PER_FRAME	1
#define MAX_ANIM_FILE 200
#define MAX_ANIM_PATH 512

/* 32-bit signature */
typedef unsigned long aSignature_t;

#define ANIM_NULL_NAME "noname"

#define PATH_SEPARATOR_CHAR '\\'
#define SEQUENCE_UNTITLED_NAME "untitled"


/* Defines */

/* Maximum length of a name, such as animation, architecture, platform, etc. */
#define ANIM_NAME_LEN   64

/* Maximum length of a description, such as for an architecture, platform, codegen, etc. */
#define ANIM_DESC_LEN   256

/* Maximum line length for reading from a file */
#define MAX_LINE_LEN   256

/* Maximum length (includes null) of a suffix to add to an animation name to make it unique (if required) */
#define ANIM_NAME_UNIQUE_SUFFIX_LEN 5

/* Maximum number of variants (settings) an animation can support */
#define ANIM_MAX_VARIANTS   5

#define AECOLOR_UNKNOWN 0x00406080

typedef struct
{
    unsigned long   top;
    unsigned long   left;
    unsigned long   bottom;
    unsigned long   right;
}  aeRect_t;


/*------------------------------------------
    Color, RGB, Maps to platform color
------------------------------------------*/
typedef unsigned long aeColor_t;

/*------------------------------------------
    LED Value
    Range: 0...(2^bpp-1)
    Value of an individual LED.
------------------------------------------*/
typedef unsigned char aeLedValue_t;

/*------------------------------------------
    Element Value
    Value of an individual an element as
    defined by the platform. This is used
    by the platform to get and set a
    specific element color.
------------------------------------------*/
typedef unsigned long aeElementValue_t;

typedef enum
{
	aeChange_eIncrease = 0,
	aeChange_eDecrease
} aeChange_t;


typedef enum
{
	aeOrientation_eVertical = 0,
	aeOrientation_eHorizontal
} aeOrientation_t;


typedef enum
{
	aePosition_eCurrent = 0,
	aePosition_eFirst,
	aePosition_eLast,
	aePosition_ePrev,
	aePosition_eNext
} aePosition_t;

#endif /* ASTD_H_ */
