
/* FiveSquare */
#include "aefivesquare.h"

/* Layout */
static void arch_Layout_CreateAnim(void);
static void arch_Layout_CreateEdit(void);
static void arch_Layout_ResizeEdit(void);

/* I/O */
static void arch_IO_ReadLine(void);
static void arch_IO_WriteLine(void);

/* Data */
static void arch_Data_AppendFrame(void);
static void arch_Data_DeleteCurrFrame(void);
static void arch_Data_DuplicateCurrFrame(void);
static void arch_Data_ClearCurrFrame(void);

static void arch_Data_UpdateCurrFrameIndex(void);
static void arch_Data_SetFrameSelection(void);

static void arch_Data_UpdateLedValue(void);
static void arch_Data_GetLedValue(void);

static void arch_Animation_GetLedValue(void);
static void arch_Animation_Advance(void);
static void arch_Animation_Reset(void);



void aeFiveSquare_SetupArch(aeArchitecture_t* arch)
{
	memset(arch, 0, sizeof(aeArchitecture_t));

	
	arch->layoutCreateFunc;
	arch->layoutCreateEdit;
	arch->layoutResizeEdit;

	arch->ioReadLine;
	arch->ioWriteLine;

	arch->dataAppendFrame;
	arch->dataDeleteCurrFrame;
	arch->dataDuplicateCurrFrame;
	arch->dataClearCurrFrame;

	arch->dataUpdateCurrFrameIndex;
	arch->dataSetFrameSelection;

	arch->dataUpdateLedValue;
	arch->dataGetLedValue;

	arch->animGetLedValue;
	arch->animAdvance;
	arch->animReset;
}




/* Layout */
void arch_Layout_CreateAnim(void)
{
}

void arch_Layout_CreateEdit(void)
{
}

void arch_Layout_ResizeEdit(void)
{
}


/* I/O */
void arch_IO_ReadLine(void)
{
}

void arch_IO_WriteLine(void)
{
}


/* Data */
void arch_Data_AppendFrame(void)
{
}

void arch_Data_DeleteCurrFrame(void)
{
}

void arch_Data_DuplicateCurrFrame(void)
{
}

void arch_Data_ClearCurrFrame(void)
{
}


void arch_Data_UpdateCurrFrameIndex(void)
{
}

void arch_Data_SetFrameSelection(void)
{
}


void arch_Data_UpdateLedValue(void)
{
}

void arch_Data_GetLedValue(void)
{
}


void arch_Animation_GetLedValue(void)
{
}

void arch_Animation_Advance(void)
{
}

void arch_Animation_Reset(void)
{
}





void ComputeLayout(EditWndLedLayout_t* evd, int locX, int locY, int ledSize, int ledSpace)
{
	int	row, col;
	
	LedElement_t* le;
	
	
	evd->LedCount = LED_ELEMENTS;
	
	for (row = 0; row < 5; row++)
	{
		for (col = 0; col < 5; col++)
		{
			le = &evd->LedList[row * 5 + col];
			
			le->x = locX + (4 - col) * (ledSize + ledSpace);
			le->y = locY + row * (ledSize + ledSpace);
			le->width  = ledSize;
			le->height = ledSize;
		}
	}
}



static BOOL GetLedState(EditWndData_t* data, int frameIndex, int ledIndex)
{
    unsigned long   frame;
    unsigned long   mask;

    frame = data->frames[frameIndex];

    mask = (1 << ledIndex);

    return (frame & mask) != 0;
}

static void SetLedState(EditWndData_t* data, int frameIndex, int ledIndex, BOOL state)
{
    unsigned long   frame;
    unsigned long   mask;

    frame = data->frames[frameIndex];

    mask = (1 << ledIndex);

    if (state)
        frame = (frame | mask);
    else
        frame = (frame & ~mask);

    data->frames[frameIndex] = frame;
}



static void GetPatternStr(unsigned long pattern, int row, char* str, int strlen)
{
    int patIndex;
    int strIndex;
    int setBit;
    int i;

    if (str != NULL && strlen > 5)
    {
        strIndex = 4;
        patIndex = row * 5;

        for (i = 0; i < 5; i++)
        {
            setBit = (pattern & (1 << patIndex)) != 0;
    
            str[strIndex] = setBit ? '@' : '.';

            strIndex--;
            patIndex++;
        }
        
        str[5] = 0;
    }
}


/*
;==========	RowTest
			FrameByFrameAnim RowTest, 0x04, 0x80, RowTestData

RowTestData
			IdxJump	RowTestData, DATAINDEX
			retlw	0x1f		; 0
			retlw	0x00
			retlw	0x00
			retlw	0x00
			retlw	0xe0		; 1
			retlw	0x03
			retlw	0x00
			retlw	0x00
			retlw	0x00		; 2
			retlw	0x7c
			retlw	0x00
			retlw	0x00
			retlw	0x00		; 3
			retlw	0x80
			retlw	0x0f
			retlw	0x00
			retlw	0x00		; 4
			retlw	0x00
			retlw	0xf0
			retlw	0x01
			bsf		SEQFLAGS, FLAG_DATA_DONE
			return

*/


void FiveSquare_WriteLine(EditWndData_t* data, char* line, int lineLen, BOOL includeAssembly)
{
    char	str[200];
    char    datastr[20];
    char    signature[10];
    char	name[ANIM_NAME_LEN];
    char    pattern[10];
    unsigned long   frameData;

    /* name */
    strcpy(name, data->name);
    if (strlen(name) == 0)
        strcpy(name, ANIM_NULL_NAME);

    if (data->ioState == AnimIOState_eSig)
    {
        /* signature */
        SigToStr(ASIG_FIVESQUARE,signature,sizeof(signature));
        sprintf(line, "; %s\n", signature);

        data->ioState = AnimIOState_eName;
    }

    else if (data->ioState == AnimIOState_eName)
    {
        sprintf(line, "; %s\n", name);

        data->ioState = AnimIOState_eDetails;
    }

    else if (data->ioState == AnimIOState_eDetails)
    {
        /* speed, repeat, and count */
        sprintf(line, "; %d %d %d\n", data->speed, data->repeat, data->frameCount);

        data->ioState = AnimIOState_eFrames;
        data->ioCount = 0;
    }
    else if (data->ioState == AnimIOState_eFrames)
    {
        int nextSix = data->ioCount + 6;
        /* frames */
        strcpy(line, ";");

        while (data->ioCount < data->frameCount && data->ioCount < nextSix)
        {
            sprintf(datastr, " %8.8x", data->frames[data->ioCount]);
            strcat(line, datastr);
            data->ioCount++;
        }

        strcat(line, "\n");

        if (data->ioCount == data->frameCount)
        {
            if (includeAssembly)
            {
                data->ioState = AnimIOState_eAssemblyHeader;
                data->ioCount = 0;
            }
            else
                data->ioState = AnimIOState_eDone;
        }
    }
    else if (data->ioState == AnimIOState_eAssemblyHeader)
    {
        line[0] = 0;

        sprintf(str, ";========== %s (%d frames)\n", name, data->frameCount);
        strcat(line, str);

        sprintf(str, "\t\t\tFrameByFrameAnim %s, %d, 0x%2.2x, %sData\n", name, data->repeat, data->speed, name);
        strcat(line, str);

        sprintf(str, "%sData\n",name);
        strcat(line, str);

        sprintf(str, "\t\t\tIdxJump %sData, DATAINDEX\n",name);
        strcat(line, str);

        data->ioState = AnimIOState_eAssemblyBody;

    }
    else if (data->ioState == AnimIOState_eAssemblyBody)
    {
        line[0] = 0;
        frameData = data->frames[data->ioCount];

        val = ((frameData >>  0) & 0xff);
        GetPatternStr(frameData, 0, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        val = ((frameData >>  8) & 0xff);
        GetPatternStr(frameData, 1, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        val = ((frameData >> 16) & 0xff);
        GetPatternStr(frameData, 2, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        val = ((frameData >> 24) & 0xff);
        GetPatternStr(frameData, 3, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        GetPatternStr(frameData, 4, pattern, sizeof(pattern));
        sprintf(str, "\t\t\t\t\t\t\t\t; %s\n", pattern);
        strcat(line, str);

        strcat(line, "\n");

        data->ioCount++;
        
        if (data->ioCount == data->frameCount)
            data->ioState = AnimIOState_eAssemblyTrailer;

    }
    else if (data->ioState == AnimIOState_eAssemblyTrailer)
    {
        line[0] = 0;

        sprintf(str, "\t\t\tbsf\t\tSEQFLAGS, FLAG_DATA_DONE\n");
        strcat(line, str);

        sprintf(str, "\t\t\treturn\n");
        strcat(line, str);

        data->ioState = AnimIOState_eDone;
    }
    else
    {
        /* ERROR */
    }
}

void Read5SQULine(EditWndData_t* data, char* line, BOOL searchEndForEndOfAssembly)
{
    char	str[200];
    aSignature_t    sig;
    int     speed;
    int     count;
    int     repeat;
    unsigned long frame0, frame1, frame2, frame3, frame4, frame5;

	if (data->ioState == AnimIOState_eSig)
	{
		/* signature */
		sscanf(line, "; %s\n", str);
		sig = StrToSig(str, strlen(str));

		data->ioState = AnimIOState_eName;
	}

	else if (data->ioState == AnimIOState_eName)
	{
		/* name */
		sscanf(line, "; %s\n", str);
		strncpy(data->name, str, sizeof(data->name));
		data->name[ANIM_NAME_LEN-1] = 0;

		data->ioState = AnimIOState_eDetails;
	}

	else if (data->ioState == AnimIOState_eDetails)
	{
		/* speed, repeat, and count */
		sscanf(line, "; %d %d %d\n", &speed, &repeat, &count);
		data->speed = speed;
		data->repeat = repeat;
		data->frameCount = count;

		data->sAnimFrameIndex = 0;
		data->sEditFrameIndex = 0;
        data->ioCount = 0;

		data->ioState = AnimIOState_eFrames;
	}

	else if (data->ioState == AnimIOState_eFrames)
	{

		/* frames */
		sscanf(line, "; %x %x %x %x %x %x", &frame0, &frame1, &frame2, &frame3, &frame4, &frame5);

		/* Use ioCount to track loaded frames */
		data->frames[data->ioCount++] = frame0;

		if (data->ioCount < data->frameCount)
			data->frames[data->ioCount++] = frame1;
		 
		if (data->ioCount < data->frameCount)
			data->frames[data->ioCount++] = frame2;
		 
		if (data->ioCount < data->frameCount)
			data->frames[data->ioCount++] = frame3;

		if (data->ioCount < data->frameCount)
			data->frames[data->ioCount++] = frame4;

		if (data->ioCount < data->frameCount)
			data->frames[data->ioCount++] = frame5;

		if (data->ioCount == data->frameCount)
        {
            if (searchEndForEndOfAssembly)
                data->ioState = AnimIOState_eAssembly;
            else
                data->ioState = AnimIOState_eDone;

        }
	}
    else if (data->ioState == AnimIOState_eAssembly)
    {
        if (strstr(line, "return") != NULL)
            data->ioState = AnimIOState_eDone;
    }
	else
	{
        /* ERROR */
	}
	            
            
}

void DoDeleteFrame(EditWndData_t* data)
{
	int	deletedIndex;
	int	i;
		
	/* Only Allow delete if more than one frame */
	if (data->frameCount > 1)
	{
		deletedIndex = data->sEditFrameIndex;
		
		/* Copy down the frames */
		for (i = deletedIndex; i < data->frameCount - 1; i++)
		{
			data->frames[i] = data->frames[i + 1];
		}
		
		/* Clear the old data so we don't see it again */
		data->frames[data->frameCount - 1] = 0;
		
		/* Adjust frame count */
		data->frameCount--;
		
		/* Keep sEditFrameIndex in range */
		if (data->sEditFrameIndex >= data->frameCount)
			data->sEditFrameIndex = data->frameCount-1;
		/* Keep sAnimFrameIndex in range */
		if (data->sAnimFrameIndex >= data->frameCount)
			data->sAnimFrameIndex = data->frameCount-1;
	}
	/* If one frame, then clear it */
	else
	{
		data->frames[0] = 0;
		
	}
    SetChangedFlag(data);
}

void DoClearFrame(EditWndData_t* data)
{
	int	index;
	
	index = data->sEditFrameIndex;
	
	data->frames[index] = 0;
    SetChangedFlag(data);
}

void DoDuplicateFrame(EditWndData_t* data)
{
	int	duplicatedIndex;
	int i;
	
	/* Only Allow duplicate if there is room */
	if (data->frameCount < MAX_LED_FRAMES - 1)
	{
		duplicatedIndex = data->sEditFrameIndex;
		
		/* Increase frame count */
		data->frameCount++;

		for (i = data->frameCount - 2; i >= duplicatedIndex; i--)
		{
			data->frames[i+1] = data->frames[i];
		}
		
		
		/* Point to copied frame, not original */
		data->sEditFrameIndex++;
		
	}
    SetChangedFlag(data);
}

void DoAddFrame(EditWndData_t* data)
{
	data->frameCount++;
	data->sEditFrameIndex = data->frameCount - 1;
	
    SetChangedFlag(data);
}



EditWndData_t* FiveSquare_CreateCopy(EditWndData_t* animData)
{
    EditWndData_t* data;

    data = FiveSquare_CreateNewData();

    if (data != NULL)
    {
        *data = *(EditWndData_t*)animData;

        data->changed = FALSE;
        data->sEditFrameIndex = 0;
    }

    return data;
}

EditWndData_t* FiveSquare_CreateNewData(void)
{
    EditWndData_t* data;
    data = (EditWndData_t*)malloc(sizeof(EditWndData_t));

    if (data != NULL)
    {
        memset((char*)data,0,sizeof(EditWndData_t));
        data->frameCount = 1;
        data->repeat = 4;
        data->speed = 0x80;
        data->belongsToSequence = FALSE;
    }

    return data;
}

