/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeeightsquaremax7219testpic.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aeeightsquaremax7219testpic.h"
#include "aearchlist.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aedebug.h"
#include <stdio.h>

static aErr_t aeEightSquareMax7219Test_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref);
static void   aeEightSquare_GetPatternStr(unsigned long pattern, char* str, int strlen);
static void   aeEightSquareMax7219Test_WriteAnimPicAssembly_CodeRoutineBody(aeAnimationData_t* animData, aeCodeGen_t* codegen, FILE* fileref);
/*static void   aeEightSquareMax7219Test_WriteAnimPicAssembly(aeAnimationData_t* data, aeCodeGen_t* codegen, char* line, int lineLen); */


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEightSquareMax7219Test_CodeGen_Register(void)
{
    aeCodeGen_t codegen;

    memset(&codegen, 0, sizeof(aeCodeGen_t));

    codegen.codegenSignature = ASIG_CG_EIGHTSQUARE_7219TEST_ASM;
	codegen.archSignature    = ASIG_AR_EIGHTSQUARE;
    codegen.codegenfunc      = aeEightSquareMax7219Test_WriteCode;

    codegen.requiresUniqueNames = FALSE;

    aeArchList_CodeGen_Add(&codegen);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeEightSquareMax7219Test_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref)
{
    aErr_t  err = AERR_NO_ERROR;
    char    index;

    aeAnimationData_t* animData;

    animData = aeSequence_Anim_Get(sequence, 0);

    /*------------------------*/
    /* Animation Routines (tables) */
    /*------------------------*/
    /* Loop over animations */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines with animation data that are needed by either this animation */
        /* or a referenced one */
        if (aeSequence_Anim_HasNeededAnimData(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            aeEightSquareMax7219Test_WriteAnimPicAssembly_CodeRoutineBody(animData, codegen, fileref);
        }
    }

    
    return err;
}



void aeEightSquareMax7219Test_WriteAnimPicAssembly_CodeRoutineBody(aeAnimationData_t* animData, aeCodeGen_t* codegen, FILE* fileref)
{
    unsigned char   framePtr[8];
    unsigned        frameIndex;
    unsigned        row;
    char            pattern[20];
    char            name[ANIM_NAME_LEN];

    fprintf(fileref, "\n");
    fprintf(fileref, ";\t%s\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));

    frameIndex = 0;

    for (frameIndex = 0; frameIndex < aeAnimation_GetFrameCount(animData); frameIndex++)
    {
        aeAnimation_GetFrameDataBuffer(animData, frameIndex, framePtr, sizeof(framePtr));

        /* MAX7219Cmd  MAX7219_ADDR_DIGIT_0+0, 0x01 */
        for (row = 0; row < 8; row++)
        {
            aeEightSquare_GetPatternStr(framePtr[row], pattern, sizeof(pattern));
            fprintf(fileref, "\t\t\tMAX7219Cmd	MAX7219_ADDR_DIGIT_0+%d, 0x%2.2x\t\t; %s\n", row, framePtr[row], pattern); /* MAX7219Cmd  MAX7219_ADDR_DIGIT_0+0, 0x01 */
        }

        fprintf(fileref, "\t\t\tcall\tLongDelay\n");

    }
    fprintf(fileref, "\t\t\tcall\tLongDelay\n");
    fprintf(fileref, "\t\t\tcall\tLongDelay\n");

}

void aeEightSquare_GetPatternStr(unsigned long pattern, char* str, int strlen)
{
    int patIndex;
    int strIndex;
    int setBit;
    int i;

    if (str != NULL && strlen > 8)
    {
        strIndex = 7;
        patIndex = 0;

        for (i = 0; i < 8; i++)
        {
            setBit = (pattern & (1 << patIndex)) != 0;
    
            str[strIndex] = setBit ? '@' : '.';

            strIndex--;
            patIndex++;
        }
        
        str[8] = 0;
    }
}




#if 0
void aeEightSquareMax7219Test_WriteAnimPicAssembly(aeAnimationData_t* animData, aeCodeGen_t* codegen, char* line, int lineLen)
{
    char	str[200];
    char	name[ANIM_NAME_LEN];
    char    pattern[20];
    unsigned   frameData;
    unsigned   val;
    AnimIOState_t ioState;
    aeAnimSubtype_t subtype;
    
    subtype = aeAnimation_GetSubtype(animData);
    ioState = aeAnimation_IO_GetState(animData);
    

    if (ioState == AnimIOState_eCodeFileHeader)
    {
        line[0] = 0;
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, ";===============================================\n");
        strcat(line, "; POV8 Sequence Data\n");
        strcat(line, ";\n");
        strcat(line, ";\n");
        strcat(line, "; David D. Kaufman\n");
        strcat(line, ";===============================================\n");
        strcat(line, "\n");
        strcat(line, "#define LED_COUNT 8\n");
        if (codegen->speedReduction != 0)
        {
            sprintf(str, "#define SPEED_REDUCTION %d\n", codegen->speedReduction);
            strcat(line, str);
        }
        strcat(line, "#include \"..\\ComponentLibrary\\LEDDirectDrive_Animator.asm\"\n");
        strcat(line, "\n");
        strcat(line, "\t\t\torg 0x100\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);

    }
    else if (ioState == AnimIOState_eCodeDispatchHeader)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Dispatch\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "AnimationDispatch\n");
        strcat(line, "\t\t\tIdxJump AnimationDispatch, CURSEQ\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeDispatchEntry)
    {
        aeAnimation_GetNameVariant(animData, name, sizeof(name));
        sprintf(line, "\t\t\tgoto %s\n", name);

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);
    }
    else if (ioState == AnimIOState_eCodeDispatchTrailer)
    {
        line[0] = 0;
        strcat(line, "\t\t\tbsf		SEQFLAGS,FLAG_LIST_DONE\n");
        strcat(line, "\t\t\treturn\n");
        strcat(line, "\n");
        strcat(line, "\n");

        aeAnimation_IO_Count_Reset(animData);
        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineHeader);
    }
    else if (ioState == AnimIOState_eCodeRoutineIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Entry Points\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeRoutineHeader)
    {
        line[0] = 0;
        aeAnimation_GetNameVariant(animData, name, sizeof(name));

            /*
            (tab x 3)FrameByFrameAnim Thirds, DefaultBehavior, 0x04, 0x80, 0x80, ThirdsData
            */
            sprintf(str, "\t\t\t; %s (%d frames)\n", aeAnimation_GetNameUnique(animData, name, sizeof(name)), aeAnimation_GetFrameCount(animData));
            strcat(line, str);
    
            sprintf(str, "\t\t\tFrameByFrameAnim %s, OneBytePatterns, %d, NoTransition, 0x%2.2x, %sData\n", name, aeAnimation_GetRepeat(animData), aeAnimation_GetSpeed(animData), aeAnimation_GetName(animData));
            strcat(line, str);

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);
        aeAnimation_IO_Count_Reset(animData);

    }
    else if (ioState == AnimIOState_eCodeTableIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Tables\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);

    }
    else if (ioState == AnimIOState_eCodeRoutineBody)
    {
        line[0] = 0;
        frameData = aeAnimation_GetFrameData(animData, aeAnimation_IO_Count_Get(animData));
        
        if (aeAnimation_IO_Count_Get(animData) == 0)
        {
            strcat(line, "\n");
            sprintf(str, "%sData\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
            sprintf(str, "\t\t\tIdxLongJump %sData, DATAINDEX\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
        }

        val = (frameData & 0xff);
        aePOV8_GetPatternStr(frameData, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        aeAnimation_IO_Count_Incr(animData);

        if (aeAnimation_IO_Count_Get(animData) == aeAnimation_GetFrameCount(animData))
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineTrailer);

    }
    else if (ioState == AnimIOState_eCodeRoutineTrailer)
    {
        line[0] = 0;

        sprintf(str, "\t\t\tbsf\t\tSEQFLAGS, FLAG_DATA_DONE\n");
        strcat(line, str);

        sprintf(str, "\t\t\treturn\n");
        strcat(line, str);

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else if (ioState == AnimIOState_eCodeFileTrailer)  
    {
        line[0] = 0;
        strcat(line, "\n");
		strcat(line, "\t\t\tend\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else
    {
        AEASSERTED("aePOV8_SLAK_WriteAnimPicAssembly: Unknown iostate");
    }
}
#endif

