/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aeeightsquaremax7219testpic.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aeeightsquarepic.h"
#include "aearchlist.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aedebug.h"
#include <stdio.h>

static aErr_t aeEightSquare_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref);
static void aeEightSquare_WriteCodeFileHeader(FILE* fileref);
static void aeEightSquare_WriteCodeDispatchHeader(FILE* fileref);
static void aeEightSquare_WriteCodeDispatchEntry(aeAnimationData_t* animData, FILE* fileref);
static void aeEightSquare_WriteCodeDispatchTrailer(FILE* fileref);
static void aeEightSquare_WriteCodeRoutineIntro(FILE* fileref);
static void aeEightSquare_WriteCodeRoutineEntryPoint(aeAnimationData_t* animData, FILE* fileref);
static void aeEightSquare_WriteCodeTableIntro(FILE* fileref);
static void aeEightSquare_WriteCodeFileTrailer(FILE* fileref);

static void   aeEightSquare_GetPatternStr(unsigned long pattern, char* str, int strlen);
static void   aeEightSquare_WriteCodeRoutineBody(aeAnimationData_t* animData, aeCodeGen_t* codegen, FILE* fileref);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeEightSquare_CodeGen_Register(void)
{
    aeCodeGen_t codegen;

    memset(&codegen, 0, sizeof(aeCodeGen_t));

    codegen.codegenSignature = ASIG_CG_EIGHTSQUARE_ASM;
	codegen.archSignature    = ASIG_AR_EIGHTSQUARE;
    codegen.codegenfunc      = aeEightSquare_WriteCode;

    codegen.requiresUniqueNames = TRUE;

    aeArchList_CodeGen_Add(&codegen);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeEightSquare_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref)
{
    aErr_t  err = AERR_NO_ERROR;
    char    index;
    int     variant;

    aeAnimationData_t* animData;

    /*------------------------*/
    /* Code File Header       */
    /*------------------------*/
    aeEightSquare_WriteCodeFileHeader(fileref);

    /*------------------------*/
    /* Dispatch Header        */
    /*------------------------*/
    aeEightSquare_WriteCodeDispatchHeader(fileref);

    /*------------------------*/
    /* Dispatch Entries       */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write dispatch entries for active animations */
        if (aeSequence_Anim_IsActive(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);

            /* Get the line and write it to the file */
            aeEightSquare_WriteCodeDispatchEntry(animData, fileref);
        }
    }

    /*------------------------*/
    /* Dispatch Trailer       */
    /*------------------------*/
    aeEightSquare_WriteCodeDispatchTrailer(fileref);

    /*------------------------*/
    /* Code Intro             */
    /*------------------------*/
    aeEightSquare_WriteCodeRoutineIntro(fileref);


    /*------------------------*/
    /* Animation Entry Points */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines and only the first time they are encountered */
        if (aeSequence_Anim_IsFirstActiveOccurance(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);

            /* Tell the animation we want to write the routine header */
            aeEightSquare_WriteCodeRoutineEntryPoint(animData, fileref);
        }
    }

    /*------------------------*/
    /* Table Intro             */
    /*------------------------*/
    aeEightSquare_WriteCodeTableIntro(fileref);

    /*------------------------*/
    /* Animation Routines (tables) */
    /*------------------------*/
    /* Loop over animations */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write animation data that is needed (either by this animation or another one that references it) */
        if (aeSequence_Anim_HasNeededAnimData(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            aeEightSquare_WriteCodeRoutineBody(animData, codegen, fileref);
        }
    }

    /*------------------------*/
    /* Code File Trailer      */
    /*------------------------*/
    aeEightSquare_WriteCodeFileTrailer(fileref);
    
    return err;
}


void aeEightSquare_WriteCodeFileHeader(FILE* fileref)
{
    fprintf(fileref, "\n");
    fprintf(fileref, "\n");
    fprintf(fileref, "\n");
    fprintf(fileref, "\n");
    fprintf(fileref,";===============================================\n");
    fprintf(fileref,"; EightSquare Sequence Data\n");
    fprintf(fileref,";\n");
    fprintf(fileref,";\n");
    fprintf(fileref,"; David D. Kaufman\n");
    fprintf(fileref,";===============================================\n");
    fprintf(fileref, "\n");

    #if 0
    if (codegen->speedReduction != 0)
    {
        sprintf(str, "#define SPEED_REDUCTION %d\n", codegen->speedReduction);
        strcat(line, str);
    }
    #endif
    fprintf(fileref, "#include \"EightSquareLibrary.asm\"\n");

    fprintf(fileref, "\n");
    fprintf(fileref, "\n");

}

void aeEightSquare_WriteCodeDispatchHeader(FILE* fileref)
{
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "\t\t;	Animation Dispatch\n");
    fprintf(fileref, "\t\t;\n");
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "AnimationDispatch\n");
    fprintf(fileref, "\t\t\tIdxJump AnimationDispatch, CURSEQ\n");

}

void aeEightSquare_WriteCodeDispatchEntry(aeAnimationData_t* animData, FILE* fileref)
{
    char	name[ANIM_NAME_LEN];
    aeAnimation_GetNameVariant(animData, name, sizeof(name));
    fprintf(fileref, "\t\t\tgoto %s\n", name);


}

void aeEightSquare_WriteCodeDispatchTrailer(FILE* fileref)
{
    fprintf(fileref, "\t\t\tbsf		SEQFLAGS,FLAG_LIST_DONE\n");
    fprintf(fileref, "\t\t\treturn\n");
    fprintf(fileref, "\n");
    fprintf(fileref, "\n");
}

void aeEightSquare_WriteCodeRoutineIntro(FILE* fileref)
{
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "\t\t;	Animation Entry Points\n");
    fprintf(fileref, "\t\t;\n");
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "\n");

}

void aeEightSquare_WriteCodeRoutineEntryPoint(aeAnimationData_t* animData, FILE* fileref)
{
    char	        name[ANIM_NAME_LEN];
    char	        varName[ANIM_NAME_LEN*2];
    char	        dataName[ANIM_NAME_LEN*2];
    char            stylestr[40];
    aeAnimSubtype_t subtype;

    aeAnimation_GetNameVariant(animData, name, sizeof(name));
    subtype = aeAnimation_GetSubtype(animData);
    aeAnimation_GetNameUniqueVariant(animData, varName, sizeof(varName));
    aeAnimation_GetNameUnique(animData, dataName, sizeof(dataName));

    if (subtype == aeAnimSubtype_eFrameByFrame)
    {
        /*
        (tab x 3)FrameByFrameAnim diagonal, 4, 0x80, diagonalData
        */
        fprintf(fileref, "\t\t\t; %s (%d frames)\n", 
                varName, 
                aeAnimation_GetFrameCount(animData));

        fprintf(fileref, "\t\t\tFrameByFrameAnim %s, %d, 0x%2.2x, %sData\n", 
                varName, 
                aeAnimation_GetRepeat(animData), 
                aeAnimation_GetSpeed(animData), 
                dataName);

    }
    else if (subtype == aeAnimSubtype_ePixelList)
    {
        if (aeAnimation_GetWrapStyle(animData) == aeAnimStyle_eSinglePass)
            strcpy(stylestr, "PixelListSinglePass");
        else
            strcpy(stylestr, "PixelListWrapStyle");
        /*
        (tab x 3)PixelListAnim SnakeFillOne, 	2, 0x20,  1, SnakeFillData, PixelListSinglePass
        (tab x 3)PixelListAnim SnakeFillLong,	3, 0x10, 16, SnakeFillData, PixelListSinglePass
        */        
        fprintf(fileref, "\t\t\t; %s (%d pixels)\n", 
                varName, 
                aeAnimation_GetFrameCount(animData));

        fprintf(fileref, "\t\t\tPixelListAnim %s, %d, 0x%2.2x, %d, %sData, %s\n", 
                varName, 
                aeAnimation_GetRepeat(animData), 
                aeAnimation_GetSpeed(animData),
                aeAnimation_GetRunLength(animData), 
                dataName, 
                stylestr);

    }
    else if (subtype == aeAnimSubtype_eStrip)
    {
        /*
        (tab x 3)StripAnim StripTest, 4, 0x80, 0, StripTestData, StripSinglePass | StripHorizontal
        */
        if (aeAnimation_GetWrapStyle(animData) == aeAnimStyle_eSinglePass)
            strcpy(stylestr, "PixelListSinglePass");
        else
            strcpy(stylestr, "PixelListWrapStyle");

        if (aeAnimation_GetDirection(animData) == aeAnimDirection_eHorizontal)
            strcat(stylestr, "| StripHorizontal");
        else
            strcat(stylestr, "| StripVertical");

        fprintf(fileref, "\t\t\t; %s (%d rows)\n", 
                varName, 
                aeAnimation_GetFrameCount(animData));

        fprintf(fileref, "\t\t\tStripAnim %s, %d, 0x%2.2x, 0, %sData, %s\n", 
                varName, 
                aeAnimation_GetRepeat(animData), 
                aeAnimation_GetSpeed(animData), 
                dataName, 
                stylestr);
    }
    else
    {
        AEASSERTED("aeEightSquare_WriteCodeRoutineEntryPoint: unknown subtype");
    }

    fprintf(fileref, "\n");
}

void aeEightSquare_WriteCodeTableIntro(FILE* fileref)
{
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "\t\t;	Animation Tables\n");
    fprintf(fileref, "\t\t;\n");
    fprintf(fileref, "\t\t;---------------------------------------------\n");
    fprintf(fileref, "\n");
}

void aeEightSquare_WriteCodeRoutineBody(aeAnimationData_t* animData, aeCodeGen_t* codegen, FILE* fileref)
{
    char	        name[ANIM_NAME_LEN];
    char            pattern[20];
    aeAnimSubtype_t subtype;
    unsigned char   framePtr[8];
    unsigned        frameIndex;
    unsigned        row;
    unsigned int    val;

    subtype = aeAnimation_GetSubtype(animData);

    fprintf(fileref, "\n");
    fprintf(fileref, "%sData\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
    fprintf(fileref, "\t\t\tIdxLongJump %sData, DATAINDEX\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));

    frameIndex = 0;

    for (frameIndex = 0; frameIndex < aeAnimation_GetFrameCount(animData); frameIndex++)
    {

        if (subtype == aeAnimSubtype_eFrameByFrame)
        {
            aeAnimation_GetFrameDataBuffer(animData, frameIndex, framePtr, sizeof(framePtr));
            /* MAX7219Cmd  MAX7219_ADDR_DIGIT_0+0, 0x01 */
            for (row = 0; row < 8; row++)
            {
                aeEightSquare_GetPatternStr(framePtr[row], pattern, sizeof(pattern));
                fprintf(fileref, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", framePtr[row], pattern);
            }
            fprintf(fileref, "\n");
        }
        else if (subtype == aeAnimSubtype_ePixelList)
        {
            val = aeAnimation_GetFrameData(animData, frameIndex);
            fprintf(fileref, "\t\t\tretlw\t0x%2.2x\n", val);
        }
        else if (subtype == aeAnimSubtype_eStrip)
        {
            val = aeAnimation_GetFrameData(animData, frameIndex);
            aeEightSquare_GetPatternStr(val, pattern, sizeof(pattern));
            fprintf(fileref, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        }


    }
    fprintf(fileref, "\t\t\tbsf\t\tSEQFLAGS, FLAG_DATA_DONE\n");
    fprintf(fileref, "\t\t\treturn\n");
    fprintf(fileref, "\n");
}

void aeEightSquare_WriteCodeFileTrailer(FILE* fileref)
{
    fprintf(fileref, "\n");
    fprintf(fileref, "\t\t\tend\n");
    fprintf(fileref, "\n");
}

#if 0
void aeEightSquare_WriteAnimPicAssembly_CodeRoutineBody(aeAnimationData_t* animData, aeCodeGen_t* codegen, FILE* fileref)
{
    unsigned char   framePtr[8];
    unsigned        frameIndex;
    unsigned        row;
    char            pattern[20];
    char            name[ANIM_NAME_LEN];

    fprintf(fileref, "\n");
    fprintf(fileref, ";\t%s\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));

    frameIndex = 0;

    for (frameIndex = 0; frameIndex < aeAnimation_GetFrameCount(animData); frameIndex++)
    {
        aeAnimation_GetFrameDataBuffer(animData, frameIndex, framePtr, sizeof(framePtr));

        /* MAX7219Cmd  MAX7219_ADDR_DIGIT_0+0, 0x01 */
        for (row = 0; row < 8; row++)
        {
            aeEightSquare_GetPatternStr(framePtr[row], pattern, sizeof(pattern));
            fprintf(fileref, "\t\t\tMAX7219Cmd	MAX7219_ADDR_DIGIT_0+%d, 0x%2.2x\t\t; %s\n", row, framePtr[row], pattern); /* MAX7219Cmd  MAX7219_ADDR_DIGIT_0+0, 0x01 */
        }

        fprintf(fileref, "\t\t\tcall\tLongDelay\n");

    }
    fprintf(fileref, "\t\t\tcall\tLongDelay\n");
    fprintf(fileref, "\t\t\tcall\tLongDelay\n");

}
#endif

void aeEightSquare_GetPatternStr(unsigned long pattern, char* str, int strlen)
{
    int patIndex;
    int strIndex;
    int setBit;
    int i;

    if (str != NULL && strlen > 8)
    {
        strIndex = 7;
        patIndex = 0;

        for (i = 0; i < 8; i++)
        {
            setBit = (pattern & (1 << patIndex)) != 0;
    
            str[strIndex] = setBit ? '@' : '.';

            strIndex--;
            patIndex++;
        }
        
        str[8] = 0;
    }
}



