/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aefiveseven.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aefiveseven.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aearchlist.h"
#include "aematrixlayoututils.h"

#include <stdio.h>

#define AEFIVESQUARE_COLOR_ON  0xff2020
#define AEFIVESQUARE_COLOR_OFF_EDIT 0xe0d0d0
#define AEFIVESQUARE_COLOR_OFF_ANIM 0xf0f0f0

/* Layout */
static void aeFiveSeven_InitData(aeAnimationData_t* data);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSeven_Arch_Register(void)
{
    aeArchitectureInfo_t    archInfo;

    archInfo.signature = ASIG_AR_FIVESEVEN;

    archInfo.subtypeFlags = 
        aeAnimSubtypeFlag_eFrameByFrame |
        aeAnimSubtypeFlag_ePixelList;
        /*  | aeAnimSubtypeFlag_eStrip; */

    /* LEDs either on or off */
    archInfo.bitsPerPixel = 1;

    /* Needs five bytes to hold 35 bits */
    archInfo.bytesPerFrame = 5; 

    /* Frame-by-Frame needs the same data as a frame */
    archInfo.bytesPerEntry[aeAnimSubtypeFlagBit_eFrameByFrame] = archInfo.bytesPerFrame;
    archInfo.ledsPerEntry[aeAnimSubtypeFlagBit_eFrameByFrame]  = 35;

    /* Pixel List needs one byte to store value of 0..63 */
    archInfo.bytesPerEntry[aeAnimSubtypeFlagBit_ePixelList]    = 1;
    archInfo.ledsPerEntry[aeAnimSubtypeFlagBit_ePixelList]     = 1;

    /* Strip needs one byte because row or column is only 5 or 7 LEDs */
    archInfo.bytesPerEntry[aeAnimSubtypeFlagBit_eStrip]        = 1;
    archInfo.ledsPerEntry[aeAnimSubtypeFlagBit_eStrip]         = 7;


    aeArchList_Arch_Add(&archInfo);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSeven_Layout_Register(void)
{
    aePlatform_t    platform;

    aeMatrix_Layout_GetPlatformDefaults(&platform);

    strcpy(platform.name, "FiveSeven");

    platform.layoutSignature         = ASIG_LO_FIVESEVEN;
    platform.archSignature           = ASIG_AR_FIVESEVEN;

    platform.layoutWidth             = 7;
    platform.layoutHeight            = 5;

    platform.initAnimData        = aeFiveSeven_InitData;

    platform.elementShape        = aeElementShape_eRectangle;

    aeArchList_Layout_Add(&platform);
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSeven_InitData(aeAnimationData_t* animData)
{
    AEASSERT(animData);

	/* Defaults are good for FrameByFrame and PixelList. */

    #if 1
    /* For Strip, we set-up eight columns */
	if (aeAnimation_GetSubtype(animData) == aeAnimSubtype_eStrip)
	{
		int addFrame, i;
		
		aeAnimation_SetWrapStyle(animData, aeAnimStyle_eWrapped);
		//aeAnimation_SetDirection(animData, aeAnimDirection_eHorizontal);

		if (aeAnimation_GetFrameCount(animData) < 8)
		{
			addFrame = 8 - aeAnimation_GetFrameCount(animData);
			for (i = 0; i < addFrame; i++)
				aeAnimation_AppendFrame(animData);
		}

        aeAnimation_SetNeedsResizeOnFrameCountChange(animData, TRUE);

        /* Clear the changed flag. It was set when the frames were added */
        aeAnimation_ResetChangedFlag(animData);
	}
    #endif
    
}





