/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aefivesquarepic.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aefivesquare.h"
#include "aearchlist.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aedebug.h"
#include <stdio.h>

#define AEMATRIX_WIDTH  5
#define AEMATRIX_HEIGHT 5


static void   aeFiveSquare_WriteAnimPicAssembly(aeAnimationData_t* data, char* line, int lineLen);
static aErr_t aeFiveSquare_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref);
static void   aeFiveSquare_GetPatternStr(unsigned long pattern, int row, char* str, int strlen);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSquare_CodeGen_Register(void)
{
    aeCodeGen_t codegen;

    memset(&codegen, 0, sizeof(codegen));

    codegen.codegenSignature = ASIG_FIVESQUARE_PICASM;
	codegen.archSignature    = ASIG_FIVESQUARE;
    codegen.codegenfunc      = aeFiveSquare_WriteCode;
    codegen.requiresUniqueNames = TRUE;

    #if 0
	char			extension[20]; /* Need #define */
    char			prefix[20];
    char			suffix[20];
    char            name[ANIM_NAME_LEN];
    char            desc[ANIM_DESC_LEN];
    #endif
    
    aeArchList_CodeGen_Add(&codegen);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aeFiveSquare_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref)
{
    aErr_t  err = AERR_NO_ERROR;
    char    line[500];
    char    index;
    int     variant;

    aeAnimationData_t* animData;

    animData = aeSequence_Anim_Get(sequence, 0);

    /*------------------------*/
    /* Code File Header       */
    /*------------------------*/
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeFileHeader);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);

    /*------------------------*/
    /* Dispatch Header        */
    /*------------------------*/
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);

    /*------------------------*/
    /* Dispatch Entries       */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write dispatch entries for active animations */
        if (aeSequence_Anim_IsActive(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Tell the animation we want to write the dispatch line */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);
            
            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);
    
            /* Get the line and write it to the file */
            aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
            fprintf(fileref, line);
        }

    }

    /*------------------------*/
    /* Dispatch Trailer       */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchTrailer);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);


    /*------------------------*/
    /* Code Intro             */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineIntro);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);



    /*------------------------*/
    /* Animation Entry Points */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines and only the first time they are encountered */
        if (aeSequence_Anim_IsFirstActiveOccurance(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);

            /* Tell the animation we want to write the routine header */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineHeader);
            aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
            fprintf(fileref, line);
        }
    }

    /*------------------------*/
    /* Table Intro             */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeTableIntro);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);



    /*------------------------*/
    /* Animation Routines (tables) */
    /*------------------------*/
    /* Loop over animations */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines with animation data */
        /* DDK 2009.06.06: Also, write inactive routines if data is used by an active routine */
        if (aeSequence_Anim_HasNeededAnimData(sequence, index) )
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Tell the animation we want to write the routine body and trailer */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);
            do
            {
                aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
                fprintf(fileref, line);
            } while (aeAnimation_IO_GetState(animData) != AnimIOState_eDone);
        }
    }

    /*------------------------*/
    /* Code File Trailer      */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeFileTrailer);
    aeFiveSquare_WriteAnimPicAssembly(animData, line, sizeof(line));
    fprintf(fileref, line);


    return err;
}





void aeFiveSquare_WriteAnimPicAssembly(aeAnimationData_t* animData, char* line, int lineLen)
{
    char	str[200];
    char	name[ANIM_NAME_LEN*2];
    char	varName[ANIM_NAME_LEN*2];
    char	dataName[ANIM_NAME_LEN*2];
    char    pattern[10];
    char    stylestr[40];
    unsigned   frameData;
    unsigned   val;
    AnimIOState_t ioState;
    aeAnimSubtype_t subtype;
    
    subtype = aeAnimation_GetSubtype(animData);
    ioState = aeAnimation_IO_GetState(animData);
    

    if (ioState == AnimIOState_eCodeFileHeader)
    {
        line[0] = 0;
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, ";===============================================\n");
        strcat(line, "; FiveSquare Sequence Data\n");
        strcat(line, ";\n");
        strcat(line, ";\n");
        strcat(line, "; David D. Kaufman\n");
        strcat(line, ";===============================================\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "#include \"FiveSquareLibrary.h\"\n");
        strcat(line, "\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);

    }
    else if (ioState == AnimIOState_eCodeDispatchHeader)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Dispatch\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "AnimationDispatch\n");
        strcat(line, "\t\t\tIdxLongJump AnimationDispatch, CURSEQ\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeDispatchEntry)
    {
        aeAnimation_GetNameUniqueVariant(animData, name, sizeof(name));
        sprintf(line, "\t\t\tgoto %s\n", name);

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);
    }
    else if (ioState == AnimIOState_eCodeDispatchTrailer)
    {
        line[0] = 0;
        strcat(line, "\t\t\tbsf		SEQFLAGS,FLAG_LIST_DONE\n");
        strcat(line, "\t\t\treturn\n");
        strcat(line, "\n");
        strcat(line, "\n");

        aeAnimation_IO_Count_Reset(animData);
        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineHeader);
    }
    else if (ioState == AnimIOState_eCodeRoutineIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Entry Points\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeRoutineHeader)
    {
        line[0] = 0;
        aeAnimation_GetNameUniqueVariant(animData, varName, sizeof(varName));
        aeAnimation_GetNameUnique(animData, dataName, sizeof(dataName));

        if (subtype == aeAnimSubtype_eFrameByFrame)
        {
            /*
            (tab x 3)FrameByFrameAnim diagonal, 4, 0x80, diagonalData
            */
            sprintf(str, "\t\t\t; %s (%d frames)\n", 
                    varName, 
                    aeAnimation_GetFrameCount(animData));
            strcat(line, str);
    
            sprintf(str, "\t\t\tFrameByFrameAnim %s, %d, 0x%2.2x, %sData\n", 
                    varName, 
                    aeAnimation_GetRepeat(animData), 
                    aeAnimation_GetSpeed(animData), 
                    dataName);
            strcat(line, str);
        }
        else if (subtype == aeAnimSubtype_ePixelList)
        {
            if (aeAnimation_GetWrapStyle(animData) == aeAnimStyle_eSinglePass)
                strcpy(stylestr, "PixelListSinglePass");
            else
                strcpy(stylestr, "PixelListWrapStyle");
            /*
            (tab x 3)PixelListAnim SnakeFillOne, 	2, 0x20,  1, SnakeFillData, PixelListSinglePass
            (tab x 3)PixelListAnim SnakeFillLong,	3, 0x10, 16, SnakeFillData, PixelListSinglePass
            */        
            sprintf(str, "\t\t\t; %s (%d pixels)\n", 
                    varName, 
                    aeAnimation_GetFrameCount(animData));
            strcat(line, str);

            sprintf(str, "\t\t\tPixelListAnim %s, %d, 0x%2.2x, %d, %sData, %s\n", 
                    varName, 
                    aeAnimation_GetRepeat(animData), 
                    aeAnimation_GetSpeed(animData),
                    aeAnimation_GetRunLength(animData), 
                    dataName, 
                    stylestr);
            strcat(line, str);
        }
        else if (subtype == aeAnimSubtype_eStrip)
        {
            /*
            (tab x 3)StripAnim StripTest, 4, 0x80, 0, StripTestData, StripSinglePass | StripHorizontal
            */
            if (aeAnimation_GetWrapStyle(animData) == aeAnimStyle_eSinglePass)
                strcpy(stylestr, "PixelListSinglePass");
            else
                strcpy(stylestr, "PixelListWrapStyle");

            if (aeAnimation_GetDirection(animData) == aeAnimDirection_eHorizontal)
                strcat(stylestr, "| StripHorizontal");
            else
                strcat(stylestr, "| StripVertical");

            sprintf(str, "\t\t\t; %s (%d rows)\n", 
                    varName, 
                    aeAnimation_GetFrameCount(animData));
            strcat(line, str);

            sprintf(str, "\t\t\tStripAnim %s, %d, 0x%2.2x, 0, %sData, %s\n", 
                    varName, 
                    aeAnimation_GetRepeat(animData), 
                    aeAnimation_GetSpeed(animData), 
                    dataName, 
                    stylestr);
            strcat(line, str);
        }
        else
        {
            AEASSERTED("aeFiveSquare_WriteAnimPicAssembly: unknown subtype");
        }

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);
        aeAnimation_IO_Count_Reset(animData);

    }
    else if (ioState == AnimIOState_eCodeTableIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Tables\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);

    }
    else if (ioState == AnimIOState_eCodeRoutineBody)
    {
        line[0] = 0;
        frameData = aeAnimation_GetFrameData(animData, aeAnimation_IO_Count_Get(animData));
        
        if (aeAnimation_IO_Count_Get(animData) == 0)
        {
            strcat(line, "\n");
            sprintf(str, "%sData\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
            sprintf(str, "\t\t\tIdxLongJump %sData, DATAINDEX\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
        }

        if (subtype == aeAnimSubtype_eFrameByFrame)
        {
            val = ((frameData >>  0) & 0xff);
            aeFiveSquare_GetPatternStr(frameData, 0, pattern, sizeof(pattern));
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
            strcat(line, str);
    
            val = ((frameData >>  8) & 0xff);
            aeFiveSquare_GetPatternStr(frameData, 1, pattern, sizeof(pattern));
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
            strcat(line, str);
    
            val = ((frameData >> 16) & 0xff);
            aeFiveSquare_GetPatternStr(frameData, 2, pattern, sizeof(pattern));
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
            strcat(line, str);
    
            val = ((frameData >> 24) & 0xff);
            aeFiveSquare_GetPatternStr(frameData, 3, pattern, sizeof(pattern));
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
            strcat(line, str);
    
            aeFiveSquare_GetPatternStr(frameData, 4, pattern, sizeof(pattern));
            sprintf(str, "\t\t\t\t\t\t\t\t; %s\n", pattern);
            strcat(line, str);
    
            strcat(line, "\n");
        }
        else if (subtype == aeAnimSubtype_ePixelList)
        {
            val = (frameData & 0xff);
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t;\n", val);
            strcat(line, str);
        }
        else if (subtype == aeAnimSubtype_eStrip)
        {
            val = (frameData & 0xff);
            aeFiveSquare_GetPatternStr(frameData, 0, pattern, sizeof(pattern));
            sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
            strcat(line, str);
        }

        aeAnimation_IO_Count_Incr(animData);

        if (aeAnimation_IO_Count_Get(animData) == aeAnimation_GetFrameCount(animData))
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineTrailer);

    }
    else if (ioState == AnimIOState_eCodeRoutineTrailer)
    {
        line[0] = 0;

        sprintf(str, "\t\t\tbsf\t\tSEQFLAGS, FLAG_DATA_DONE\n");
        strcat(line, str);

        sprintf(str, "\t\t\treturn\n");
        strcat(line, str);

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else if (ioState == AnimIOState_eCodeFileTrailer)  
    {
        line[0] = 0;
        strcat(line, "\n");
		strcat(line, "\t\t\tend\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else
    {
        AEASSERTED("aeFiveSquare_WriteAnimPicAssembly: Unknown iostate");
    }
}

/*
;==========	RowTest
			FrameByFrameAnim RowTest, 0x04, 0x80, RowTestData

RowTestData
			IdxJump	RowTestData, DATAINDEX
			retlw	0x1f		; 0
			retlw	0x00
			retlw	0x00
			retlw	0x00
			retlw	0xe0		; 1
			retlw	0x03
			retlw	0x00
			retlw	0x00
			retlw	0x00		; 2
			retlw	0x7c
			retlw	0x00
			retlw	0x00
			retlw	0x00		; 3
			retlw	0x80
			retlw	0x0f
			retlw	0x00
			retlw	0x00		; 4
			retlw	0x00
			retlw	0xf0
			retlw	0x01
			bsf		SEQFLAGS, FLAG_DATA_DONE
			return

*/


void aeFiveSquare_GetPatternStr(unsigned long pattern, int row, char* str, int strlen)
{
    int patIndex;
    int strIndex;
    int setBit;
    int i;

    if (str != NULL && strlen > 5)
    {
        strIndex = 4;
        patIndex = row * 5;

        for (i = 0; i < 5; i++)
        {
            setBit = (pattern & (1 << patIndex)) != 0;
    
            str[strIndex] = setBit ? '@' : '.';

            strIndex--;
            patIndex++;
        }
        
        str[5] = 0;
    }
}


