/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aefivesquare.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aefivesquarepumpkin.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aearchlist.h"

#include <stdio.h>

#define AEFIVESQUARE_COLOR_BG       0x000000
#define AEFIVESQUARE_COLOR_ON       0xff2020
#define AEFIVESQUARE_COLOR_OFF_EDIT 0xe0d0d0
#define AEFIVESQUARE_COLOR_OFF_ANIM 0xfcffff
#define AEFIVESQUARE_COLOR_OFF_GIF  0x202020

#define AEMATRIX_WIDTH  5
#define AEMATRIX_HEIGHT 5


/* Layout */
static void         aeFiveSquarePumpkin_Layout_Create(aeLayout_t* layout);
static void         aeFiveSquarePumpkin_Layout_CreateEdit(aeLayout_t* layout, aeAnimationData_t* animData);

static aeColor_t    aeFiveSquarePumpkin_Data_GetElementColor(aeAnimationData_t* animData, int elementIndex);
static aeColor_t    aeFiveSquarePumpkin_Animation_GetElementColor(aeAnimationData_t* animData, int elementIndex);
static void         aeFiveSquarePumpkin_Data_AdvanceElementColor(aeAnimationData_t* animData, int elementIndex);
static int          aeFiveSquarePumpkin_Animation_GetElementColorList(aeAnimationData_t* animData, aeColor_t* colorList, int colorListSize);



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSquarePumpkin_Layout_Register(void)
{
    aePlatform_t    platform;

    memset(&platform, 0, sizeof(platform));

    strcpy(platform.name, "Pumpkin");

    platform.layoutSignature         = ASIG_LO_FIVESQUAREPUMPKIN;
    platform.archSignature           = ASIG_AR_FIVESQUARE;

    platform.layoutWidth             = 5;
    platform.layoutHeight            = 5;

    platform.elementShape            = aeElementShape_eUnknown;

    platform.animGetElementColor     = aeFiveSquarePumpkin_Animation_GetElementColor;
    platform.animGetElementColorList = aeFiveSquarePumpkin_Animation_GetElementColorList;
    platform.dataAdvanceElementColor = aeFiveSquarePumpkin_Data_AdvanceElementColor;
    platform.dataGetElementColor     = aeFiveSquarePumpkin_Data_GetElementColor;
    platform.layoutCreateAnim        = aeFiveSquarePumpkin_Layout_Create;
    platform.layoutCreateEdit        = aeFiveSquarePumpkin_Layout_CreateEdit;

    platform.initAnimData            = NULL;
    platform.initAnimDataRead        = NULL;

    /* Recommended or meaningful subtype flags */
    platform.subtypeFlags = 
        aeAnimSubtypeFlag_eFrameByFrame |
        aeAnimSubtypeFlag_ePixelList;


    aeArchList_Layout_Add(&platform);
}



/* Layout */

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
typedef struct
{
    int x;
    int y;

} aePoint_t;

static void aeLayoutUtils_ComputeLayout(aeLayout_t* layout, int elementCount, aePoint_t* locations, int ledSize, int ledSpace)
{
	aeRect_t	bounds;
	int			elementIndex;
    int         maxX, maxY;
	aeLayoutElement_t* le;

    /* Find the bounds */
    maxX = 0;
    maxY = 0;
    for (elementIndex = 0; elementIndex < 25; elementIndex++)
    {
        if (locations[elementIndex].x > maxX)
            maxX = locations[elementIndex].x;

        if (locations[elementIndex].y > maxY)
            maxY = locations[elementIndex].y;

    }

	bounds.left   = 0;
	bounds.top    = 0;
	bounds.right  = (maxX + 3) * (ledSpace + ledSize);
	bounds.bottom = (maxY + 3) * (ledSpace + ledSize);

	/* Set bounds for layout */
	aeLayout_SetLayoutExtent(layout, bounds);

    /* Set element count */
	aeLayout_SetElementCount(layout, elementCount);

    /* Place each item */
	for (elementIndex = 0; elementIndex < elementCount; elementIndex++)
	{
        le = aeLayout_GetElement(layout,elementIndex);

        /* locX and locY contain the center of the LED */
        le->locX = ledSpace + ledSize/2 + locations[elementIndex].x * (ledSpace + ledSize);
        le->locY = ledSpace + ledSize/2 + locations[elementIndex].y * (ledSpace + ledSize);

        le->elementType = aeElementType_e10mmRound;

        le->ledCount = 1;
        le->colors[0] = AEFIVESQUARE_COLOR_ON;
	}


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static void aeFiveSquarePumpkin_ComputeLayout(aeLayout_t* layout, int rowsOrColumns, aeAnimDirection_t direction)
{
    #define PUMPKIN_ELEMENT_COUNT 25

    int			ledSpace;
	int			ledSize;

    aePoint_t   locations[PUMPKIN_ELEMENT_COUNT] =
    {
        { 1, 1 },
        { 5, 1 },
        { 1, 5 },
        { 3, 3 },
        { 5, 5 },

        { 16, 1 },
        { 20, 1 },
        { 16, 5 },
        { 18, 3 },
        { 20, 5 },

        { 10, 10 },
        { 8,  12 },
        { 12, 12 },

        { 3,  18 },
        { 6,  18 },
        { 9,  18 },
        { 12, 18 },
        { 15, 18 },
        { 18, 18 },

        { 3,  20 },
        { 6,  20 },
        { 9,  20 },
        { 12, 20 },
        { 15, 20 },
        { 18, 20 }

    };

    ledSize  = 5; /* millimeters */
    ledSpace = 2;

    aeLayoutUtils_ComputeLayout(layout, PUMPKIN_ELEMENT_COUNT, locations, ledSize, ledSpace);


}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSquarePumpkin_Layout_Create(aeLayout_t* layout)
{
	aeFiveSquarePumpkin_ComputeLayout(layout, AEMATRIX_HEIGHT, aeAnimDirection_eVertical);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSquarePumpkin_Layout_CreateEdit(aeLayout_t* layout, aeAnimationData_t* animData)
{
	aeFiveSquarePumpkin_ComputeLayout(layout, AEMATRIX_HEIGHT, aeAnimDirection_eVertical);
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aeFiveSquarePumpkin_Data_AdvanceElementColor(aeAnimationData_t* animData, int elementIndex)
{
    int          ledIndex;
	aeLedValue_t value;

    ledIndex = elementIndex;

    value = aeAnimation_Edit_GetLedValue(animData, ledIndex);

    if (value == 0)
        value = 1;
    else
        value = 0;

    aeAnimation_Edit_UpdateLedValue(animData, ledIndex, value);
}

/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeColor_t aeFiveSquarePumpkin_Data_GetElementColor(aeAnimationData_t* animData, int elementIndex)
{
	aeLedValue_t value;
 
	value = aeAnimation_Edit_GetLedValue(animData, elementIndex);

	return (value) ? AEFIVESQUARE_COLOR_ON : AEFIVESQUARE_COLOR_OFF_EDIT;
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aeColor_t aeFiveSquarePumpkin_Animation_GetElementColor(aeAnimationData_t* animData, int elementIndex)
{
	aeLedValue_t	value;

	value = aeAnimation_Anim_GetLedValue(animData, elementIndex);

	return (value) ? AEFIVESQUARE_COLOR_ON : AEFIVESQUARE_COLOR_OFF_ANIM;
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
int aeFiveSquarePumpkin_Animation_GetElementColorList(aeAnimationData_t* animData, aeColor_t* colorList, int colorListSize)
{
    int count = 0;

    colorList[count++] = AEFIVESQUARE_COLOR_BG;
    colorList[count++] = AEFIVESQUARE_COLOR_OFF_GIF;
    colorList[count++] = AEFIVESQUARE_COLOR_ON;

    return count;
}

