/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aefileio.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "astd.h"


/* Layout */
#define AELAYOUT_INCR_ROW_THEN_COL 0
#define AELAYOUT_INCR_COL_THEN_ROW 1

#define AELAYOUT_ROW_INCR_LEFT_TO_RIGHT 0
#define AELAYOUT_ROW_INCR_RIGHT_TO_LEFT 1

#define AELAYOUT_COL_INCR_TOP_TO_BOTTOM 0
#define AELAYOUT_COL_INCR_BOTTOM_TO_TOP 1


/*-----------------------------------------------

   layout
   visibleColCount
   visibleRowCount
   totalColCount
   totalRowCount
   rowOrColMajor
   ledSize
   ledSpace
   elementType
   onColor


       <---------------


   (x)   (x)   (x)   (x)   (x)
                                  
   (x)   (x)   (x)   (x)   (x)     |
                                   |
   (x)   (x)   (x)   (x)   (x)     |
                                   |
   (x)   (x)   (x)   (x)   (x)     |
                                   |
   (x)   (x)   (x)   (x)   (x)     V

   (x)   (x)   (x)   (x)   (x)
   
--------------------------------------------------*/


void aeLayout_ComputeMatrix(aeLayout_t* layout, 
                            int visibleColCount, int visibleRowCount, 
                            int totalColCount, int totalRowCount,
                            char rowOrColMajor,
                            int ledSize, int ledSpace, 
                            aeElementType_t elementType, aeColor_t onColor)
{
    int			row;
    int			col;
    aeRect_t	bounds;
    int			elementIndex;
    int			elementCount;
    int			vertDim;
    int			horzDim;

    aeLayoutElement_t* le;

    /* Dimensions */
    vertDim = ledSpace + visibleRowCount * (ledSpace + ledSize);
    horzDim = ledSpace + visibleColCount * (ledSpace + ledSize);

    bounds.left   = 0;
    bounds.top    = 0;
    bounds.right  = horzDim;
    bounds.bottom = vertDim;

    /* Set bounds for layout */
    layout->layoutExtent = bounds;

    /* Total elements */
    elementCount = visibleRowCount * visibleColCount;

    /* Calculate spacing around each LED */
    /*                1                   2                   3                   4                   5                */
    /* |<-space->[---LED---]<-space->[---LED---]<-space->[---LED---]<-space->[---LED---]<-space->[---LED---]<-space->| */
    /*      1                   2                   3                   4                   5                   6      */
    /* ledSpace = ((bounds.right - bounds.left) - (rowCount * ledSize)) / 6; */

    layout->elementCount = elementCount;

    /* Increasing ElementIndex identifies LEDs vertically */
    for (elementIndex = 0; elementIndex < elementCount; elementIndex++)
    {
        row = elementIndex % POV8_HEIGHT;
        col = elementIndex / POV8_HEIGHT;

        le = &layout->element[elementIndex];

        /* locX and locY contain the center of the LED */
        le->locX = ledSpace + ledSize/2 + col * (ledSize + ledSpace);
        le->locY = ledSpace + ledSize/2 + row * (ledSize + ledSpace);

        /* Set the element type */
        le->elementType = aeElementType_e10mmRound;

        /* Set the element colors */
        le->ledCount = 1;
        le->colors[0] = onColor;
    }
}

