/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aepov12.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aepov12.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aearchlist.h"
#include "aematrixlayoututils.h"

#include <stdio.h>

#define AEFIVESQUARE_COLOR_ON  0xff2020
#define AEFIVESQUARE_COLOR_OFF_EDIT 0xe0d0d0
#define AEFIVESQUARE_COLOR_OFF_ANIM 0xf0f0f0

#define AEPOV8_COLOR_ON 0xffff00
#define AEPOV8_COLOR_OFF  0
#define AEPOV8_COLOR_OFF_ANIM 0

/* Layout */
static void         aePOV12_InitData(aeAnimationData_t* data);
static void         aePOV12_InitDataRead(aeAnimationData_t* animData);
static void         aePOV12_Layout_CreateAnim(aeLayout_t* layout);
static void         aePOV12_Layout_CreateEdit(aeLayout_t* layout, aeAnimationData_t* animData);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aePOV12_Layout_Register(void)
{
    aePlatform_t    platform;

    #if 0

    aeMatrix_Layout_GetPlatformDefaults(&platform);

    strcpy(platform.name, "POV12");

    platform.layoutSignature         = ASIG_LO_POV12;
    platform.archSignature           = ASIG_AR_SLAK;

    platform.layoutWidth             = 1;
    platform.layoutHeight            = 12;

    platform.layoutCreateAnim    = aePOV12_Layout_CreateAnim;
    platform.layoutCreateEdit    = aePOV12_Layout_CreateEdit;
    platform.initAnimData        = aePOV12_InitData;
    platform.initAnimDataRead    = aePOV12_InitDataRead;

	platform.dataAdvanceElementColor    = aePOV12_Data_AdvanceElementColor;
	platform.dataGetElementColor		= aePOV12_Data_GetElementColor;
	platform.animGetElementColor		= aePOV12_Animation_GetElementColor;

    /* Recommended or meaningful subtype flags */
    platform.subtypeFlags =      aeAnimSubtypeFlag_eFrameByFrame;

    #endif
    
    aeArchList_Layout_Add(&platform);
}



/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
static void aePOV12_p_InitData_Common(aeAnimationData_t* animData)
{
    aeAnimation_SetEditSubtype(animData, aeAnimSubtype_eStrip);
    aeAnimation_SetNeedsResizeOnFrameCountChange(animData, TRUE);

}

void aePOV12_InitData(aeAnimationData_t* animData)
{
    int addFrame, i;

    AEASSERT(animData);

    if (aeAnimation_GetFrameCount(animData) < 20)
    {
        addFrame = 20 - aeAnimation_GetFrameCount(animData);
        for (i = 0; i < addFrame; i++)
            aeAnimation_AppendFrame(animData);
    }

    /* Clear the changed flag. It was set when the frames were added */
    aeAnimation_ResetChangedFlag(animData);

    aePOV12_p_InitData_Common(animData);
}

void aePOV12_InitDataRead(aeAnimationData_t* animData)
{
    aePOV12_p_InitData_Common(animData);
}



void aePOV12_Layout_CreateAnim(aeLayout_t* layout)
{
    /* No animation layout */
    aeLayout_SetElementCount(layout, 12);
    aeMatrix_Layout_ComputeLayout(layout, 0, 0, aeAnimDirection_eHorizontal);

}

void aePOV12_Layout_CreateEdit(aeLayout_t* layout, aeAnimationData_t* animData)
{
    int columns;

    columns= aeAnimation_GetFrameCount(animData);

    aeMatrix_Layout_ComputeLayout(layout, columns, 0, aeAnimDirection_eHorizontal);
}



void aePOV12_Data_AdvanceElementColor(aeAnimationData_t* animData, int elementIndex)
{
    int value;

    value = aeAnimation_Edit_GetLedValue(animData, elementIndex);

    value = !value;

    aeAnimation_Edit_UpdateLedValue(animData, elementIndex, value);
}


aeColor_t aePOV12_Data_GetElementColor(aeAnimationData_t* animData, int elementIndex)
{
	aeLedValue_t value;
	aeColor_t	colorOn;
    int         ledIndex;

	colorOn = AEPOV8_COLOR_ON;

    ledIndex = elementIndex;
	value = aeAnimation_Edit_GetLedValue(animData, ledIndex);

	return (value) ? colorOn : AEPOV8_COLOR_OFF;
}


aeColor_t aePOV12_Animation_GetElementColor(aeAnimationData_t* animData, int elementIndex)
{
	aeLedValue_t	value;

    value = aeAnimation_Anim_GetLedValue(animData, elementIndex);

	return (value) ? AEPOV8_COLOR_ON : AEPOV8_COLOR_OFF_ANIM;
}



