/*-----------------------------------------------------------------------------
 *
 *  ANIMATOR
 *  LED Animation Sequence Editor
 *
 *  Copyright David D. Kaufman, 2009
 *  
 *  aepov8slakpic.c
 *
 *
 *
 *
 *
 *------------------------------------------------------------------------------*/

#include "aepov8slakpic.h"
#include "aearchlist.h"
#include "aedebug.h"
#include "asig.h"
#include "aeanimation.h"
#include "aedebug.h"
#include <stdio.h>

#define AEPOV8_HEIGHT 8


static void   aePOV8_SLAK_WriteAnimPicAssembly(aeAnimationData_t* data, aeCodeGen_t* codegen, char* line, int lineLen);
static aErr_t aePOV8_SLAK_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref);
static void   aePOV8_GetPatternStr(unsigned long pattern, char* str, int strlen);


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
void aePOV8_SLAK_CodeGen_Register(void)
{
    aeCodeGen_t codegen;

    memset(&codegen, 0, sizeof(aeCodeGen_t));

    codegen.codegenSignature = ASIG_POV8_SLAK_ASM;
	codegen.archSignature    = ASIG_SLAK;
    codegen.codegenfunc      = aePOV8_SLAK_WriteCode;

    codegen.requiresUniqueNames = TRUE;

    #if 0
	char			extension[20]; /* Need #define */
    char			prefix[20];
    char			suffix[20];
    char            name[ANIM_NAME_LEN];
    char            desc[ANIM_DESC_LEN];
    #endif
    
    aeArchList_CodeGen_Add(&codegen);
}


/*----------------------------------------------------------------------
Summary
    

Desc
    
----------------------------------------------------------------------*/
aErr_t aePOV8_SLAK_WriteCode(aeSequence_t* sequence, aeCodeGen_t* codegen, FILE* fileref)
{
    aErr_t  err = AERR_NO_ERROR;
    char    line[500];
    char    index;
    int     variant;

    aeAnimationData_t* animData;

    animData = aeSequence_Anim_Get(sequence, 0);

    /*------------------------*/
    /* Code File Header       */
    /*------------------------*/
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeFileHeader);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);

    /*------------------------*/
    /* Dispatch Header        */
    /*------------------------*/
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);

    /*------------------------*/
    /* Dispatch Entries       */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write dispatch entries for active animations */
        if (aeSequence_Anim_IsActive(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Tell the animation we want to write the dispatch line */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);
            
            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);
    
            /* Get the line and write it to the file */
            aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
            fprintf(fileref, line);
        }

    }

    /*------------------------*/
    /* Dispatch Trailer       */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchTrailer);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);


    /*------------------------*/
    /* Code Intro             */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineIntro);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);



    /*------------------------*/
    /* Animation Entry Points */
    /*------------------------*/
    /* Loop over animation */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines and only the first time they are encountered */
        if (aeSequence_Anim_IsFirstActiveOccurance(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Specify which variant we want */
            variant = aeSequence_Anim_GetVariant(sequence, index);
            aeAnimation_SetCurrentVariant(animData, variant);

            /* Tell the animation we want to write the routine header */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineHeader);
            aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
            fprintf(fileref, line);
        }
    }

    /*------------------------*/
    /* Table Intro             */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeTableIntro);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);



    /*------------------------*/
    /* Animation Routines (tables) */
    /*------------------------*/
    /* Loop over animations */
    for (index = 0; index < aeSequence_GetCount(sequence); index++)
    {
        /* Only write active routines with animation data */
        if (aeSequence_Anim_HasNeededAnimData(sequence, index))
        {
            animData = aeSequence_Anim_Get(sequence, index);

            /* Tell the animation we want to write the routine body and trailer */
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);
            do
            {
                aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
                fprintf(fileref, line);
            } while (aeAnimation_IO_GetState(animData) != AnimIOState_eDone);
        }
    }

    /*------------------------*/
    /* Code File Trailer      */
    /*------------------------*/
    animData = aeSequence_Anim_Get(sequence, 0);
    aeAnimation_IO_SetState(animData, AnimIOState_eCodeFileTrailer);
    aePOV8_SLAK_WriteAnimPicAssembly(animData, codegen, line, sizeof(line));
    fprintf(fileref, line);


    return err;
}





void aePOV8_SLAK_WriteAnimPicAssembly(aeAnimationData_t* animData, aeCodeGen_t* codegen, char* line, int lineLen)
{
    char	str[200];
    char	name[ANIM_NAME_LEN];
    char    pattern[20];
    unsigned   frameData;
    unsigned   val;
    AnimIOState_t ioState;
    aeAnimSubtype_t subtype;
    
    subtype = aeAnimation_GetSubtype(animData);
    ioState = aeAnimation_IO_GetState(animData);
    

    if (ioState == AnimIOState_eCodeFileHeader)
    {
        line[0] = 0;
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, "\n");
        strcat(line, ";===============================================\n");
        strcat(line, "; POV8 Sequence Data\n");
        strcat(line, ";\n");
        strcat(line, ";\n");
        strcat(line, "; David D. Kaufman\n");
        strcat(line, ";===============================================\n");
        strcat(line, "\n");
        strcat(line, "#define LED_COUNT 8\n");
        if (codegen->speedReduction != 0)
        {
            sprintf(str, "#define SPEED_REDUCTION %d\n", codegen->speedReduction);
            strcat(line, str);
        }
        strcat(line, "#include \"..\\ComponentLibrary\\LEDDirectDrive_Animator.asm\"\n");
        strcat(line, "\n");
        strcat(line, "\t\t\torg 0x100\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);

    }
    else if (ioState == AnimIOState_eCodeDispatchHeader)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Dispatch\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "AnimationDispatch\n");
        strcat(line, "\t\t\tIdxJump AnimationDispatch, CURSEQ\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeDispatchEntry)
    {
        aeAnimation_GetNameVariant(animData, name, sizeof(name));
        sprintf(line, "\t\t\tgoto %s\n", name);

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchHeader);
    }
    else if (ioState == AnimIOState_eCodeDispatchTrailer)
    {
        line[0] = 0;
        strcat(line, "\t\t\tbsf		SEQFLAGS,FLAG_LIST_DONE\n");
        strcat(line, "\t\t\treturn\n");
        strcat(line, "\n");
        strcat(line, "\n");

        aeAnimation_IO_Count_Reset(animData);
        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineHeader);
    }
    else if (ioState == AnimIOState_eCodeRoutineIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Entry Points\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeDispatchEntry);

    }
    else if (ioState == AnimIOState_eCodeRoutineHeader)
    {
        line[0] = 0;
        aeAnimation_GetNameVariant(animData, name, sizeof(name));

            /*
            (tab x 3)FrameByFrameAnim Thirds, DefaultBehavior, 0x04, 0x80, 0x80, ThirdsData
            */
            sprintf(str, "\t\t\t; %s (%d frames)\n", aeAnimation_GetNameUnique(animData, name, sizeof(name)), aeAnimation_GetFrameCount(animData));
            strcat(line, str);
    
            sprintf(str, "\t\t\tFrameByFrameAnim %s, OneBytePatterns, %d, NoTransition, 0x%2.2x, %sData\n", name, aeAnimation_GetRepeat(animData), aeAnimation_GetSpeed(animData), aeAnimation_GetName(animData));
            strcat(line, str);

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);
        aeAnimation_IO_Count_Reset(animData);

    }
    else if (ioState == AnimIOState_eCodeTableIntro)
    {
        line[0] = 0;
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\t\t;	Animation Tables\n");
        strcat(line, "\t\t;\n");
        strcat(line, "\t\t;---------------------------------------------\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineBody);

    }
    else if (ioState == AnimIOState_eCodeRoutineBody)
    {
        line[0] = 0;
        frameData = aeAnimation_GetFrameData(animData, aeAnimation_IO_Count_Get(animData));
        
        if (aeAnimation_IO_Count_Get(animData) == 0)
        {
            strcat(line, "\n");
            sprintf(str, "%sData\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
            sprintf(str, "\t\t\tIdxLongJump %sData, DATAINDEX\n",aeAnimation_GetNameUnique(animData, name, sizeof(name)));
            strcat(line, str);
        }

        val = (frameData & 0xff);
        aePOV8_GetPatternStr(frameData, pattern, sizeof(pattern));
        sprintf(str, "\t\t\tretlw\t0x%2.2x\t\t; %s\n", val, pattern);
        strcat(line, str);

        aeAnimation_IO_Count_Incr(animData);

        if (aeAnimation_IO_Count_Get(animData) == aeAnimation_GetFrameCount(animData))
            aeAnimation_IO_SetState(animData, AnimIOState_eCodeRoutineTrailer);

    }
    else if (ioState == AnimIOState_eCodeRoutineTrailer)
    {
        line[0] = 0;

        sprintf(str, "\t\t\tbsf\t\tSEQFLAGS, FLAG_DATA_DONE\n");
        strcat(line, str);

        sprintf(str, "\t\t\treturn\n");
        strcat(line, str);

        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else if (ioState == AnimIOState_eCodeFileTrailer)  
    {
        line[0] = 0;
        strcat(line, "\n");
		strcat(line, "\t\t\tend\n");
        strcat(line, "\n");

        aeAnimation_IO_SetState(animData, AnimIOState_eDone);
    }
    else
    {
        AEASSERTED("aePOV8_SLAK_WriteAnimPicAssembly: Unknown iostate");
    }
}


void aePOV8_GetPatternStr(unsigned long pattern, char* str, int strlen)
{
    int patIndex;
    int strIndex;
    int setBit;
    int i;

    if (str != NULL && strlen > 8)
    {
        strIndex = 7;
        patIndex = 0;

        for (i = 0; i < 8; i++)
        {
            setBit = (pattern & (1 << patIndex)) != 0;
    
            str[strIndex] = setBit ? '@' : '.';

            strIndex--;
            patIndex++;
        }
        
        str[8] = 0;
    }
}


